classdef EllipsoidMap < SurfaceMap & handle
    %ELLIPSOIDMAP Summary of this class goes here
    %   Detailed explanation goes here
    %
    % Version:  0.7.0 (01.11.2014) Duplicate map
    %           0.3.0 (02.09.2013) Use Extra Stack for the extra channels
    %                              Property dialog
    %           0.2.3 (27.06.2013) New Class approach
    %                              Derived from 
    %                                   sm_ellipsoidFit 
    %                                   sm_xyzT
    %                                   sm_ellipsoidPoints
    properties
        
        % Fitting properties
        Ellipsoid

        FittingMask = 1;            % Mask use for fitting
        ReferenceMaskFrame = 1;

        % Ellipsoid properties
        EllipsoidCoords             % List of the coordinates of each points in Coords
                                    % {t}[Index]         

        % GeoTable related properties
        GeoCoords                       % Geographical coordinates of each point
                                        %   {t}[Lat,Long,h,R]
        GeoCoordsTableSize = [90,180];  % Desired length for [DeltaLat,DeltaLong]
                                        %   if [], then automatic estimation
        DeltaLat                    % Vector with the diferent latitudes to be analyzed
        DeltaLong                   % Vector with the different longitudes to be analyzed
        GeoCoordsTable              % {t}[DeltaLat samples,DeltaLong samples,2]
                                    % where the third dimension contains 
                                    % [mean_h, mean_R]
        GeoCoordsTableIndex         % {t}[DeltaLat samples,DeltaLong samples,1]
                                    % where the third dimension contains 
                                    % the index to the average Coord position
        LinkTable                   % nx1 table with [GeoCoordsTable Index]

        % Projection properties        
        MapProj
                                    
        % Unfolding properties
        UnfoldCoords                % Map 2D coordinates 
                                    %   {t}[x,y]
        UseHeights = [false,false]; % Use of Heights in the unfolding
        HeightFactor = 1            % Factor to multiply the heights
        UseDefR = 0;                % Use a defined value of R in the ellipsoid
        ROption = 1                   % R to be added to h
                                    % 1: Maximum
                                    % 2: Local
        RefShift = [0,0];           % Reference shift in GeoCoordsTable (latitude,longitude)
        
        % Output map properties
        OutputTableIndex            % GeoCoordsTable to XY Map index
                                    %   {t}[Index]
        OutputSize = [500,500];     % Output Image Size
        AutoOutputSize = true;      % Automatic estimation of OutputSize
        BorderProj = 2;
        InterpolationMethod = 3;    % Method to interpolate
        InterpolationMask 
        
        % Grid properties
        GridBorderColor = 255;
        LatLines                    % Lax2 element vector with the list of
                                        % (latitudes,color) of the lines
        LongLines                   % Lox2 element vector with the list of
                                        % (longitudes,color) of the lines
        UnfoldLats                  % Index to GeoTable cols (LatLines x t)
        UnfoldLongs                 % Index to GeoTable rows (LongLines x t)

    end
    
    properties (Access = private)
        

    end;           
    
    properties (Dependent = true, Access = private)
        
%         MapName
        InterpolationName
        UseAzimuthal       % Internal use
    end;        

    
    
    methods

        % =================================================================
        % Constructor
        % -----------------------------------------------------------------
        function obj = EllipsoidMap(VolObj5D,IsCopy,varargin)
            if nargin < 2
                IsCopy = false;
            end
            obj = obj@SurfaceMap(VolObj5D,varargin{:});

            % Default parameters
            obj.InitializeProperties;
            
            % Create GUI
            obj.CreateGUI;
            
            if ~IsCopy
                % Properties Dialog
%                 EllipsoidProperties([],[],obj,true);
                EllipsoidProperties(obj,true);
            end            
        end;        
        
        % =================================================================
        % Destructor
        % -----------------------------------------------------------------
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------        
            if ~isempty(obj.ParentImage)
                obj.ParentImage.RemoveMap(obj);
            end;
        end;           
        
        % =================================================================
        % Dependent properties
        % -----------------------------------------------------------------
        function S = get.InterpolationName(obj)
            L = obj.InterpolationNameList;
            if obj.InterpolationMethod > 0 && obj.InterpolationMethod <= length(L)
                S = lower(L{obj.InterpolationMethod});
            else
                S = lower(L{1});
            end;
        end;
        function set.InterpolationName(obj,Value)
            L = obj.InterpolationNameList;
            if ischar(Value)
                obj.InterpolationMethod = find(strcmpi(Value,L),1);
            elseif Value > 0 && Value <= length(L)
                obj.InterpolationMethod = Value;
            else
                obj.InterpolationMethod = 1;
            end
        end
        
        
        % =================================================================
        function InitializeProperties(obj)
            % Initialize properties
            % Version: 0.7.0 (01.10.2014)
            % -------------------------------------------------------------
            obj.Ellipsoid = EllipsoidReference;
            obj.MapProj = EllipsoidProjection;
            obj.LatLines = obj.GetDefaultGrid(1);
            obj.LongLines = obj.GetDefaultGrid(2);
            
        end
        
        % =================================================================
        
        function UpdateMap(obj,Status)
            % Call the update routines
            obj.EnableEverything(false);
            
            if ~isempty(obj.DialogListener)
                delete(obj.DialogListener);
            end;
            if obj.NextUpdate > 0
                Status = obj.NextUpdate;
                obj.NextUpdate = 0;
            end;
            if Status > 0
                try
                    % Save current properties in a temporary file
                    k = 1;
                    PropertiesFilename = fullfile(obj.ParentImage.ProjectFolder,'Last.emp');
                    while exist(PropertiesFilename,'file')
                        k = k + 1;
                        PropertiesFilename = fullfile(obj.ParentImage.ProjectFolder,['Last',num2str(k),'.emp']);
                    end
                    obj.SaveProperties(PropertiesFilename);
                    PropFileSaved = true;
                catch
                    h = msgbox({'The map property file could not be saved...'}...
                        ,'Error','error','modal');
                    PropFileSaved = false;
                    uiwait(h);
                end
                try 
                    % Analyze Status to determine the updates
                    obj.LogHistory.NewTask('Creating Ellipsoid Map');
                    if Status < 2
                        Task2 = obj.LogHistory.NewTask(' - FITTING:');
                        obj.UpdateFitting;
                        obj.LogHistory.FinishTask(Task2);
                    end;
                    if Status < 3
                        Task2 = obj.LogHistory.NewTask(' - GEOGRAPHICAL TABLE:');
                        obj.UpdateGeoCoordsTable;
                        obj.LogHistory.FinishTask(Task2);
                    end;
                    if Status < 4
                        Task2 = obj.LogHistory.NewTask(' - UNFOLDING:');
                        obj.UpdateUnfolding;
                        obj.LogHistory.FinishTask(Task2);
                    end;
                    if Status < 5
                        Task2 = obj.LogHistory.NewTask(' - MAP IMAGE:');
                        obj.UpdateMapImage;
                        obj.LogHistory.FinishTask(Task2);
                    end;
                    obj.LogHistory.FinishAll;
                catch
                    obj.NextUpdate = 1;
                    obj.LogHistory.FinishAll;
                    h = msgbox({'There was a problem during the map creation...'...
                        ;'Please, check the properties and apply them again in order to update the map'}...
                        ,'Error','error','modal');
                    uiwait(h);
                end
                if PropFileSaved
                    try
                        % Move the file to the final destination
                        if ~exist(obj.Folder,'dir')
                            mkdir(obj.Folder);
                        end
                        NewFilename = fullfile(obj.Folder,[obj.Name,'.emp']);
                        %                 k = 1;
                        %                 while exist(NewFilename,'file')
                        %                     k = k + 1;
                        %                     NewFilename = fullfile(obj.ParentImage.ProjectFolder,['Last',num2str(k),'.emp']);
                        %                 end
                        movefile(PropertiesFilename,NewFilename);
                    catch
                        h = msgbox({'The map property file could not be saved in the corresponding subfolder...'...
                            ;'The same information is available in the file "load*.emp" located in the project folder.'}...
                            ,'Error','error','modal');
                        uiwait(h);
                    end
                end
            elseif Status == 0 && isempty(obj.Stack.Data)
                delete(obj);
            end
            
            
            obj.ParentImage.EnableEverything(true);
        end
        
%%      % =================================================================
        % Fitting functions
        % =================================================================
        function UpdateFitting(obj)
            % Update the fitting to the base geometrical object (ellipsoid)
            % INPUT ARGUMENTS:
            % -------------------------------------------------------------
            if ~isempty(obj.ParentImage)
                % -----------------------------------------------------
                % Initialization
                % -----------------------------------------------------
                NMP = length(obj.ParentImage.Stack.MaskPoints{1});
                obj.UnfoldCoords = cell(NMP,1); 
                obj.UnfoldLats = zeros(size(obj.LatLines,1),NMP,'uint16');
                obj.UnfoldLongs = zeros(size(obj.LongLines,1),NMP,'uint16');                
                obj.GeoCoords = cell(NMP,1);
                obj.GeoCoordsTable = cell(NMP,1);
                obj.GeoCoordsTableIndex = cell(NMP,1);
                obj.LinkTable = cell(NMP,1);
                obj.EllipsoidCoords  = cell(NMP,1);
                obj.OutputTableIndex = cell(NMP,1);
                % -----------------------------------------------------
                % Roto-translation of coordinates to origin (normalization
                % -----------------------------------------------------
                TL = [0,0,0];
                for t = 1:NMP
                    TL(1) = obj.LogHistory.NewTask([' --> Mask frame ',num2str(t,'%u'),' of ',num2str(NMP,'%u')],TL(1));
                    % -----------------------------------------------------
                    % Roto-translation of coordinates to origin (normalization
                    % -----------------------------------------------------
                    L = obj.ParentImage.Stack.GetMaskCoordsXYZ(1,t,true);
                    NormCoords = (obj.Ellipsoid.NormT*L')';
                    % -----------------------------------------------------
                    % Additional Rotation:
                    % -----------------------------------------------------
                    NormCoords = [sign(obj.Ellipsoid.Rotation(1))*NormCoords(:,abs(obj.Ellipsoid.Rotation(1)))...
                        ,sign(obj.Ellipsoid.Rotation(2))*NormCoords(:,abs(obj.Ellipsoid.Rotation(2)))...
                        ,sign(obj.Ellipsoid.Rotation(3))*NormCoords(:,abs(obj.Ellipsoid.Rotation(3)))];
                    % -----------------------------------------------------
                    % Calculation of geodetic coordinates
                    % -----------------------------------------------------
                    TL(2) = obj.LogHistory.NewTask(' ----> Getting geographical coordinates',TL(2));
                    obj.GetEllipsoidPoints(NormCoords,t);
                    obj.LogHistory.FinishTask(TL(2));
                end;
                obj.LogHistory.FinishTask(TL(1));
            end;
        end;

           
        % =================================================================
        % Function that the distance of each points to an ellipsoid, and the homologous points        
        function GetEllipsoidPoints(obj,Coords,t)
            % Version:  0.2 (03.11.2014)
            %           0.1 (13.03.2014)
            % INPUT ARGUMENTS
            % Coords:       nx3 table with the (x,y,z) coordinates of the points
            % t:            time frame point
            % -------------------------------------------------------------------------
            % Initialization
            % -------------------------------------------------------------------------
            NPoints = size(Coords,1);
            EPoints = zeros(NPoints,4,'single');
            EPoints(:,4) = 1;
%             EPoints(NPoints,4) = 0;             % Ellipsoid points (x,y,z)
            obj.GeoCoords{t} = zeros(NPoints,4,'single');      % Geodetic coordinates (lat, long, h, R)
            obj.EllipsoidCoords{t} = zeros(NPoints,1,'uint32');
            NormalVector(NPoints,3) = 0;     % Normal XY vectors
            % -------------------------------------------------------------------------
            % Determination of octant of each point
            % -------------------------------------------------------------------------
            NegDir = uint8(Coords < 0);
            Octs = NegDir(:,1)*1 + NegDir(:,2)*2 + NegDir(:,3)*4 + 1;
            % -------------------------------------------------------------------------
            % Analysis of each octant
            % -------------------------------------------------------------------------
            IndexOct = false(size(Coords,1),1);
            IndexGeo = false(size(IndexOct));
            for o = 1:8       % Analysis of each octant
                % ---------------------------------------------------------------------
                % First octant SemiAxis
                % ---------------------------------------------------------------------
                [CurrentSemiAxis,Reflections] = obj.Ellipsoid.FirstOctantSemiAxis(o);
                LocalSortedAxes = obj.Ellipsoid.SortedAxes(o,:);
                LocalSemiAxis = CurrentSemiAxis(LocalSortedAxes);
                % ---------------------------------------------------------------------
                % First octant coordinates
                % ---------------------------------------------------------------------
                IndexOct(:) = (Octs == o);
%                 LocalPoints = abs(Coords(IndexOct,obj.AxisOrder(o,:)));
                LocalPoints = abs(Coords(IndexOct,LocalSortedAxes));
                % ---------------------------------------------------------------------
                % Calculate Distance to ellipsoid
                % ---------------------------------------------------------------------
                EPoints(IndexOct,1:3) = VDist2EllipsoidSpecial(LocalSemiAxis,LocalPoints);
                % ---------------------------------------------------------------------
                % Original Octant Coordinates
                % ---------------------------------------------------------------------
%                 EPoints(IndexOct,1) = EPoints(IndexOct,1) * Reflections(LocalSortedAxes(1));
%                 EPoints(IndexOct,2) = EPoints(IndexOct,2) * Reflections(LocalSortedAxes(2));
%                 EPoints(IndexOct,3) = EPoints(IndexOct,3) * Reflections(LocalSortedAxes(3));
%                 EPoints(IndexOct,1:3) = EPoints(IndexOct,obj.Ellipsoid.UnsortedAxes(o,:));
%                 EPoints(IndexOct,obj.Ellipsoid.UnsortedAxes(o,:)) = EPoints(IndexOct,1:3);
                EPoints(IndexOct,LocalSortedAxes) = EPoints(IndexOct,1:3);
                EPoints(IndexOct,1) = EPoints(IndexOct,1) * Reflections(1);
                EPoints(IndexOct,2) = EPoints(IndexOct,2) * Reflections(2);
                EPoints(IndexOct,3) = EPoints(IndexOct,3) * Reflections(3);
                % ---------------------------------------------------------------------
                % Determination of Geo coordinates: latitudes and longitudes
                % ---------------------------------------------------------------------
                % Special case: |Latitude| = 90
                Indexes = (EPoints(IndexOct,1) == 0) & (EPoints(IndexOct,2) == 0);
                IndexGeo(:) = false;
                IndexGeo(IndexOct) = Indexes;
                obj.GeoCoords{t}(IndexGeo,1) = sign(EPoints(IndexGeo,3))*90;
                Indexes = ~Indexes;
                IndexGeo(IndexOct) = Indexes;
                % Longitudes
                K = sqrt(sum(EPoints(IndexGeo,1:2).^2,2));
                obj.GeoCoords{t}(IndexGeo,2) = acosd(EPoints(IndexGeo,1)./K);
                % Latitudes
                K = sum(EPoints(IndexGeo,1:2).^2,2);
                K = K ./ (sqrt(sum([K,EPoints(IndexGeo,3).^2],2)).*sqrt(K));
                Indexes = abs(K) > 1;
                K(Indexes) = sign(K(Indexes));
                obj.GeoCoords{t}(IndexGeo,1) = acosd(K);
                % ---------------------------------------------------------------------
                % Determination of R (normal distance from Ellipsoid Point to axis)
                % ---------------------------------------------------------------------
                NormalVector(IndexOct,:) = ...
                    [EPoints(IndexOct,1)./CurrentSemiAxis(1).^2, ...
                     EPoints(IndexOct,2)./CurrentSemiAxis(2).^2, ...
                     EPoints(IndexOct,3)./CurrentSemiAxis(3).^2];
                K = sqrt(sum(NormalVector(IndexOct,:).^2,2));
                NormalVector(IndexOct,1) = NormalVector(IndexOct,1) ./K;
                NormalVector(IndexOct,2) = NormalVector(IndexOct,2) ./K;
                NormalVector(IndexOct,3) = NormalVector(IndexOct,3) ./K;
                NormalVector(IndexOct,:) = EPoints(IndexOct,1:3) ./ NormalVector(IndexOct,:);
                obj.GeoCoords{t}(IndexOct,4) = min(NormalVector(IndexOct,:),[],2);
            end;
            IndexGeo = EPoints(:,2) < 0;
            obj.GeoCoords{t}(IndexGeo,2) = -obj.GeoCoords{t}(IndexGeo,2);
            IndexGeo = EPoints(:,3) < 0;
            obj.GeoCoords{t}(IndexGeo,1) = -obj.GeoCoords{t}(IndexGeo,1);
            % -------------------------------------------------------------------------
            % Determination of Geo coordinates: Heights
            % -------------------------------------------------------------------------
            obj.GeoCoords{t}(:,3) = sqrt(sum((Coords - EPoints(:,1:3)).^2,2));
            IndexGeo = (abs(Coords(:,3)) - abs(EPoints(:,3))) < 0;
            obj.GeoCoords{t}(IndexGeo,3) = - obj.GeoCoords{t}(IndexGeo,3);
            % ---------------------------------------------------------------------
            % Determination of 3D Ellipsoid Points
            % ---------------------------------------------------------------------
            % Remove additional rotation
%             EPoints(:,abs(obj.Ellipsoid.Rotation)) = ...
%                     [sign(obj.Ellipsoid.Rotation(1))*EPoints(:,1)...
%                     ,sign(obj.Ellipsoid.Rotation(2))*EPoints(:,2)...
%                     ,sign(obj.Ellipsoid.Rotation(3))*EPoints(:,3)];
            EPoints(:,1) = sign(obj.Ellipsoid.Rotation(1))*EPoints(:,1);
            EPoints(:,2) = sign(obj.Ellipsoid.Rotation(2))*EPoints(:,2);
            EPoints(:,3) = sign(obj.Ellipsoid.Rotation(3))*EPoints(:,3);
            EPoints(:,abs(obj.Ellipsoid.Rotation)) = EPoints(:,1:3);
            % Transfer to original position
%             EPoints(:,:) = (obj.Ellipsoid.NormT\EPoints')';
            EPoints(:,:) = (obj.Ellipsoid.InvNormT*EPoints')';                        
            % Units to pixel
            EPoints(:,1) = EPoints(:,1) / obj.ParentImage.Stack.PixelSize(1);
            EPoints(:,2) = EPoints(:,2) / obj.ParentImage.Stack.PixelSize(2);
            EPoints(:,3) = EPoints(:,3) / obj.ParentImage.Stack.PixelSize(3);
            EPoints(:,1:3) = round(EPoints(:,1:3));
            % Remove points outside the image (convert to zeros)
            IndexGeo = EPoints(:,2) < 1 | EPoints(:,1) < 1 | EPoints(:,3) < 1 ...
                | EPoints(:,2) > obj.ParentImage.Stack.DataSize(1) ...
                | EPoints(:,1) > obj.ParentImage.Stack.DataSize(2) ...
                | EPoints(:,3) > obj.ParentImage.Stack.DataSize(3);
            obj.EllipsoidCoords{t}(IndexGeo) = 0;
            IndexGeo = ~IndexGeo;
            obj.EllipsoidCoords{t}(IndexGeo) = uint32(sub2ind(obj.ParentImage.Stack.DataSize(1:3)...
                ,EPoints(IndexGeo,2),EPoints(IndexGeo,1),EPoints(IndexGeo,3)));
        end
        
        
%%      % =================================================================
        % GeoTable functions 
        % =================================================================

        function UpdateGeoCoordsTable(obj)
            % Update the fitting to the base geometrical object (ellipsoid)
            % INPUT ARGUMENTS:
            % -------------------------------------------------------------
            % Initialization
            % Add an extra channel for the distance in the Stack:

            % -------------------------------------------------------------
            % GeoCoordsTableSize
            % -------------------------------------------------------------
            TL = [0,0];
            [obj.DeltaLat,obj.DeltaLong] = obj.MapProj.GetGeoSampling(obj.GeoCoordsTableSize);
%             if isempty(obj.DeltaLat) || isempty(obj.DeltaLong)
%                 if isempty(obj.GeoCoordsTableSize)
%                     TL(1) = obj.LogHistory.NewTask(' --> Estimating Table Size');
%                     obj.EstimateGeoSampling;
%                     obj.LogHistory.FinishTask(TL(1));
%                     obj.GeoCoordsTableSize(1) = length(obj.DeltaLat);
%                     obj.GeoCoordsTableSize(2) = length(obj.DeltaLong);
%                 else
%                     [obj.DeltaLat,obj.DeltaLong] = obj.MapProj.GetGeoSampling(obj.GeoCoordsTableSize);
% %                     L = linspace(-90,90,obj.GeoCoordsTableSize(1)+1)';
% %                     obj.DeltaLat = L(2:end);
% %                     L = linspace(-180,180,obj.GeoCoordsTableSize(2)+1)';
% %                     obj.DeltaLong = L(2:end);
%                 end;
%             end;
            % -------------------------------------------------------------
            % Create GeoCoordsTable for every time point
            % -------------------------------------------------------------            
            for t = 1:length(obj.ParentImage.Stack.MaskPoints{1})
                TL(2) = obj.LogHistory.NewTask([' --> Creating Table for Mask frame ',num2str(t,'%u')...
                    ,' of ',num2str(length(obj.ParentImage.Stack.MaskPoints{1}),'%u')],TL(2));
                %                     obj.GridLatCoords{t} = zeros(NLatP*NLatL,3);             % Initialization
                %                     obj.GridLongCoords{t} = zeros(NLongP*NLongL,3);             % Initialization
                % ---------------------------------------------------------
                % Calculation of HR coordinates of the grid
                % ---------------------------------------------------------
                obj.CreateGeoTable(t);
                %                     obj.MainExtraChannel(2,1)
            end;
            obj.LogHistory.FinishTask(TL(2));
        end            
            
        
        function EstimateGeoSampling(obj)
            % Estimate generic sampling for the largest ellipsoid octant
            % Version:  0.2 (22.05.2014)
            %           0.1 (14.03.2014)
            % INPUT ARGUMENTS
            % OUTPUT ARGUMENTS
            % -------------------------------------------------------------
            % Maximum SemiAxis
            % -------------------------------------------------------------
%             SAp = max(obj.GetSemiAxis) ./ obj.ParentImage.Stack.PixelSize;
            SAp = max(obj.Ellipsoid.GetRotatedSemiAxis) ./ obj.ParentImage.Stack.PixelSize;
            % -------------------------------------------------------------
            % Latitudes
            % -------------------------------------------------------------
            a = ceil(max(SAp(1:2)));
            b = ceil(SAp(3));
            DeltaLatAux = zeros(a*b,1);
            k = 0;
            yaux = 0;
            for ii = a:-1:0
                y = round(sqrt(1 - (ii./a)^2)*b);     % Ellipse formula: (x/a)^2 + (y/b)^2 = 1
                for jj = yaux:y
                    k = k + 1;
                    DeltaLatAux(k,1) = atand(jj/ii);
                end;
                yaux = y;
            end
            obj.DeltaLat = [-flipud(DeltaLatAux(2:k-1,1));DeltaLatAux(1:k,1)];
            % -------------------------------------------------------------
            % Longitudes
            % -------------------------------------------------------------
            a = ceil(SAp(1));
            b = ceil(SAp(2));
            DeltaLongAux = zeros(a*b,1);
            k = 0;
            yaux = 0;
            for ii = a:-1:0
                y = round(sqrt(1 - (ii./a)^2)*b);     % Ellipse formula: (x/a)^2 + (y/b)^2 = 1
                for jj = yaux:y
                    k = k + 1;
                    DeltaLongAux(k,1) = atand(jj/ii);
                end;
                yaux = y;
            end
            FullDeltaLong = [DeltaLongAux(1:k,1);(DeltaLongAux(2:k,1)+90)];
            obj.DeltaLong = [-flipud(FullDeltaLong(1:end-1,1)); FullDeltaLong];
        end
        
        % =================================================================

        function CreateGeoTable(obj,t)
            % Function that creates a table of data for the map unfolding
            % Version:  0.1 (13.03.2014)
            % INPUT ARGUMENTS
            % t:        Time index of Coords
            % OUTPUT ARGUMENTS
            % -------------------------------------------------------------
            % Initialization
            % -------------------------------------------------------------
            N = length(obj.GeoCoords{t});
            obj.GeoCoordsTable{t} = zeros(length(obj.DeltaLat),length(obj.DeltaLong),2,'single');
            obj.GeoCoordsTableIndex{t} = zeros(length(obj.DeltaLat),length(obj.DeltaLong),1,'uint32');
            Planeh = zeros(length(obj.DeltaLat),length(obj.DeltaLong),'single');
            PlaneR = zeros(length(obj.DeltaLat),length(obj.DeltaLong),'single');
            obj.OutputTableIndex{t} = zeros(length(obj.DeltaLat)*length(obj.DeltaLong),1,'uint32');
            GeoCoordsAux = zeros(length(obj.DeltaLat),length(obj.DeltaLong),1,'single');
            obj.LinkTable{t} = zeros(N,1,'uint32');
            LinkAux = zeros(N,2,'uint32');
            Coords = zeros(N,3,'single');
            [Coords(:,1),Coords(:,2),Coords(:,3)] = ind2sub(obj.ParentImage.Stack.DataSize(1:3),single(obj.ParentImage.Stack.MaskList{1}{t}));
%             PlaneCoords = zeros(length(obj.DeltaLat)*length(obj.DeltaLong),3,'single');
            PlaneCoords1 = zeros(length(obj.DeltaLat),length(obj.DeltaLong),'single');
            PlaneCoords2 = zeros(length(obj.DeltaLat),length(obj.DeltaLong),'single');
            PlaneCoords3 = zeros(length(obj.DeltaLat),length(obj.DeltaLong),'single');
%             ExtendedPlane = zeros(length(obj.DeltaLat),length(obj.DeltaLong)+2,'double'); 
            % -------------------------------------------------------------
            % Sort GeoCoords
            % -------------------------------------------------------------
            %             [GeoCoordsSorted,SIndex] = sortrows(obj.GeoCoords{t},[1,2]);
            % -------------------------------------------------------------
            % Creation of auxiliary link table
            % -------------------------------------------------------------
%             obj.LogHistory.NewTask('-- Creating table of coordinates');

            MaxLat = length(obj.DeltaLat);
            MaxLong = length(obj.DeltaLong);
            LIndexes = false(N,1);
            for ii = 1:MaxLat
                LIndexes2 = obj.GeoCoords{t}(:,1) <= obj.DeltaLat(ii);
                LIndexes = xor(LIndexes,LIndexes2);
                LinkAux(LIndexes,1) = ii;
                LIndexes = LIndexes2;
            end;
%             LIndexes = obj.GeoCoords{t}(:,2) > obj.DeltaLong(MaxLong-1);
%             obj.GeoCoords{t}(LIndexes,2) =  -obj.GeoCoords{t}(LIndexes,2);
            LIndexes = false(N,1);
            for jj = 1:MaxLong % MaxLong-1
                LIndexes2 = obj.GeoCoords{t}(:,2) <= obj.DeltaLong(jj);
                LIndexes = xor(LIndexes,LIndexes2);
                LinkAux(LIndexes,2) = jj;
                LIndexes = LIndexes2;
            end;
            % -------------------------------------------------------------
            % Creation link table
            % -------------------------------------------------------------
            obj.LinkTable{t} = sub2ind([MaxLat,MaxLong],LinkAux(:,1),LinkAux(:,2));
            % -------------------------------------------------------------
            % Creation GeoCoords table
            % -------------------------------------------------------------
%             obj.LogHistory.NewTask('-- Filling table of coordinates');
%             Offset = MaxLat*MaxLong;
            for k = 1:N
                % Sum h
                Planeh(obj.LinkTable{t}(k)) = Planeh(obj.LinkTable{t}(k)) + obj.GeoCoords{t}(k,3);
                % Sum R
                PlaneR(obj.LinkTable{t}(k)) = PlaneR(obj.LinkTable{t}(k)) + obj.GeoCoords{t}(k,4);
                % Counter
                GeoCoordsAux(obj.LinkTable{t}(k)) = GeoCoordsAux(obj.LinkTable{t}(k)) + 1;
                % Sum coordinates
                PlaneCoords1(obj.LinkTable{t}(k)) = PlaneCoords1(obj.LinkTable{t}(k)) + Coords(k,1);
                PlaneCoords2(obj.LinkTable{t}(k)) = PlaneCoords2(obj.LinkTable{t}(k)) + Coords(k,2);
                PlaneCoords3(obj.LinkTable{t}(k)) = PlaneCoords3(obj.LinkTable{t}(k)) + Coords(k,3);
            end;
            % -------------------------------------------------------------
            % Poles
            % -------------------------------------------------------------
            GeoCoordsAux(1,:) = sum(GeoCoordsAux(1,:));
            GeoCoordsAux(end,:) = sum(GeoCoordsAux(end,:));
            Planeh(1,:) = sum(Planeh(1,:));
            Planeh(end,:) = sum(Planeh(end,:));
            PlaneR(1,:) = sum(PlaneR(1,:));
            PlaneR(end,:) = sum(PlaneR(end,:));
            PlaneCoords1(1,:) = sum(PlaneCoords1(1,:));
            PlaneCoords2(1,:) = sum(PlaneCoords2(1,:));
            PlaneCoords3(1,:) = sum(PlaneCoords3(1,:));
            PlaneCoords1(end,:) = sum(PlaneCoords1(end,:));
            PlaneCoords2(end,:) = sum(PlaneCoords2(end,:));
            PlaneCoords3(end,:) = sum(PlaneCoords3(end,:));
            % -------------------------------------------------------------
            % Average of overlapping points
            % -------------------------------------------------------------
            Planeh = double(Planeh./GeoCoordsAux);
            PlaneR = double(PlaneR./GeoCoordsAux);
            PlaneCoords1 = double(PlaneCoords1./GeoCoordsAux);
            PlaneCoords2 = double(PlaneCoords2./GeoCoordsAux);
            PlaneCoords3 = double(PlaneCoords3./GeoCoordsAux);
            % -------------------------------------------------------------
            % Reference Longitude
            % -------------------------------------------------------------
            if obj.Ellipsoid.LongShift ~= 0
                [~,RefZero] = min(abs(obj.DeltaLong));
                [~,obj.RefShift(2)] = min(abs(obj.DeltaLong-obj.Ellipsoid.LongShift));
                obj.RefShift(2) = RefZero - obj.RefShift(2);
%                 Planeh(:,1:end-1)=circshift(Planeh(:,1:end-1),obj.RefShift(2),2);
%                 PlaneR(:,1:end-1)=circshift(PlaneR(:,1:end-1),obj.RefShift(2),2);
%                 PlaneCoords1(:,1:end-1)=circshift(PlaneCoords1(:,1:end-1),obj.RefShift(2),2);
%                 PlaneCoords2(:,1:end-1)=circshift(PlaneCoords2(:,1:end-1),obj.RefShift(2),2);
%                 PlaneCoords3(:,1:end-1)=circshift(PlaneCoords3(:,1:end-1),obj.RefShift(2),2);
%                 GeoCoordsAux(:,1:end-1) = circshift(GeoCoordsAux(:,1:end-1),obj.RefShift(2),2);
                Planeh(:,:)=circshift(Planeh,obj.RefShift(2),2);
                PlaneR(:,:)=circshift(PlaneR,obj.RefShift(2),2);
                PlaneCoords1(:,:)=circshift(PlaneCoords1,obj.RefShift(2),2);
                PlaneCoords2(:,:)=circshift(PlaneCoords2,obj.RefShift(2),2);
                PlaneCoords3(:,:)=circshift(PlaneCoords3,obj.RefShift(2),2);
                GeoCoordsAux(:,:) = circshift(GeoCoordsAux,obj.RefShift(2),2);
            else
                obj.RefShift(2) = 0;
            end;
            % -------------------------------------------------------------
            % Last Longitude = First Longitude
            % -------------------------------------------------------------
%             Planeh(:,end) = Planeh(:,1);
%             PlaneR(:,end) = PlaneR(:,1);
%             PlaneCoords1(:,end) = PlaneCoords1(:,1);
%             PlaneCoords2(:,end) = PlaneCoords2(:,1);
%             PlaneCoords3(:,end) = PlaneCoords3(:,1);
%             GeoCoordsAux(:,end) = GeoCoordsAux(:,1);
            % -------------------------------------------------------------
            % Interpolation of the GeoCoordsTable (R = 0) positions
            % -------------------------------------------------------------
            % Coordinates of data and missing points
            Idx = find(GeoCoordsAux(:) ~= 0);
            [Rows,Cols] = ind2sub([MaxLat,MaxLong],Idx);
            Idx2 = find(GeoCoordsAux(:) == 0);
            [Rows2,Cols2] = ind2sub([MaxLat,MaxLong],Idx2);
            % Interpolation of h
%             F = scatteredInterpolant(Rows,Cols...
%                 ,Plane(Idx)...
%                 ,'nearest' ...
%                 ,'nearest');
%             Plane(Idx2) = F(Rows2,Cols2);
            Planeh(Idx2) = griddata(Rows,Cols,Planeh(Idx),Rows2,Cols2,'nearest');
%             hFilter = fspecial('average',[5,5]);
%             hFilter = fspecial('gaussian',[5,5],1);
            hFilter = fspecial('average',ceil([length(obj.DeltaLat),length(obj.DeltaLong)]/50));
            Planeh = imfilter(Planeh,hFilter,'replicate');
            Planeh(1,:) = max(Planeh(1,:));
            Planeh(end,:) = max(Planeh(end,:));

            obj.GeoCoordsTable{t}(:,:,1) = single(Planeh);
            % Interpolation of R
%             F = scatteredInterpolant(Rows,Cols...
%                 ,Plane2(Idx)...
%                 ,'nearest' ...
%                 ,'nearest');
%             Plane2(Idx2) = F(Rows2,Cols2);
            PlaneR(Idx2) = griddata(Rows,Cols,PlaneR(Idx),Rows2,Cols2,'nearest');
            PlaneR = imfilter(PlaneR,hFilter,'replicate');
            PlaneR(1,:) = max(PlaneR(1,:));
            PlaneR(end,:) = max(PlaneR(end,:));
            obj.GeoCoordsTable{t}(:,:,2) = single(PlaneR);

            % Interpolation of PlaneCoords 1
            PlaneCoords1(Idx2) = griddata(Rows,Cols,PlaneCoords1(Idx),Rows2,Cols2,'nearest');
            PlaneCoords1 = imfilter(PlaneCoords1,hFilter,'replicate');
            PlaneCoords1(1,:) = max(PlaneCoords1(1,:));
            PlaneCoords1(end,:) = max(PlaneCoords1(end,:));
            PlaneCoords1 = round(PlaneCoords1);
            % Interpolation of PlaneCoords 2
            PlaneCoords2(Idx2) = griddata(Rows,Cols,PlaneCoords2(Idx),Rows2,Cols2,'nearest');
            PlaneCoords2 = imfilter(PlaneCoords2,hFilter,'replicate');
            PlaneCoords2(1,:) = max(PlaneCoords2(1,:));
            PlaneCoords2(end,:) = max(PlaneCoords2(end,:));
            PlaneCoords2 = round(PlaneCoords2);
            % Interpolation of PlaneCoords 3
            PlaneCoords3(Idx2) = griddata(Rows,Cols,PlaneCoords3(Idx),Rows2,Cols2,'nearest');
            PlaneCoords3 = imfilter(PlaneCoords3,hFilter,'replicate');
            PlaneCoords3(1,:) = max(PlaneCoords3(1,:));
            PlaneCoords3(end,:) = max(PlaneCoords3(end,:));
            PlaneCoords3 = round(PlaneCoords3);
            
            obj.GeoCoordsTableIndex{t}(:) = uint32(sub2ind(obj.ParentImage.Stack.DataSize(1:3) ...
                ,PlaneCoords1(:),PlaneCoords2(:),PlaneCoords3(:)));

% %             % Coordinates of data and missing points
% %             ExtendedPlane(:,2:end-1) = GeoCoordsAux;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             Idx = find(ExtendedPlane(:) ~= 0);
% %             [Rows,Cols] = ind2sub([MaxLat,MaxLong+2],Idx);
% %             Idx2 = find(ExtendedPlane(:) == 0);
% %             [Rows2,Cols2] = ind2sub([MaxLat,MaxLong+2],Idx2);
% %             % Interpolation of h
% % %             F = scatteredInterpolant(Rows,Cols...
% % %                 ,Plane(Idx)...
% % %                 ,'nearest' ...
% % %                 ,'nearest');
% % %             Plane(Idx2) = F(Rows2,Cols2);
% %             ExtendedPlane(:,2:end-1) = Planeh;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             ExtendedPlane(Idx2) = griddata(Rows,Cols,ExtendedPlane(Idx),Rows2,Cols2,'nearest');
% % %             hFilter = fspecial('average',[5,5]);
% % %             hFilter = fspecial('gaussian',[5,5],1);
% %             hFilter = fspecial('average',ceil([length(obj.DeltaLat),length(obj.DeltaLong)]/50));
% %             ExtendedPlane(:,:) = imfilter(ExtendedPlane,hFilter,'replicate');
% %             obj.GeoCoordsTable{t}(:,:,1) = single(ExtendedPlane(:,2:end-1));
% %             obj.GeoCoordsTable{t}(1,:,1) = max(obj.GeoCoordsTable{t}(1,:,1));
% %             obj.GeoCoordsTable{t}(end,:,1) = max(obj.GeoCoordsTable{t}(end,:,1));
% %             % Interpolation of R
% %             ExtendedPlane(:,2:end-1) = PlaneR;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             ExtendedPlane(Idx2) = griddata(Rows,Cols,ExtendedPlane(Idx),Rows2,Cols2,'nearest');
% %             ExtendedPlane(:,:) = imfilter(ExtendedPlane,hFilter,'replicate');
% %             obj.GeoCoordsTable{t}(:,:,2) = single(ExtendedPlane(:,2:end-1));
% %             obj.GeoCoordsTable{t}(1,:,2) = max(obj.GeoCoordsTable{t}(1,:,2));
% %             obj.GeoCoordsTable{t}(end,:,2) = max(obj.GeoCoordsTable{t}(end,:,2));
% %             % Interpolation of PlaneCoords 1
% %             ExtendedPlane(:,2:end-1) = PlaneCoords1;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             ExtendedPlane(Idx2) = griddata(Rows,Cols,ExtendedPlane(Idx),Rows2,Cols2,'nearest');
% %             ExtendedPlane(:,:) = imfilter(ExtendedPlane,hFilter,'replicate');
% %             PlaneCoords1 = ExtendedPlane(:,2:end-1);
% %             PlaneCoords1(1,:) = max(PlaneCoords1(1,:));
% %             PlaneCoords1(end,:) = max(PlaneCoords1(end,:));
% %             PlaneCoords1 = round(PlaneCoords1);
% %             % Interpolation of PlaneCoords 2
% %             ExtendedPlane(:,2:end-1) = PlaneCoords2;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             ExtendedPlane(Idx2) = griddata(Rows,Cols,ExtendedPlane(Idx),Rows2,Cols2,'nearest');
% %             ExtendedPlane(:,:) = imfilter(ExtendedPlane,hFilter,'replicate');
% %             PlaneCoords2 = ExtendedPlane(:,2:end-1);
% %             PlaneCoords2(1,:) = max(PlaneCoords2(1,:));
% %             PlaneCoords2(end,:) = max(PlaneCoords2(end,:));
% %             PlaneCoords2 = round(PlaneCoords2);
% %             % Interpolation of PlaneCoords 3
% %             ExtendedPlane(:,2:end-1) = PlaneCoords3;
% %             ExtendedPlane(:,1) = ExtendedPlane(:,end-1);
% %             ExtendedPlane(:,end) = ExtendedPlane(:,2);
% %             ExtendedPlane(Idx2) = griddata(Rows,Cols,ExtendedPlane(Idx),Rows2,Cols2,'nearest');
% %             ExtendedPlane(:,:) = imfilter(ExtendedPlane,hFilter,'replicate');
% %             PlaneCoords3 = ExtendedPlane(:,2:end-1);
% %             PlaneCoords3(1,:) = max(PlaneCoords3(1,:));
% %             PlaneCoords3(end,:) = max(PlaneCoords3(end,:));
% %             PlaneCoords3 = round(PlaneCoords3);
% %             
% %             obj.GeoCoordsTableIndex{t}(:) = uint32(sub2ind(obj.ParentImage.Stack.DataSize(1:3) ...
% %                 ,PlaneCoords1(:),PlaneCoords2(:),PlaneCoords3(:)));



%             obj.LogHistory.FinishTask(false);
        end        
       
        
        
%%      % =================================================================
        % Unfolding 
        % =================================================================
        
        function UpdateUnfolding(obj)
            % Update the unfolding coordinates
            % Version:  0.2 (17.03.2014)            
            % -------------------------------------------------------------
            % Perform the unfolding:
            NLat = length(obj.DeltaLat);
            NLong = length(obj.DeltaLong);
%             if obj.RefCoord(2) ~= 0
%                 DeltaLongShifted = zeros(size(obj.DeltaLong));
%                 DeltaLongShifted(1:end-1) = circshift(obj.DeltaLong(1:end-1),obj.RefShift(2));
%             else
%                 DeltaLongShifted = obj.DeltaLong;
%             end;
            GCoords = zeros(max(NLat,NLong),2);
            TL = 0;
            for t = 1:length(obj.GeoCoordsTable)
                TL = obj.LogHistory.NewTask([' --> Mask frame ',num2str(t,'%u')...
                    ,' of ',num2str(length(obj.ParentImage.Stack.MaskPoints{1}),'%u')],TL);
                % ---------------------------------------------------------
                % Data Unfolding
                % ---------------------------------------------------------
                % Initialization of the output coordinates
                obj.UnfoldCoords{t} = zeros(NLat*NLong,2);
                % Unfold longitude per longitude
                k2 = 0;
                for jj = 1:NLong
                    % Preparation of the table
                    GCoords(1:NLat,1) = obj.DeltaLat;
%                     GCoords(1:NLat,2) = DeltaLongShifted(jj); % obj.DeltaLong(jj);
                    GCoords(1:NLat,2) = obj.DeltaLong(jj);
                    % Map unfolding:
                    k = k2+1;
                    k2 = k2 + NLat;
%                     obj.UnfoldCoords{t}(k:k2,:) = obj.unfolding(GCoords(1:NLat,:),obj.RefCoord(2));
                    obj.UnfoldCoords{t}(k:k2,:) = obj.unfolding(GCoords(1:NLat,:),0);
                end;
                % ---------------------------------------------------------
                % Grid Unfolding
                % ---------------------------------------------------------
                % Longitudes
                if obj.Ellipsoid.LongShift == 0
                    for k = 1:size(obj.LongLines,1)
                        %                     [~,Idx] = min(abs(obj.DeltaLong-obj.LongLines(k,1)));
                        %                     [~,Idx] = min(abs(DeltaLongShifted-obj.LongLines(k,1)));
                        [~,Idx] = min(abs(obj.DeltaLong-obj.LongLines(k,1)));
                        obj.UnfoldLongs(k,t) = Idx;
                    end;
                else
                    DeltaLongShifted = circshift(obj.DeltaLong,obj.RefShift(2));
                    for k = 1:size(obj.LongLines,1)
                        [~,Idx] = min(abs(DeltaLongShifted-obj.LongLines(k,1)));
                        obj.UnfoldLongs(k,t) = Idx;
                    end;
                end;
                % Latitudes
                for k = 1:size(obj.LatLines,1)
                    [~,Idx] = min(abs(obj.DeltaLat-obj.LatLines(k,1)));
                    obj.UnfoldLats(k,t) = Idx;
                end;
            end;
            obj.LogHistory.FinishTask(TL);
        end        

        % =================================================================
        
        function UnfoldCoords = unfolding(obj,GCoords,RefLong)
            % Return the unfolding XY coordinates of the ECoords
            % Version:  0.1 (XX.XX.2013)
            % INPUT ARGUMENTS
            % ECoords:          List of coordinates [Lat,Long,h,R]
            % OUTPUT ARGUMENTS
            % UnfoldCoords:        List of mapped coordinates [x,y]         
            % -------------------------------------------------------------
            if nargin < 3
                RefLong = 0;
            end;
            % Longitude shift definition
            if RefLong > 0
                Long = GCoords(:,2) + RefLong;
                Long(Long > 180) = Long(Long>180)-360;
                GCoords(:,2) = Long;
            elseif RefLong < 0
                Long = GCoords(:,2) + RefLong;
                Long(Long < -180) = Long(Long<-180)+360;
                GCoords(:,2) = Long;                
            end;
            % Unfolding----------------------------------------------------
            UnfoldCoords = obj.MapProj.Unfold(GCoords);
        end
        
%%      % =================================================================
        % Map Image
        % =================================================================

        function UpdateMapImage(obj)
            % Update Figure5D Stack
            % -------------------------------------------------------------
            if ~isempty(obj.ParentImage)
                % ---------------------------------------------------------
                % Define OutputSize
                % ---------------------------------------------------------
                if obj.AutoOutputSize
                    obj.OutputSize(1) = length(obj.DeltaLat);
                    obj.OutputSize(2) = length(obj.DeltaLong);
                end;
                % ---------------------------------------------------------
                % Map image initialization
                % ---------------------------------------------------------
                % Update DataClass
                if obj.BorderProj == 4
                    obj.Stack.DataClass = 'single';
                else
                    obj.Stack.DataClass = obj.ParentImage.Stack.DataClass;
                end;
                % Predefine stack:
                obj.Stack.Initialize([obj.OutputSize(1),obj.OutputSize(2),1,obj.ParentImage.Stack.DataSize(4),obj.ParentImage.Stack.DataSize(5)]);
                obj.InitializeFigure;
                NMP = length(obj.ParentImage.Stack.MaskPoints{1});
                % Additional channel for the grid
                CGrid = obj.Stack.AddExtraChannel(NMP,'uint8',true,true);
                % Additional channel for the height
                CHeight = obj.Stack.AddExtraChannel(NMP,'single',true,true);
                % Axis titles:
                obj.Stack.ChannelYLabel(:) = {'Latitudes'};
                obj.Stack.ChannelXLabel(:) = {'Longitudes'};                
                % ---------------------------------------------------------
                % Matrices initialization
                % ---------------------------------------------------------                
                obj.Stack.MaskList{1} = cell(NMP,1);
                obj.InterpolationMask = cell(NMP,1);
                Plane = zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),'double');
                Plane2 = zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),'double');
%                Plane3 = zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2)+2,'double');
                PlaneLogical = false(obj.Stack.DataSize(1),obj.Stack.DataSize(2));
                Cube = zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),obj.Stack.DataSize(4),'single');
                % ---------------------------------------------------------
                % Grid parameters
                % ---------------------------------------------------------                
                GridFilter = [1/3 1/3 1/3];
%                 XLat = zeros(size(obj.DeltaLat,1),2,'single');
%                 XLong = zeros(size(obj.DeltaLong,1),2,'single');
                PlaneMask = false(obj.Stack.DataSize(1),obj.Stack.DataSize(2));
%                 IdxBorderGrid = cell(4,1);
%                 if obj.RefCoord(2) == 0
%                     LongColor = obj.LongLines(:,2);
%                 else
%                     LongColor = zeros(size(obj.LongLines,1),1);
%                     LongColor(1:end-1) = circshift(obj.LongLines(1:end-1,2),obj.RefShift(2));
%                     LongColor(end) = LongColor(1);
%                 end;
                % ---------------------------------------------------------
                % Interpolation initialization
                % ---------------------------------------------------------
                Interpolation = obj.InterpolationName;
                if strcmpi(Interpolation,'none');
                    UseInterpolation = false;
                else
                    UseInterpolation = true;
                end
                % ---------------------------------------------------------
                % Check Height Factor
                % ---------------------------------------------------------
                Offset = length(obj.DeltaLat)*length(obj.DeltaLong);
                if obj.ROption == 1
                    for t = 1:NMP
                        [ML,k] = min(obj.HeightFactor * obj.GeoCoordsTable{t}(1:Offset));
                        ML = ML + max(obj.GeoCoordsTable{t}(Offset+1:end));
                        if ML < 0
                            obj.HeightFactor = -max(obj.GeoCoordsTable{t}(Offset+1:end)) / obj.GeoCoordsTable{t}(k);
                        end;
                    end;
                else                    
                    for t = 1:NMP
                        [ML,k] = min(obj.HeightFactor * obj.GeoCoordsTable{t}(1:Offset) + obj.GeoCoordsTable{t}(Offset+1:end));
                        if ML < 0
                            obj.HeightFactor = -obj.GeoCoordsTable{t}(Offset+k)/obj.GeoCoordsTable{t}(k);
                        end;
                    end;
                end;
                % ---------------------------------------------------------
                % Channel border projection initialization
                % ---------------------------------------------------------
                switch obj.BorderProj
                    case 1  % Mean
                        ChannelsTable = zeros(length(obj.DeltaLat)*length(obj.DeltaLong)...
                                             ,1,obj.ParentImage.Stack.DataSize(4),'single');
                    case 2  % Max
                        ChannelsTable = ones(length(obj.DeltaLat)*length(obj.DeltaLong)...
                                             ,1,obj.ParentImage.Stack.DataSize(4),obj.Stack.DataClass)...
                                             * intmin(obj.Stack.DataClass);
                    case 3  % Min
                        ChannelsTable = ones(length(obj.DeltaLat)*length(obj.DeltaLong)...
                                             ,1,obj.ParentImage.Stack.DataSize(4),obj.Stack.DataClass)...
                                             * intmax(obj.Stack.DataClass);
                    case 4  % Sum
                        ChannelsTable = zeros(length(obj.DeltaLat)*length(obj.DeltaLong)...
                                             ,1,obj.ParentImage.Stack.DataSize(4),'single');
                end;
                ChannelsTableAux = zeros(length(obj.DeltaLat)*length(obj.DeltaLong),1,'single');
                % ---------------------------------------------------------
                TL = [0,0,0,0,0];
                for t = 1:obj.ParentImage.Stack.DataSize(5)
                    TL(1) = obj.LogHistory.NewTask([' --> Frame ',num2str(t,'%u'),' of ',num2str(NMP,'%u')],TL(1));
                    if t <= NMP
                        t2 = t;
                        % -------------------------------------------------
                        % Determination of Height & R values when using heights
                        % -------------------------------------------------
                        if any(obj.UseHeights)
                            if obj.ROption == 1
                                HR = reshape(obj.HeightFactor * obj.GeoCoordsTable{t}(:,:,1) ...
                                    ,length(obj.DeltaLat)*length(obj.DeltaLong),1)...
                                    + max(max(obj.GeoCoordsTable{t}(:,:,2)));
                            else                                
                                HR = reshape(obj.HeightFactor * obj.GeoCoordsTable{t}(:,:,1) + obj.GeoCoordsTable{t}(:,:,2)...
                                    ,length(obj.DeltaLat)*length(obj.DeltaLong),1);                                
                            end;
                        end;
                        % -----------------------------------------------------
                        % Determination of limit values
                        % -----------------------------------------------------
                        ValidIdx = isfinite(obj.UnfoldCoords{t}(:,1));
                        UnfoldPointsList = zeros(size(obj.UnfoldCoords{t}));
                        UnfoldPointsList(ValidIdx,:) = obj.UnfoldCoords{t}(ValidIdx,:);
                        if obj.UseHeights(1)
                            UnfoldPointsList(:,1) = UnfoldPointsList(:,1).*HR;
                        end;
                        if obj.UseHeights(2)
                            UnfoldPointsList(:,2) = UnfoldPointsList(:,2).*HR;
                        end;
                        mL = min(UnfoldPointsList(ValidIdx,:),[],1);
                        ML = max(UnfoldPointsList(ValidIdx,:),[],1);
                        % -----------------------------------------------------
                        % Linear conversion (line slopes)
                        % -----------------------------------------------------
                        m1 = (obj.Stack.DataSize(2)-1)/(ML(1)-mL(1));
                        m2 = (obj.Stack.DataSize(1)-1)/(ML(2)-mL(2));
                        % -----------------------------------------------------
                        % Points in pixel coordinates
                        % -----------------------------------------------------
                        PointsList = zeros(size(UnfoldPointsList),'uint32');
                        PointsList(ValidIdx,:) = uint32(round([m1*(UnfoldPointsList(ValidIdx,1) - mL(1)) + 1 ...
                            ,m2*(UnfoldPointsList(ValidIdx,2) - mL(2)) + 1]));
                        % -------------------------------------------------
                        % Update OutputTableIndex
                        % -------------------------------------------------
                        obj.OutputTableIndex{t} = sub2ind(obj.Stack.DataSize(1:2),PointsList(ValidIdx,2),PointsList(ValidIdx,1));
                        % -------------------------------------------------
                        % Valid limits for the GeoCoordsTable
                        % -------------------------------------------------
                        ValidIdx = reshape(ValidIdx,length(obj.DeltaLat),length(obj.DeltaLong));
                        Idx2 = find(any(ValidIdx,1));
                        LongLim = [Idx2(1),Idx2(end)];
                        XLong = zeros(length(Idx2),2,'single');
                        Idx2 = find(any(ValidIdx,2));
                        LatLim = [Idx2(1),Idx2(end)];
                        XLat = zeros(length(Idx2),2,'single');
                        % -------------------------------------------------
                        % Extra Channel Grid
                        % -------------------------------------------------
                        TL(2) = obj.LogHistory.NewTask(' ----> Creating 2D Grid',TL(2));
                        PlaneMask(:) = false;
                        % Longitudes---------------------------------------
                        NL = size(obj.GeoCoordsTable{t},1);
                        OffsetT = prod(obj.Stack.DataSize(1:2)) * (t-1);
                        for k = 1:size(obj.LongLines,1)
                            if ValidIdx(LatLim(1),obj.UnfoldLongs(k,t))
                                Offset = (uint32(obj.UnfoldLongs(k,t))-1)*NL;
                                % Filtering Rows
                                XLat(:,1) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),2)));
                                XLat(1:end-1,1) = XLat(2:end,1);
                                XLat(1,1) = PointsList(Offset+LatLim(1),2);
                                XLat(end,1) = PointsList(Offset+LatLim(2),2);
                                % Filtering Cols
                                XLat(:,2) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),1)));
                                XLat(1:end-1,2) = XLat(2:end,2);
                                XLat(1,2) = PointsList(Offset+LatLim(1),1);
                                XLat(end,2) = PointsList(Offset+LatLim(2),1);
                                % Line points
                                Idx = LinePointsIndexes(obj.Stack.DataSize(1:2),XLat(:,1),XLat(:,2));
                                obj.Stack.Extra{CGrid}(OffsetT + Idx) = 255*obj.Stack.Extra{CGrid}(OffsetT + Idx)+obj.LongLines(k,2);
                                %                             obj.Stack.Extra{CGrid}(OffsetT + Idx) = 255*obj.Stack.Extra{CGrid}(OffsetT + Idx)+LongColor(k);
                            end;
                        end;
                        % Left border--------------------------------------
                        Offset = uint32(LongLim(1)-1)*NL;
                        % Filtering Rows
                        XLat(:,1) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),2)));
                        XLat(1:end-1,1) = XLat(2:end,1);
                        XLat(1,1) = PointsList(Offset+LatLim(1),2);
                        XLat(end,1) = PointsList(Offset+LatLim(2),2);
                        % Filtering Cols
                        XLat(:,2) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),1)));
                        XLat(1:end-1,2) = XLat(2:end,2);
                        XLat(1,2) = PointsList(Offset+LatLim(2),1);
                        XLat(end,2) = PointsList(Offset+LatLim(2),1);
                        % Line points
                        Idx = LinePointsIndexes(obj.Stack.DataSize(1:2),XLat(:,1),XLat(:,2));
                        obj.Stack.Extra{CGrid}(OffsetT + Idx) = obj.GridBorderColor;                        
                        % Right border-------------------------------------
                        Offset = uint32(LongLim(2)-1)*NL;
                        % Filtering Rows
                        XLat(:,1) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),2)));
                        XLat(1:end-1,1) = XLat(2:end,1);
                        XLat(1,1) = PointsList(Offset+LatLim(1),2);
                        XLat(end,1) = PointsList(Offset+LatLim(2),2);
                        % Filtering Cols
                        XLat(:,2) = filter(GridFilter,1,single(PointsList(Offset+LatLim(1):Offset+LatLim(2),1)));
                        XLat(1:end-1,2) = XLat(2:end,2);
                        XLat(1,2) = PointsList(Offset+LatLim(1),1);
                        XLat(end,2) = PointsList(Offset+LatLim(2),1);
                        % Line points
                        Idx = LinePointsIndexes(obj.Stack.DataSize(1:2),XLat(:,1),XLat(:,2));
                        obj.Stack.Extra{CGrid}(OffsetT + Idx) = obj.GridBorderColor;                        
                        % Latitudes----------------------------------------
%                         NL = size(obj.GeoCoordsTable{t},1);
                        for k = 1:size(obj.LatLines,1)
                            if ValidIdx(obj.UnfoldLats(k,t),LongLim(1))
%                                 Offset = uint32(obj.UnfoldLats(k,t));
                                Offset = uint32(obj.UnfoldLats(k,t)) + (LongLim(1)-1)*NL;
                                Last = uint32(obj.UnfoldLats(k,t)) + (LongLim(2)-1)*NL;
                                % Cols
                                XLong(:,1) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,1)));
                                XLong(1:end-1,1) = XLong(2:end,1);
                                XLong(1,1) = PointsList(Offset,1);
                                XLong(end,1) = PointsList(Last,1);
                                % Rows
                                XLong(:,2) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,2)));
                                XLong(1:end-1,2) = XLong(2:end,2);
                                XLong(1,2) = PointsList(Offset,2);
                                XLong(end,2) = PointsList(Last,2);
                                % Line Points
                                Idx = LinePointsIndexes(obj.Stack.DataSize(1:2),XLong(:,2),XLong(:,1));
                                obj.Stack.Extra{CGrid}(OffsetT + Idx) = 255*obj.Stack.Extra{CGrid}(OffsetT + Idx) + obj.LatLines(k,2);
                            end;
                        end;
                        % Grid Border grid olor
                        % North Border-------------------------------------
%                         Offset = uint32(ValidLimit(1));
%                         Last = Offset + NL*(size(obj.GeoCoordsTable{t},2)-1);
                        Offset = uint32(LatLim(1)) + (LongLim(1)-1)*NL;
                        Last = uint32(LatLim(1)) + (LongLim(2)-1)*NL;
                        % Cols
                        XLong(:,1) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,1)));
                        XLong(1:end-1,1) = XLong(2:end,1);
                        XLong(1,1) = PointsList(Offset,1);
                        XLong(end,1) = PointsList(Last,1);
                        % Rows
                        XLong(:,2) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,2)));
                        XLong(1:end-1,2) = XLong(2:end,2);
                        XLong(1,2) = PointsList(Offset,2);
                        XLong(end,2) = PointsList(Last,2);
                        % Line Points
                        IdxNorth = LinePointsIndexes(obj.Stack.DataSize(1:2),XLong(:,2),XLong(:,1));
                        obj.Stack.Extra{CGrid}(OffsetT + IdxNorth) = obj.GridBorderColor;
                        % South Border-------------------------------------
%                         Offset = uint32(ValidLimit(end));
%                         Last = Offset + NL*(size(obj.GeoCoordsTable{t},2)-1);
%                         Offset = uint32(LatLim(2)) + (LongLim(1)-1)*NL;
%                         Last = Offset + LongLim(2)*NL;
                        Offset = uint32(LatLim(2)) + (LongLim(1)-1)*NL;
                        Last = uint32(LatLim(2)) + (LongLim(2)-1)*NL;
                        % Cols
                        XLong(:,1) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,1)));
                        XLong(1:end-1,1) = XLong(2:end,1);
                        XLong(1,1) = PointsList(Offset,1);
                        XLong(end,1) = PointsList(Last,1);
                        % Rows
                        XLong(:,2) = filter(GridFilter,1,single(PointsList(Offset:NL:Last,2)));
                        XLong(1:end-1,2) = XLong(2:end,2);
                        XLong(1,2) = PointsList(Offset,2);
                        XLong(end,2) = PointsList(Last,2);
                        % Line Points
                        IdxSouth = LinePointsIndexes(obj.Stack.DataSize(1:2),XLong(:,2),XLong(:,1));
                        obj.Stack.Extra{CGrid}(OffsetT + IdxSouth) = obj.GridBorderColor;
                        % -------------------------------------------------
                        % Extra Channel Height
                        % -------------------------------------------------
                        Plane(:) = 0;
                        Plane2(:) = 0;
                        for k = 1:size(PointsList,1)
                            if ValidIdx(k)
                                % Projected channels
                                Plane(PointsList(k,2),PointsList(k,1)) = Plane(PointsList(k,2),PointsList(k,1)) + obj.GeoCoordsTable{t}(k);
                                Plane2(PointsList(k,2),PointsList(k,1)) = Plane2(PointsList(k,2),PointsList(k,1)) + 1;
                            end;
                        end;
                        Plane2(:) = Plane./Plane2;
                        % -----------------------------------------------------
                        % MapCoords{t} index list
                        % -----------------------------------------------------
%                         obj.MapCoords{t} = sub2ind(obj.Stack.DataSize(1:2),PointsList(:,2),PointsList(:,1));
                    end;
                    % -----------------------------------------------------
                    % Projection of channels
                    % -----------------------------------------------------
                    obj.LogHistory.FinishTask(TL(2));
                    TL(3) = obj.LogHistory.NewTask(' ----> Projection of Channels',TL(3));
                    ChannelsTableAux(:) = 0;
                    switch obj.BorderProj
                        case 1      % mean
                            ChannelsTable(:) = 0;
                            Channels = single(obj.ParentImage.Stack.GetMaskChannels(t));
                            for k = 1:obj.ParentImage.Stack.MaskPoints{1}(t2)
                                ChannelsTable(obj.LinkTable{t2}(k),1,:) = ...
                                    ChannelsTable(obj.LinkTable{t2}(k),1,:) + Channels(k,1,:);
                                ChannelsTableAux(obj.LinkTable{t2}(k)) = ChannelsTableAux(obj.LinkTable{t2}(k)) + 1;
                            end;
                            for k = 1:obj.Stack.DataSize(4)
                                ChannelsTable(:,1,k) = ChannelsTable(:,1,k) ./ ChannelsTableAux;
                            end;
                        case 2      % max
                            ChannelsTable(:) = intmin(obj.Stack.DataClass);
                            Channels = obj.ParentImage.Stack.GetMaskChannels(t);
                            for k = 1:obj.ParentImage.Stack.MaskPoints{1}(t2)
                                ChannelsTable(obj.LinkTable{t2}(k),1,:) = ...
                                    max(ChannelsTable(obj.LinkTable{t2}(k),1,:),Channels(k,1,:));
                                ChannelsTableAux(obj.LinkTable{t2}(k)) = ChannelsTableAux(obj.LinkTable{t2}(k)) + 1;                                
                            end;
                        case 3      % min
                            ChannelsTable(:) = intmax(obj.Stack.DataClass);   
                            Channels = obj.ParentImage.Stack.GetMaskChannels(t);
                            for k = 1:obj.ParentImage.Stack.MaskPoints{1}(t2)
                                ChannelsTable(obj.LinkTable{t2}(k),:) = ...
                                    min(ChannelsTable(obj.LinkTable{t2}(k),1,:),Channels(k,1,:));
                                ChannelsTableAux(obj.LinkTable{t2}(k)) = ChannelsTableAux(obj.LinkTable{t2}(k)) + 1;
                            end;
                        case 4      % sum
                            ChannelsTable(:) = 0;
                            Channels = single(obj.ParentImage.Stack.GetMaskChannels(t));
                            for k = 1:obj.ParentImage.Stack.MaskPoints{1}(t2)
                                ChannelsTable(obj.LinkTable{t2}(k),:) = ...
                                    ChannelsTable(obj.LinkTable{t2}(k),1,:) + Channels(k,1,:);
                                ChannelsTableAux(obj.LinkTable{t2}(k)) = ChannelsTableAux(obj.LinkTable{t2}(k)) + 1;
                            end;
                    end;
                    ChannelsTable = reshape(ChannelsTable,length(obj.DeltaLat),length(obj.DeltaLong),obj.ParentImage.Stack.DataSize(4));
                    ChannelsTableAux = reshape(ChannelsTableAux,length(obj.DeltaLat),length(obj.DeltaLong));
                    % -----------------------------------------------------
                    % Poles Channels
                    % -----------------------------------------------------
                    Idx = ChannelsTableAux([LatLim(1),LatLim(2)],:) > 0;
                    for k = 1:obj.Stack.DataSize(4)
                        ChannelsTable(LatLim(1),:,k) = mean(ChannelsTable(LatLim(1),Idx(1,:),k));
                        ChannelsTable(LatLim(2),:,k) = mean(ChannelsTable(LatLim(2),Idx(2,:),k));
                    end;
                    ChannelsTableAux(1:LatLim(1)-1,:) = 0;
                    ChannelsTableAux(LatLim(2)+1:end,:) = 0;
                    ChannelsTableAux(LatLim(1),:) = sum(ChannelsTableAux(LatLim(1),Idx(1,:)));
                    ChannelsTableAux(LatLim(2),:) = sum(ChannelsTableAux(LatLim(2),Idx(2,:)));
                    
%                     if (LatLim(1) == 1 ) && (LatLim(2) == length(obj.DeltaLat));
%                         Idx = ChannelsTableAux([1,end],:) > 0;
%                         for k = 1:obj.Stack.DataSize(4)
%                             ChannelsTable(1,:,k) = mean(ChannelsTable(1,Idx(1,:),k));
%                             ChannelsTableAux(1,:) = sum(ChannelsTable(1,Idx(1,:),k));
%                             ChannelsTable(end,:,k) = mean(ChannelsTable(end,Idx(2,:),k));
%                             ChannelsTableAux(end,:) = sum(ChannelsTable(end,Idx(2,:),k));
%                         end;
%                     elseif LatLim(1) == 1
%                         Idx = ChannelsTableAux([1,end],:) > 0;
%                         for k = 1:obj.Stack.DataSize(4)
%                             ChannelsTable(1,:,k) = mean(ChannelsTable(1,Idx(1,:),k));
%                             ChannelsTableAux(1,:) = sum(ChannelsTable(1,Idx(1,:),k));
%                         end;
%                     elseif (LatLim(2) == length(obj.DeltaLat))
%                         Idx = ChannelsTableAux([1,end],:) > 0;
%                         for k = 1:obj.Stack.DataSize(4)
%                             ChannelsTable(end,:,k) = mean(ChannelsTable(1,Idx(2,:),k));
%                             ChannelsTableAux(end,:) = sum(ChannelsTable(end,Idx(2,:),k));
%                         end;
%                     end;
% % %                     % South Pole
% % %                     Row = (1:length(obj.DeltaLong))*length(obj.DeltaLat);
% % %                     Idx = ChannelsTableAux(Row) > 0;
% % % %                     ChannelsTable(Row,1,:) = mean(ChannelsTable(Row(Idx),1,:),1);
% % % %                     ChannelsTableAux(Row) = sum(ChannelsTable(Row(Idx),1,:));
% % %                     for k = 1:obj.Stack.DataSize(4)
% % %                         ChannelsTable(Row,1,k) = mean(ChannelsTable(Row(Idx),1,k),1);
% % %                         ChannelsTableAux(Row) = sum(ChannelsTable(Row(Idx),1,k));
% % %                     end;
% % %                     % South Pole 
% % %                     Row = Row - length(obj.DeltaLat)+1;
% % %                     Idx = ChannelsTableAux(Row) > 0;
% % % %                     ChannelsTable(Row,1,:) = mean(ChannelsTable(Row(Idx),1,:),1);
% % % %                     ChannelsTableAux(Row) = sum(ChannelsTable(Row(Idx),1,:));
% % %                     for k = 1:obj.Stack.DataSize(4)
% % %                         ChannelsTable(Row,1,k) = mean(ChannelsTable(Row(Idx),1,k),1);
% % %                         ChannelsTableAux(Row) = sum(ChannelsTable(Row(Idx),1,k));
% % %                     end;
                    % -----------------------------------------------------
                    % Shift of longitudes
                    % -----------------------------------------------------
                    % Agregar LongLim if used!!!!!
                    if obj.Ellipsoid.LongShift ~= 0
%                         ChannelsTable(:,1:end-1,:) = circshift(ChannelsTable(:,1:end-1,:),obj.RefShift(2),2);
%                         ChannelsTable(:,end,:) = ChannelsTable(:,1,:);
%                         ChannelsTableAux(:,1:end-1,:) = circshift(ChannelsTableAux(:,1:end-1,:),obj.RefShift(2),2);
%                         ChannelsTableAux(:,end,:) = ChannelsTableAux(:,1,:);
                        ChannelsTable(:,:,:) = circshift(ChannelsTable,obj.RefShift(2),2);
                        ChannelsTableAux(:,:,:) = circshift(ChannelsTableAux,obj.RefShift(2),2);
                    end;
                    % -----------------------------------------------------
                    % Overlapped points
                    % -----------------------------------------------------
                    obj.LogHistory.FinishTask(TL(3));
                    TL(4) = obj.LogHistory.NewTask(' ----> Average of overlapped points',TL(4));
                    Cube(:) = 0;
                    [ii,jj] = ind2sub(size(ChannelsTableAux),(1:numel(ChannelsTableAux))');
                    if any(obj.UseHeights)
                        Plane(:) = -Inf;
                        for k = 1:numel(ChannelsTableAux)
%                             if ~isnan(ChannelsTable(k,1,1))
                            if ChannelsTableAux(k) > 0
                                % Max height
                                if obj.GeoCoordsTable{t}(k) > Plane(PointsList(k,2),PointsList(k,1))
                                    Plane(PointsList(k,2),PointsList(k,1)) = obj.GeoCoordsTable{t}(k);
                                    Cube(PointsList(k,2),PointsList(k,1),:) = single(ChannelsTable(ii(k),jj(k),:));
                                end;
                            end;
                        end;
%                         Plane = isfinite(Plane);
%                         Plane(isinf(Plane)) = 0;
                        PlaneLogical(:,:) = isfinite(Plane);
                    else
                        Plane(:) = 0;                        
                        for k = 1:numel(ChannelsTableAux)
%                             if ~isnan(ChannelsTable(k,1,1))
                            if ValidIdx(k) && ChannelsTableAux(k) > 0
                                % Mean value
                                Cube(PointsList(k,2),PointsList(k,1),:) =...
                                    Cube(PointsList(k,2),PointsList(k,1),:) ...
                                    + single(ChannelsTable(ii(k),jj(k),:));
                                Plane(PointsList(k,2),PointsList(k,1)) = Plane(PointsList(k,2),PointsList(k,1))+1;
                            end;
                        end;
                        PlaneLogical(:,:) = Plane > 0;
                    end;                     
%                     obj.LogHistory.FinishTask(TL(3));
%                     TL(4) = obj.LogHistory.NewTask(' ----> Average of overlapped points',TL(4));
%                     Cube(:) = 0; 
%                     if any(obj.UseHeights)
%                         Plane(:) = -Inf;
%                         for k = 1:numel(ChannelsTableAux)
% %                             if ~isnan(ChannelsTable(k,1,1))
%                             if ChannelsTableAux(k) > 0
%                                 % Max height
%                                 if obj.GeoCoordsTable{t}(k) > Plane(PointsList(k,2),PointsList(k,1))
%                                     Plane(PointsList(k,2),PointsList(k,1)) = obj.GeoCoordsTable{t}(k);
%                                     Cube(PointsList(k,2),PointsList(k,1),:) = single(ChannelsTable(k,1,:));
%                                 end;
%                             end;
%                         end;
% %                         Plane = isfinite(Plane);
% %                         Plane(isinf(Plane)) = 0;
%                         PlaneLogical(:,:) = isfinite(Plane);
%                     else
%                         Plane(:) = 0;                        
%                         for k = 1:numel(ChannelsTableAux)
% %                             if ~isnan(ChannelsTable(k,1,1))
%                             if ChannelsTableAux(k) > 0
%                                 % Mean value
%                                 Cube(PointsList(k,2),PointsList(k,1),:) =...
%                                     Cube(PointsList(k,2),PointsList(k,1),:) ...
%                                     + single(ChannelsTable(k,1,:));
%                                 Plane(PointsList(k,2),PointsList(k,1)) = Plane(PointsList(k,2),PointsList(k,1))+1;
%                             end;
%                         end;
%                         PlaneLogical(:,:) = Plane > 0;
%                     end; 
                    % -----------------------------------------------------
                    % Poles Height 
                    % -----------------------------------------------------
                    % North Pole
                    Idx = PlaneLogical(IdxNorth) == 0;
                    PlaneLogical(IdxNorth(Idx)) = 1;
%                     Idx = Plane(IdxNorth) == 0;
                    Plane(IdxNorth(Idx)) = 1;
%                     Plane2(IdxNorth(Idx)) = obj.GeoCoordsTable{t2}(1,1,1);
                    Plane2(IdxNorth(Idx)) = obj.GeoCoordsTable{t2}(LatLim(1),LongLim(1),1);
                    Offset = prod(obj.Stack.DataSize(1:2));
                    for k = 1:obj.Stack.DataSize(4)
%                         Cube(Offset*(k-1) + IdxNorth(Idx)) = ChannelsTable(1,1,k);
                        Cube(Offset*(k-1) + IdxNorth(Idx)) = ChannelsTable(LatLim(1),LongLim(1),k);
                    end;
                    % South Pole
                    Idx = PlaneLogical(IdxSouth) == 0;
                    PlaneLogical(IdxSouth(Idx)) = 1;
%                     Idx = Plane(IdxSouth) == 0;
                    Plane(IdxSouth(Idx)) = 1;
%                     Plane2(IdxSouth(Idx)) = obj.GeoCoordsTable{t2}(end,end,1);
                    Plane2(IdxSouth(Idx)) = obj.GeoCoordsTable{t2}(LatLim(2),LongLim(2),1);                    
                    for k = 1:obj.Stack.DataSize(4)
%                         Cube(Offset*(k-1) + IdxSouth(Idx)) = ChannelsTable(end,1,k);
%                         Cube(Offset*(k-1) + IdxSouth(Idx)) = ChannelsTable(end,end,k);                        
                        Cube(Offset*(k-1) + IdxSouth(Idx)) = ChannelsTable(LatLim(2),LongLim(2),k);
                    end;       
                    ChannelsTable = reshape(ChannelsTable,length(obj.DeltaLat)*length(obj.DeltaLong),1,obj.ParentImage.Stack.DataSize(4));
                    ChannelsTableAux = reshape(ChannelsTableAux,length(obj.DeltaLat)*length(obj.DeltaLong),1);
                    % -----------------------------------------------------
                    % Update Mask
                    % -----------------------------------------------------
                    if t == t2
% %                         obj.Stack.Mask{t2,1} = sparse(Plane ~= 0);
%                         obj.Stack.Mask{t2,1} = sparse(PlaneLogical);
                        obj.Stack.MaskList{1}{t2} = find(PlaneLogical);
                    end;
                    % -----------------------------------------------------
                    % Interpolation
                    % -----------------------------------------------------
                    obj.LogHistory.FinishTask(TL(4));
                    if UseInterpolation
                        TL(5) = obj.LogHistory.NewTask(' ----> Interpolation',TL(5));
% %                         if t2 == t  % Adjust Height Extra Channel
% %                             % Interpolation Mask:
% %                             IdxInt = find(PlaneLogical);
% % %                             Idx = find(Plane ~= 0);
% %                             [RowsInt,ColsInt] = ind2sub(obj.Stack.DataSize(1:2),IdxInt);
% %                             PlaneMask = obj.Stack.Extra{CGrid}(:,:,1,t) > 0;
% %                             PlaneMask = imfill(PlaneMask,'holes');
% %                             obj.InterpolationMask{t2} = find((PlaneLogical(:) == 0) & PlaneMask(:));
% % %                             obj.InterpolationMask{t2} = find((Plane(:) == 0) & PlaneMask(:));
% %                             [RowsInt2,ColsInt2] = ind2sub(obj.Stack.DataSize(1:2),obj.InterpolationMask{t2});
% %                             % Height extra channel interpolation:
% %                             Plane2(obj.InterpolationMask{t2}) = ...
% %                                 griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
% %                             obj.Stack.Extra{CHeight}(:,:,1,t2) = single(Plane2);
% %                         end;
% %                         % Normal channels interpolation:
% %                         if any(obj.UseHeights)
% %                             for k = 1:obj.Stack.DataSize(4)
% %                                 Plane2(:,:) = Cube(:,:,k);
% %                                 Plane2(obj.InterpolationMask{t2}) = ...
% %                                     griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
% %                                 obj.Stack.Data(:,:,1,k,t) = Plane2;
% %                             end;
% %                         else
% %                             for k = 1:obj.Stack.DataSize(4)
% %                                 Plane2(:,:) = Cube(:,:,k);
% %                                 Plane2(IdxInt) = Plane2(IdxInt) ./ Plane(IdxInt);
% % %                                 F = scatteredInterpolant(Rows,Cols...
% % %                                     ,double(Plane2(Idx))...
% % %                                     ,InterpolationMethods{obj.InterpolationMethod} ...
% % %                                     ,ExtrapolationMethods{obj.ExtrapolationMethod});
% % %                                 Plane2(Idx2) = F(Rows2,Cols2);
% %                                 Plane2(obj.InterpolationMask{t2}) = ...
% %                                     griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
% %                                 obj.Stack.Data(:,:,1,k,t) = Plane2;
% %                             end;
% %                         end;
                        if t2 == t  % Adjust Height Extra Channel
                            % Interpolation Mask:
                            IdxInt = find(PlaneLogical);
%                             Idx = find(Plane ~= 0);
                            [RowsInt,ColsInt] = ind2sub(obj.Stack.DataSize(1:2),IdxInt);
                            PlaneMask = obj.Stack.Extra{CGrid}(:,:,1,t) > 0;
                            PlaneMask = imfill(PlaneMask,'holes');
                            obj.InterpolationMask{t2} = find((PlaneLogical(:) == 0) & PlaneMask(:));
%                             obj.InterpolationMask{t2} = find((Plane(:) == 0) & PlaneMask(:));
                            [RowsInt2,ColsInt2] = ind2sub(obj.Stack.DataSize(1:2),obj.InterpolationMask{t2});
                            % Height extra channel interpolation:
                            Plane2(obj.InterpolationMask{t2}) = ...
                                griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
                            obj.Stack.Extra{CHeight}(:,:,1,t2) = single(Plane2);
                        end;
                        % Normal channels interpolation:
                        if any(obj.UseHeights)
                            for k = 1:obj.Stack.DataSize(4)
                                Plane2(:,:) = Cube(:,:,k);
                                Plane2(obj.InterpolationMask{t2}) = ...
                                    griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
                                obj.Stack.Data(:,:,1,k,t) = Plane2;
                            end;
                        else
                            for k = 1:obj.Stack.DataSize(4)
                                Plane2(:,:) = Cube(:,:,k);
                                Plane2(IdxInt) = Plane2(IdxInt) ./ Plane(IdxInt);
%                                 F = scatteredInterpolant(Rows,Cols...
%                                     ,double(Plane2(Idx))...
%                                     ,InterpolationMethods{obj.InterpolationMethod} ...
%                                     ,ExtrapolationMethods{obj.ExtrapolationMethod});
%                                 Plane2(Idx2) = F(Rows2,Cols2);
                                Plane2(obj.InterpolationMask{t2}) = ...
                                    griddata(RowsInt,ColsInt,double(Plane2(IdxInt)),RowsInt2,ColsInt2,Interpolation);
                                obj.Stack.Data(:,:,1,k,t) = Plane2;
                            end;
                        end;

                        obj.LogHistory.FinishTask(TL(5));
                    else
                        if t2 == t  % Adjust Height Extra Channel
                            obj.Stack.Extra{CHeight}(:,:,1,t2) = Plane2;
                        end;
                        for k = 1:size(Cube,3)
                            obj.Stack.Data(:,:,1,k,t) = Cube(:,:,k) ./ Plane;                        
                        end;
                    end;
                end;
                % ---------------------------------------------------------
                % Update Map Image & color
                % ---------------------------------------------------------
%                 obj.Stack.ChannelDisplayRange(1,:) = [min(obj.Stack.Data(:)),max(obj.Stack.Data(:))];
%                 if any(isnan(obj.Stack.ChannelDisplayRange(1,:)))
%                     obj.Stack.ChannelDisplayRange(1,:) = [0,1];
%                 end;
%                 obj.Stack.ChannelDisplayRange(2,:) = [min(obj.Stack.Extra{1}(:)),max(obj.Stack.Extra{1}(:))];                     
%                 if any(isnan(obj.Stack.ChannelDisplayRange(2,:)))
%                     obj.Stack.ChannelDisplayRange(2,:) = [0,1];
%                 end;
%                 if obj.UseInterpolation
                    set(obj.hToolbar(7),'Visible','on','Enable','on','Value',0); % Mask
%                 else
%                     set(obj.hToolbar(7),'Visible','on','Enable','off','Value',0); % Mask
%                 end;
                obj.LogHistory.FinishTask(TL(1));
                obj.SetImageSliders(true)
%                 obj.ShowImage;
            end;
        end;
        
        
%%      % =================================================================
        % Transfer of grid and heights
        % =================================================================
        
        function TransferGrid(obj,~,~)
            % Transfer Grid to the Main Object
            % -------------------------------------------------------------
            if isempty(obj.ParentImage)
                msgbox({'The original stack has changed...'...
                    ;'Nothing can be transfered anymore!'}...
                    ,'Error','error','modal');
                return;
            end
            if obj.NextUpdate > 0
                msgbox({'The original stack masks have changed...'...
                    ;'You need to update the map (by applying properties) before transfering anything information'}...
                    ,'Error','error','modal');
                return
            end
            % -------------------------------------------------------------
            % Create Additional Channel in Main Object
            % -------------------------------------------------------------
            CGrid = obj.ParentImage.Stack.AddExtraChannel(length(obj.ParentImage.Stack.MaskPoints{1}),obj.ParentImage.Stack.DataClass,false);
            obj.ParentImage.Stack.ChannelXLabel{obj.ParentImage.Stack.DataSize(4)+CGrid} = ...
                strcat('Mapped grid (',obj.Name,')');
            % -------------------------------------------------------------
            % Create Longitude lines
            % -------------------------------------------------------------
            obj.LogHistory.NewTask('Transfering Grid');
            TL = [0,0];
            for t = 1:length(obj.LinkTable)
                TL(1) = obj.LogHistory.NewTask([' - Frame ',num2str(t,'%u'),' of ',num2str(length(obj.ParentImage.Stack.MaskPoints{1}),'%u')],TL(1));
                % List of points in (x,y,z) coordinates);
                P = zeros(numel(obj.GeoCoordsTableIndex{t}),3,'double');
                GeoCoordsTableIndexRefLong = circshift(obj.GeoCoordsTableIndex{t},-obj.RefShift(2),2);
%                 [P(:,1), P(:,2), P(:,3)] = ind2sub(obj.ParentImage.Stack.DataSize(1:3),obj.GeoCoordsTableIndex{t}(:));
                [P(:,1), P(:,2), P(:,3)] = ind2sub(obj.ParentImage.Stack.DataSize(1:3),GeoCoordsTableIndexRefLong(:));
                % Latitude Lines
%                 obj.LogHistory.NewTask('-- Creating Latitude lines');
                NL = length(obj.DeltaLat);
                NLong = length(obj.DeltaLong);
                for ii = 1:size(obj.LatLines,1)
%                     LIndexes = find(obj.GeoCoordsTableIndex{t}(obj.UnfoldLats(ii,t),:) ~= 0);
%                     LIndexes = NL * (LIndexes - 1) + double(obj.UnfoldLats(ii,t));
%                     obj.ParentImageLinePoints(CGrid,t...
%                         ,P(LIndexes,:),obj.LatLines(ii,2),true);
                    LIndexes = NL * (0:NLong-2)' + double(obj.UnfoldLats(ii,t));
                    obj.ParentImageLinePoints(CGrid,t...
                        ,P(LIndexes,:),obj.LatLines(ii,2),true);
                end;
%                 obj.LogHistory.NewTask('-- Creating Longitude lines');
                % Longitude Lines
                for ii = 1:size(obj.LongLines,1)
%                     LIndexes = find(obj.GeoCoordsTableIndex{t}(:,obj.UnfoldLongs(ii,t)) ~= 0);
%                     LIndexes = NL *(double(obj.UnfoldLongs(ii,t)) - 1) + LIndexes ;
%                     obj.ParentImageLinePoints(CGrid,t...
%                         ,P(LIndexes,:),obj.LongLines(ii,2),false);
                    LIndexes = NL *(double(obj.UnfoldLongs(ii,t))-1) + (1:NL)';
                    obj.ParentImageLinePoints(CGrid,t...
                        ,P(LIndexes,:),obj.LongLines(ii,2),false);
                    % Intersections
                    LIndexes = find(obj.GeoCoordsTableIndex{t}(obj.UnfoldLats(:,t),obj.UnfoldLongs(ii,t)) ~= 0);
                    LIndexes = NL *(double(obj.UnfoldLongs(ii,t)) - 1) + LIndexes ;
                    obj.ParentImageLinePoints(CGrid,t,P(LIndexes,:),255,false,false);
                end;
            end;
            obj.LogHistory.FinishAll;
        end;

        function TransferHeights(obj,~,~)
            % Transfer Grid to the Main Object
            % -------------------------------------------------------------
            if isempty(obj.ParentImage)
                msgbox({'The original stack has changed...'...
                    ;'Nothing can be transfered anymore!'}...
                    ,'Error','error','modal');
                return;
            end
            if obj.NextUpdate > 0
                msgbox({'The original stack masks have changed...'...
                    ;'You need to update the map (by applying properties) before transfering anything information'}...
                    ,'Error','error','modal');
                return
            end
            % -------------------------------------------------------------
            % Create Additional Channel in Main Object
            % -------------------------------------------------------------
            CHeight = obj.ParentImage.Stack.AddExtraChannel(length(obj.ParentImage.Stack.MaskPoints{1}),'single',false);
            obj.ParentImage.Stack.Extra{CHeight}(:) = NaN;
            obj.ParentImage.Stack.ChannelXLabel{obj.ParentImage.Stack.DataSize(4)+CHeight} = ...
                strcat('Mapped heights (',obj.Name,')');
            % -------------------------------------------------------------
            % Create Longitude lines
            % -------------------------------------------------------------
            obj.LogHistory.NewTask('Transfering heights:');
            TL = 0;
            for t = 1:length(obj.LinkTable)
                GeoCoordsTableRefLong = circshift(obj.GeoCoordsTable{t},-obj.RefShift(2),2);
                TL = obj.LogHistory.NewTask([' - Frame ',num2str(t,'%u'),' of ',num2str(length(obj.ParentImage.Stack.MaskPoints{1}),'%u')],TL);
                Offset = prod(obj.ParentImage.Stack.DataSize(1:3))*(t-1);
                for k = 1:length(obj.LinkTable{t})
                    obj.ParentImage.Stack.Extra{CHeight}(Offset+obj.ParentImage.Stack.MaskList{1}{t}(k)) ...
                        = GeoCoordsTableRefLong(obj.LinkTable{t}(k));
%                         = obj.GeoCoordsTable{t}(obj.LinkTable{t}(k));
                end;
            end;
            obj.LogHistory.FinishAll;
        end;
        
        % =================================================================        

        function TransferEllipsoid(obj,~,~)
            % Transfer Grid to the Main Object
            % -------------------------------------------------------------
            if isempty(obj.ParentImage)
                msgbox({'The original stack has changed...'...
                    ;'Nothing can be transfered anymore!'}...
                    ,'Error','error','modal');
                return;
            end
            if obj.NextUpdate > 0
                msgbox({'The original stack masks have changed...'...
                    ;'You need to update the map (by applying properties) before transfering anything information'}...
                    ,'Error','error','modal');
                return
            end
            % -------------------------------------------------------------
            % Create Additional Channel in Main Object
            % -------------------------------------------------------------
            CEllipsoid = obj.ParentImage.Stack.AddExtraChannel(length(obj.ParentImage.Stack.MaskPoints{1}),'single',false);
            obj.ParentImage.Stack.Extra{CEllipsoid}(:) = NaN;
            obj.ParentImage.Stack.ChannelXLabel{obj.ParentImage.Stack.DataSize(4)+CEllipsoid} = ...
                strcat('Mapped Ellipsoid with Heights (',obj.Name,')');
            % -------------------------------------------------------------
            % Create Longitude lines
            % -------------------------------------------------------------
            obj.LogHistory.NewTask('Transfering Ellipsoid with heights:');
            TL = 0;
            for t = 1:length(obj.EllipsoidCoords)
                GeoCoordsTableRefLong = circshift(obj.GeoCoordsTable{t},-obj.RefShift(2),2);
                TL = obj.LogHistory.NewTask(['- Frame ',num2str(t,'%u'),' of ',num2str(length(obj.ParentImage.Stack.MaskPoints{1}),'%u')],TL);
                Offset = prod(obj.ParentImage.Stack.DataSize(1:3))*(t-1);
                
                Indexes = obj.EllipsoidCoords{t} ~= 0;
                    obj.ParentImage.Stack.Extra{CEllipsoid}(Offset+obj.EllipsoidCoords{t}(Indexes)) ...
                        = GeoCoordsTableRefLong(obj.LinkTable{t}(Indexes));
               
%                 for k = 1:length(obj.EllipsoidCoords{t})
%                     if obj.EllipsoidCoords{t}(k) ~= 0
%                         obj.ParentImage.Stack.Extra{CEllipsoid}(Offset+obj.EllipsoidCoords{t}(k)) ...
%                             = GeoCoordsTableRefLong(obj.LinkTable{t}(k));
%     %                         = obj.GeoCoordsTable{t}(obj.LinkTable{t}(k));
%                     end
%                 end;
            end;
            obj.LogHistory.FinishAll;
        end;
        
%         function TransferEllipsoid(~,~,obj)
%             % Transfer Grid to the Main Object
%             % -------------------------------------------------------------
%             % Create Additional Channel in Main Object
%             % -------------------------------------------------------------
%             % Create Main Extra Channel------------------------------------
%             ExtraChannel = obj.ParentImage.AddExtraChannel(obj.ParentImage.NCoords,obj.ParentImage.Stack.DataClass);
%             obj.ParentImage.Stack.ChannelXLabel{obj.ParentImage.Stack.DataSize(4)+ExtraChannel} = ...
%                 strcat('Mapped Ellipsoid (',obj.Name,')');
%             % Transfer Grid coordinates
%             obj.ParentImage.Coords2ExtraStack(ExtraChannel,obj.EllipsoidXYZ);
%         end
        
%%      % =================================================================
        % Transfer of ROIs
        % =================================================================

        function ReceivedIdx = ReceiveRoiIndexes(obj,Indexes,T)
            % Convert Parent Coords to Child Coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes to a 3D Stack
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (SurfaceMap.MapCoords)
            % -------------------------------------------------------------
%             MapIndexes = zeros(length(Indexes),1,'uint32');
%             for n = 1:length(Indexes);
%                 MapIndexes(n) = obj.OutputTableIndex{T}(obj.LinkTable(Indexes(n)));
%             end;
%             ReceivedIdx = unique(MapIndexes);
%             ReceivedIdx = unique(obj.OutputTableIndex{T}(obj.LinkTable{T}(Indexes)));
            if T > length(obj.OutputTableIndex)
                T = length(obj.OutputTableIndex);
            end;
            OutputTableIndexRefLong = circshift(reshape(obj.OutputTableIndex{T},length(obj.DeltaLat),length(obj.DeltaLong))...
                ,-obj.RefShift(2),2);
%             ReceivedIdx = unique(OutputTableIndexRefLong(obj.LinkTable{T}(Indexes)));
            ReceivedIdx = unique(OutputTableIndexRefLong(obj.LinkTable{T}(ismember(obj.ParentImage.Stack.MaskList{1}{T},Indexes))));
        end;

        
        function SentIdx = SendRoiIndexes(obj,Indexes,T)
            % Convert Child Coords to parent coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes to a 3D stack
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (SurfaceMap.MapCoords)
            % -------------------------------------------------------------
            if T > length(obj.OutputTableIndex)
                T = length(obj.OutputTableIndex);
            end;
            OutputTableIndexRefLong = circshift(reshape(obj.OutputTableIndex{T},length(obj.DeltaLat),length(obj.DeltaLong))...
                ,-obj.RefShift(2),2);
%             AuxIndexes = find(ismember(obj.OutputTableIndex{T},Indexes));
            AuxIndexes = find(ismember(OutputTableIndexRefLong(:),Indexes));
            SentIdx = obj.ParentImage.Stack.MaskList{1}{T}(ismember(obj.LinkTable{T},AuxIndexes));
        end;
            
        function M = Parent2Child(obj,L,T)
            % Convert Parent Coords to Child Coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes (Map32D.Coords)
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (SurfaceMap.MapCoords)
            % -------------------------------------------------------------
            Pos = false(length(obj.ParentImage.Stack.MaskPoints{1}(T)),1);
            for n = 1:length(L);
                Pos = Pos | (obj.ParentImage.Stack.MaskList{1}{T} == L(n));
            end;
            M = obj.MapCoords{T}(Pos);
        end;
        
        function M = Child2Parent(obj,L,T)
            % Convert Child Coords to Parent Coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes (SurfaceMap.MapCoords)
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (Map32D.Coords)
            % -------------------------------------------------------------
            Pos = false(length(obj.ParentImage.Stack.MaskPoints{1}(T)),1);
            for n = 1:length(L);
                Pos = Pos | (obj.MapCoords{T} == L(n));
            end;
            M = obj.ParentImage.Stack.MaskList{1}{T}(Pos);
        end;


%%      % =================================================================
        % SAVE PROPERTIES 
        % -----------------------------------------------------------------
        
        function SaveProperties(obj,Filename)
            % Save Properties in a File
            % -------------------------------------------------------------
            FileVersion = 0.7;
            Pathname = fileparts(Filename);
            if ~exist(Pathname,'dir');
                mkdir(Pathname);
            end;
            fid = fopen(Filename,'w');
            if fid > 0
                % Headerline
                fprintf(fid,'Map32D - EllipsoidMap\n');
                % Version
                fprintf(fid,'Version\t%f\n',FileVersion);

                % PROJECT FOLDER-------------------------------------------                
                fprintf(fid,'* PROJECT FOLDER & STACK\n');
                % Project Folder
                fprintf(fid,'ProjectFolder\t%s\n',obj.ParentImage.ProjectFolder);
                % Folder
                fprintf(fid,'Folder\t%s\n',obj.Folder);
                % PrefixName
                fprintf(fid,'PrefixName\t%s\n',obj.PrefixName);
                % VoxelSize
                fprintf(fid,'PixelSize\t%s\n',mat2str(obj.ParentImage.Stack.PixelSize));
                % MaskName
                fprintf(fid,'MaskName\t%s\n',obj.ParentImage.Stack.MaskName{1});
                % MaskFileName
                fprintf(fid,'MaskFileName\t%s\n',obj.ParentImage.Stack.MaskFileName{1});

                % ELLIPSOID AND FITTING PARAMETERS ------------------------
                fprintf(fid,'* ELLIPSOID & FITTING PARAMETERS\n');
                % Current Approximation
                fprintf(fid,'FittingApproximation\t%s\n',obj.Ellipsoid.CurrentApprox);
                % AutoFitting
                fprintf(fid,'AutoFitting\t%s\n',mat2str(obj.Ellipsoid.AutoFitting));
                % FittingMask
                fprintf(fid,'FittingMask\t%s\n',mat2str(obj.FittingMask));
                % FittingMaskName
                fprintf(fid,'FittingMaskName\t%s\n',obj.ParentImage.Stack.MaskName{obj.FittingMask});
                % FittingMaskFileName
                fprintf(fid,'FittingMaskFileName\t%s\n',obj.ParentImage.Stack.MaskFileName{obj.FittingMask});
                % ReferenceMaskFrame
                fprintf(fid,'ReferenceMaskFrame\t%s\n',num2str(obj.ReferenceMaskFrame));
                % Ellipsoid Properties ------------------------------------
                % Center
                fprintf(fid,'Center\t%s\n',mat2str(obj.Ellipsoid.Center));
                % SemiAxis
                fprintf(fid,'SemiAxis\t%s\n',mat2str(obj.Ellipsoid.SemiAxis));
                % AxisVectors
                fprintf(fid,'AxisVectors\t%s\n',mat2str(obj.Ellipsoid.AxisVectors));
                % Transformation method -----------------------------------
                % NormT
                fprintf(fid,'NormT\t%s\n',mat2str(obj.Ellipsoid.NormT));
                % InvNormT
                fprintf(fid,'InvNormT\t%s\n',mat2str(obj.Ellipsoid.InvNormT));
                % Rotation and axis----------------------------------------
                % MainAxis
                fprintf(fid,'MainAxis\t%s\n',obj.Ellipsoid.MainAxis);
                % RefLongAxis
                fprintf(fid,'RefLongAxis\t%s\n',obj.Ellipsoid.RefLongAxis);
                % LongShift
                fprintf(fid,'LongShift\t%s\n',num2str(obj.Ellipsoid.LongShift));
                
                % PROJECTION PARAMETERS -----------------------------------
                fprintf(fid,'* PROJECTION PARAMETERS\n');
                % Projection name
                fprintf(fid,'ProjName\t%s\n',obj.MapProj.Name);
                % Projection parameters
                fprintf(fid,'ProjParams\t%s\n',mat2str(obj.MapProj.Param));
                % Unfolding ------------------------------------------------------
                % GeoCoordsTableSize
                fprintf(fid,'GeoCoordsTableSize\t%s\n',mat2str(obj.GeoCoordsTableSize));
                
                % MAP IMAGE------------------------------------------------
                fprintf(fid,'* MAP IMAGE\n');                
                % HeightFactor
                fprintf(fid,'HeightFactor\t%s\n',num2str(obj.HeightFactor));
                % Use Heights
                fprintf(fid,'UseHeights\t%s\n',mat2str(obj.UseHeights));
                % ROption
                fprintf(fid,'ROption\t%s\n',num2str(obj.ROption));
                % BorderProj
                fprintf(fid,'BorderProj\t%s\n',num2str(obj.BorderProj));
                % InterpolationName
                fprintf(fid,'InterpolationName\t%s\n',obj.InterpolationName);
                % OutputSize
                fprintf(fid,'OutputSize\t%s\n',mat2str(obj.OutputSize));
                % Grid lines ------------------------------------------------------
                % LatLines
                fprintf(fid,'LatLines\t%s\n',mat2str(obj.LatLines));
                % LongLines
                fprintf(fid,'LongLines\t%s\n',mat2str(obj.LongLines));
                % GridBorderColor
                fprintf(fid,'GridBorderColor\t%s\n',num2str(obj.GridBorderColor));                
                % Closing file ----------------------------------------------------
                fclose(fid);
            end
        end
        
        

%%      % =================================================================
        % Grid functions 
        % -----------------------------------------------------------------
        
        function ParentImageLinePoints(obj,Channel,Frame,P,Value,Closed,InterpFlag)
            % List of points to form a line between two points
            % Version:  0.1 (08.04.2014)
            % INPUT ARGUMENTS
            % Channel:      Number of Extra channel
            % Frame:        Frame
            % P:            nx3 table with pixel coordinates
            % [rows,cols,pags]
            % Rows:         nx1 vector with the pixel row coordinates of the points
            % Cols:         nx1 vector with the pixel column coordinates of the points
            % Pags:         nx1 vector with the pixel page coordinates (z) of the points
            % Value:        Value for the points (Default = 1)
            % Closed:       Flag to know if it is a closed line (Default = false)
            % Interp:       Flag for the estimation of intermediate points
            %               (Closed is ignored)
            % -------------------------------------------------------------------------
            % Initialization
            % -------------------------------------------------------------------------
            if nargin < 7
                InterpFlag = true;
            end;
            if nargin < 6
                Closed = false;
            end;
            if nargin < 5
                Value = 1;
            end;
           % -------------------------------------------------------------------------
            % Initialization
            % -------------------------------------------------------------------------
            if Value == 0
                disp('ouch');
            end;
            Offset = prod(obj.ParentImage.Stack.DataSize(1:3))* (Frame-1);
            if InterpFlag
                for n = 2:size(P,1)
                    K = max(abs(P(n,:)-P(n-1,:)))+1;
                    Idx = sub2ind(obj.ParentImage.Stack.DataSize(1:3),round(linspace(P(n-1,1),P(n,1),K))...
                        ,round(linspace(P(n-1,2),P(n,2),K))...
                        ,round(linspace(P(n-1,3),P(n,3),K)));
                    obj.ParentImage.Stack.Extra{Channel}(Offset+Idx) = Value;
                end;
                if Closed
                    K = max(abs(P(1,:)-P(n,:)))+1;
                    Idx = sub2ind(obj.ParentImage.Stack.DataSize(1:3),round(linspace(P(n,1),P(1,1),K))...
                        ,round(linspace(P(n,2),P(1,2),K))...
                        ,round(linspace(P(n,3),P(1,3),K)));
                    obj.ParentImage.Stack.Extra{Channel}(Offset+Idx) = Value;
                end;
            else
                Idx = sub2ind(obj.ParentImage.Stack.DataSize(1:3),P(:,1),P(:,2),P(:,3));
                obj.ParentImage.Stack.Extra{Channel}(Offset+Idx) = Value;
            end;
        end
        
        % =================================================================        
        
        function CreateMenubar(obj)
            % Create MenuBar
            % INPUT ARGUMENTS
            % MenuStruct:     Optional base Menubar (not really used)
            % -------------------------------------------------------------
            % Define standard menubar
            MenuStruct = obj.DefineMenubar;
            % -------------------------------------------------------------
            % Modify existing menu 
            % -------------------------------------------------------------
            MenuStruct.Callback{1} = struct(...
                'Label',{{'Duplicate Map';'Save';'Quit'}}...
                ,'Separator',   logical([0,1,1]') ...
                ,'Enable',      logical([1,0,1]') ...
                ,'Callback',    {{{@obj.DuplicateMap} ...
                                ;{@obj.SaveFigure} ...
                                ;{@obj.CloseFigure}}} ...
                ,'hMenu',       zeros(3,1)...
                );
            KeepItems = [2,5,6,7,8];
            MenuStruct.Callback{3}.Label = MenuStruct.Callback{3}.Label(KeepItems);
            MenuStruct.Callback{3}.Separator = MenuStruct.Callback{3}.Separator(KeepItems);
            MenuStruct.Callback{3}.Enable = MenuStruct.Callback{3}.Enable(KeepItems);
            MenuStruct.Callback{3}.Callback = MenuStruct.Callback{3}.Callback(KeepItems);
            MenuStruct.Callback{3}.hMenu = MenuStruct.Callback{3}.hMenu(KeepItems);
            % -------------------------------------------------------------
            % Add Ellipsoid Menu
            % -------------------------------------------------------------
            MenuStruct.Label{end+1} = 'Map';
            MenuStruct.Separator(end+1) = 0;
            MenuStruct.Enable(end+1) = 1;
            MenuStruct.hMenu(end+1) = 0;
            MenuStruct.Callback{end+1} = struct(...
                    'Label',        {{'Transfer Grid...' ...
                                     ;'Transfer Heights' ...
                                     ;'Transfer Ellipsoid w/Heights' ...
                                     ;'Map properties' ...
                                    }} ...
                    ,'Separator',   logical([0,0,0,1]') ...
                    ,'Enable',      logical([1,1,1,1]') ...
                    ,'Callback',    {{{@obj.TransferGrid} ...
                                     ;{@obj.TransferHeights}...
                                     ;{@obj.TransferEllipsoid}...
                                     ;{@obj.PropertiesDialog}...;{@obj.EllipsoidProperties}...
                                    }} ...
                    ,'hMenu',       zeros(4,1) ...
                    );
                
            % -------------------------------------------------------------
            % Superclass CreateMenubar
            % -------------------------------------------------------------
%             CreateMenubar@SurfaceMap(obj,MenuStruct);
            CreateMenubar@Figure5D(obj,MenuStruct);
        end;

        % =================================================================

        function DuplicateMap(obj,~,~)
            % Create another map windows with the same properties
            % -------------------------------------------------------------

            % -------------------------------------------------------------
            % Create new figure
            NewObj = EllipsoidMap(obj.ParentImage,true,'Name',[obj.Name,'-copy']);
            NewObj.Folder = obj.Folder;
            NewObj.NextUpdate = obj.NextUpdate;            
            % -------------------------------------------------------------
            % Copy Stack
            NewObj.Stack = copy(obj.Stack);
            % Copy Rois
            NewObj.Roi = obj.Roi;
            NewObj.RoiDrawColor = obj.RoiDrawColor;
            % -------------------------------------------------------------
            % Fitting properties
            NewObj.Ellipsoid = obj.Ellipsoid;
            NewObj.FittingMask = obj.FittingMask;
            NewObj.ReferenceMaskFrame = obj.ReferenceMaskFrame;
            NewObj.EllipsoidCoords = obj.EllipsoidCoords;
            % GeoTable related properties
            NewObj.GeoCoords = obj.GeoCoords;
            NewObj.GeoCoordsTableSize = obj.GeoCoordsTableSize;
            NewObj.DeltaLat = obj.DeltaLat;
            NewObj.DeltaLong = obj.DeltaLong;
            NewObj.GeoCoordsTable = obj.GeoCoordsTable;
            NewObj.GeoCoordsTableIndex = obj.GeoCoordsTableIndex;
            NewObj.LinkTable = obj.LinkTable;
            % Projection properties
            NewObj.MapProj = obj.MapProj;
            % Unfolding properties
            NewObj.UnfoldCoords = obj.UnfoldCoords;
            NewObj.UseHeights = obj.UseHeights;
            NewObj.HeightFactor = obj.HeightFactor;
            NewObj.ROption = obj.ROption;
            NewObj.RefShift = obj.RefShift;
            % Output map properties
            NewObj.OutputTableIndex = obj.OutputTableIndex;
            NewObj.OutputSize = obj.OutputSize;
            NewObj.AutoOutputSize = obj.AutoOutputSize;
            NewObj.BorderProj = obj.BorderProj;
            NewObj.InterpolationMethod = obj.InterpolationMethod;    % Method to interpolate
            NewObj.InterpolationMask = obj.InterpolationMask;
            % Grid properties
            NewObj.GridBorderColor = obj.GridBorderColor;
            NewObj.LatLines = obj.LatLines;
            NewObj.LongLines = obj.LongLines;
            NewObj.UnfoldLats = obj.UnfoldLats;
            NewObj.UnfoldLongs = obj.UnfoldLongs;


            % -------------------------------------------------------------
            % Update Map
            NewObj.SetImageSliders(true);
            NewObj.InitializeFigure;
            NewObj.RoiManager_Update;
            NewObj.ShowRoi(true);
        end
        
        % =================================================================
        
        function PropertiesDialog(obj,~,~)
            % Dialog to change map properties
            % -------------------------------------------------------------
            EllipsoidProperties(obj);
            %                     obj.UpdateMap(Status);
        end
        
        
    end

    methods (Access=private)

    end    
    

    methods (Static = true)

%         function MPT = GetMapParamsTable(MP,Idx)
%             switch Idx
%                 case 1 % 'Equirectangular'
%                     MPT = cell(1,2);
%                     MPT{1,1} = 'Standard parallel';
%                     MPT{1,2} = MP{1}(1);
%                 case 2 % 'Mercator'
%                     MPT = cell(2,2);
%                     MPT{1,1} = 'North truncation';
%                     MPT{1,2} = MP{2}(1);
%                     MPT{2,1} = 'South truncation';
%                     MPT{2,2} = MP{2}(2);
%                 case 3 % 'Gall-Stereographic'...
%                     MPT = [];
%                 case 4 % 'Miller'...
%                     MPT = [];
%                 case 5 % 'Lambert-Cylindrical'...
%                     MPT = [];
%                 case 6 % 'Gall-Peters'...
%                     MPT = [];
%                 case 7 % 'Sinusoidal'...
%                     MPT = [];
%                 case 8 % 'Mollweide'...
%                     MPT = cell(2,2);
%                     MPT{1,1} = 'Iteration error';
%                     MPT{1,2} = MP{8}(1);
%                     MPT{2,1} = 'Maximum interations';
%                     MPT{2,2} = MP{8}(2);
%                 case 9 % 'KavrayskiyVII'...
%                     MPT = [];
%                 case 10  % 'WagnerVI'...
%                     MPT = [];
%                 case 11 % 'Bonne'...
%                     MPT = cell(1,2);
%                     MPT{1,1} = 'Standard parallel';
%                     MPT{1,2} = MP{11}(1);
%                     
% %                 ,'Orthographic'...
% %                 ,'Stereographic'...
% %                 ,'Lambert'...
% %                 ,'Postel'...
%             end
%         end
        
        
        function LTable = GetDefaultGrid(Option)
            % Function that define a grid
            % Version:  0.2 (23.06.2014)
            if Option == 1
                % ---------------------------------------------------------
                % Latitude lines
                % ---------------------------------------------------------
                Lines = (-75:15:75)';
                L = length(Lines)-1;
                Color = (0:L)'/L * 100 + 100;
                LTable = [Lines,Color];
            else
                % ---------------------------------------------------------
                % Longitude lines
                % ---------------------------------------------------------
                Lines = (-180:15:165)';
                L = length(Lines)-1;
                Color = (0:L)'/L * 100 + 100;
                LTable = [Lines,Color];
            end;
        end;        
        
%         function L = MapNameList(Idx)
%             L = {'Equirectangular'...
%                 ,'Mercator'...
%                 ,'Braun'...               % ,'Gall-Stereographic'...
%                 ,'Miller'...
%                 ,'Lambert-Cylindrical'... % ,'Gall-Peters'...
%                 ,'Sinusoidal'...
%                 ,'Mollweide'...
%                 ,'WagnerVI'...            % ,'KavrayskiyVII'...
%                 ,'Bonne'...
%                 };
% %             L = {'Equirectangular'...
% %                 ,'Mercator'...
% %                 ,'Gall-Stereographic'...
% %                 ,'Miller'...
% %                 ,'Lambert-Cylindrical'...
% %                 ,'Gall-Peters'...
% %                 ,'Sinusoidal'...
% %                 ,'Mollweide'...
% %                 ,'KavrayskiyVII'...
% %                 ,'WagnerVI'...
% %                 ,'Bonne'...
% %                 };
% %                 ,'Orthographic'...
% %                 ,'Stereographic'...
% %                 ,'Lambert'...
% %                 ,'Postel'...
% %                 };
%             if nargin == 1
%                 L = L(Idx);
%             end;
%         end            

        function L = InterpolationNameList(Idx)
            L = {'None'...
                ,'Linear'...
                ,'Cubic'...
                ,'Natural'...
                ,'Nearest'...
                };
            if nargin == 1
                if Idx > 0 && Idx <= length(L)
                    L = L(Idx);
                else
                    L = L(1);
                end
            end;            
        end

    end
        
end





