classdef EllipsoidProjection
    % Class that control the projections
    %
    % Version:  0.7.0 (01.10.2014) Duplicate map

    properties (Dependent = true)
        Name            % Current projection name
        Param           % Current projection parameters 
    end
    
    properties (SetAccess = private, GetAccess = public)
        Current = 1;    % Current projection Index
        ProjNames       % List of projections names
        ProjParams      % List of projections parameters
    end
    
    methods
        
        % -----------------------------------------------------------------
        % Constructor
        function obj = EllipsoidProjection()
            
            % Definition of projections
            % -------------------------------------------------------------
            obj.ProjNames = cell(9,1);
            % 'Equirectangular'
            obj.ProjNames{1} = 'Equirectangular';
            obj.ProjParams(1).Name = {'Standard parallel'};
            obj.ProjParams(1).Value = 0;           % Standard Parallel
            % 'Mercator'
            obj.ProjNames{2} = 'Mercator';
            obj.ProjParams(2).Name = {'North truncation';'South truncation'};
            obj.ProjParams(2).Value = [85;-85];
            % 'Braun'                    (ex 'Gall-Stereographic')
            obj.ProjNames{3} = 'Braun';
            obj.ProjParams(3).Name =[];
            obj.ProjParams(3).Value =[];
            % 'Miller'...
            obj.ProjNames{4} = 'Miller';
            obj.ProjParams(4).Name =[];
            obj.ProjParams(4).Value =[];
            % 'Lambert-Cylindrical'...
            obj.ProjNames{5} = 'Lambert-Cylindrical';
            obj.ProjParams(5).Name =[];
            obj.ProjParams(5).Value =[];
            % 'Sinusoidal'...
            obj.ProjNames{6} = 'Sinusoidal';
            obj.ProjParams(6).Name =[];
            obj.ProjParams(6).Value =[];
            % 'Mollweide'...
            obj.ProjNames{7} = 'Mollweide';
            obj.ProjParams(7).Name = {'Iteration error';'Maximum interations'};
            obj.ProjParams(7).Value = [0.01;50];
            % 'WagnerVI'...
            obj.ProjNames{8} = 'Wagner-VI';
            obj.ProjParams(8).Name =[];
            obj.ProjParams(8).Value=[];
            % 'Bonne'...
            obj.ProjNames{9} = 'Bonne';
            obj.ProjParams(9).Name = {'Standard parallel'};
            obj.ProjParams(9).Value = 45;
        end
        
        % -----------------------------------------------------------------
        % Name property methods
        function Result = get.Name(obj)
            Result = obj.ProjNames{obj.Current};
        end
        function obj = set.Name(obj,Value)
            if ischar(Value)
                obj.Current = find(strcmpi(Value,obj.ProjNames),1);
            elseif Value <= length(obj.ProjNames)
                obj.Current = Value;
            else
                obj.Current = 1;
            end
        end
        
        % -----------------------------------------------------------------
        % Param property methods
        function Result = get.Param(obj)
            Result = obj.ProjParams(obj.Current).Value;
        end
        function obj = set.Param(obj,Value)
            switch obj.ProjNames{obj.Current}
                case 'Equirectangular'
                    if (Value(1) > 85) || (Value(1) < -85)
                        Value(1) = 0;
                    end
                case 'Mercator'
                    if (Value(1) >= 90) || (Value(1) < -80)
                        Value(1) = 85;
                    end
                    if (Value(2) <= -90) || (Value(2) > 80)
                        Value(2) = -85;
                    end;
                case 'Mollweide'
                    if Value(1) == 0
                        Value(1) = 0.0001;
                    else
                        Value(1) = abs(Value(1));
                    end;
                    if Value(2) < 1
                        Value(2) = 10;
                    end;
                    Value(2) = round(Value(2));
                case 'Bonne'
                    if abs(Value(1)) > 90
                        Value(1) = 45;
                    end;
                otherwise
                    Value = [];
            end
            obj.ProjParams(obj.Current).Value = Value;
        end
        
        % -----------------------------------------------------------------
        function T = GetParamsTable(obj,IdxName)
            % Create a cell table whose columns are the parameters  name
            % and value respectively
            % -------------------------------------------------------------
            if ischar(IdxName)
                Idx = find(strcmpi(IdxName,obj.ProjNames),1);
            elseif IdxName <= length(obj.ProjNames)
                Idx = IdxName;
            else
                T = [];
                return
            end
            N = length(obj.ProjParams(Idx).Name);
            T = cell(N,2);
            for n = 1:N
                T{n,1} = obj.ProjParams(Idx).Name{n};
                T{n,2} = obj.ProjParams(Idx).Value(n);
            end
        end
        
        % -----------------------------------------------------------------        
        function PCoords = Unfold(obj,GCoords)
            % Unfold Geographical coords into plane coords according to the
            % current map
            % INPUT ARGUMENTS:
            % GCoords:      Geographical coordinates table [lat,long]
            % OUTPUT ARGUMENTS:
            % PCoords:      Unfolded plane coordinates [X,Y]
            % -------------------------------------------------------------
            PCoords = zeros(size(GCoords,1),2);
            P = obj.ProjParams(obj.Current).Value;
            switch obj.ProjNames{obj.Current}
                % CYLINDRICAL:
                case 'Equirectangular'
                    PCoords = [GCoords(:,2).*cosd(P), GCoords(:,1)];
                case 'Mercator'
                    Idx = (GCoords(:,1) <= P(1)) & (GCoords(:,1) > P(2));
                    PCoords(Idx,:) = [GCoords(Idx,2), log(tand(45 + GCoords(Idx,1)/2))];
                    PCoords(~Idx,:) = NaN;
                case {'Braun','Gall-Stereographic'}
                    PCoords = [GCoords(:,2)/sqrt(2), (1+sqrt(2)/2)*tand(GCoords(:,1)/2)];
                case 'Miller'
                    PCoords = [GCoords(:,2), (5/4)*log(tand(45 + 2*GCoords(:,1)/5))];
                case {'Lambert-Cylindrical','Gall-Peters'}
                    PCoords = [GCoords(:,2), sind(GCoords(:,1))];
%                 case 'Gall-Peters'
%                     UnfoldCoords = [pi.*Long/(180*sqrt(2))... 
%                                 ,sqrt(2).*sind(Lat)];
                % PSEUDO-CYLINDRICAL:
                case 'Sinusoidal'
                    PCoords = [GCoords(:,2).*cosd(GCoords(:,1)), GCoords(:,1)];
                case 'Mollweide'
                    Aux = not(abs(GCoords(:,1) == 90));
                    LatAux = GCoords(Aux,1);
                    t = LatAux*pi/180;
                    Idx = true(length(t),1);
                    k = 0;
                    Deltat = zeros(length(t),1);
                    while any(Idx) || k < P(2)
                        k = k + 1;
                        Deltat(Idx) = (t(Idx) + sin(t(Idx)) -pi*sind(LatAux(Idx)))./(1+1*cos(t(Idx)));
                        t(Idx) = t(Idx) - Deltat(Idx);
                        Idx = abs(Deltat) > P(1);
                    end;
                    Theta = GCoords(:,1)*pi/180;
                    Theta(Aux) = t/2;
                    PCoords = [(2*sqrt(2)/180).*GCoords(:,2).*cos(Theta)... 
                                ,sqrt(2).*sin(Theta)];
%                 case 'KavrayskiyVII'
%                     UnfoldCoords = [(3/2) .* Long*pi/180 .* sqrt((1/3)-(Lat/180).^2)... 
%                                 ,Lat];
                case {'Wagner-VI','Kavrayskiy-VII'}
                    PCoords = [GCoords(:,2) .* sqrt(1-3*((GCoords(:,1)/180).^2))... 
                                ,GCoords(:,1)];
                case 'Bonne'
                    if P(1) == 0
                        PCoords = [pi.*GCoords(:,2)/(180*sqrt(2))...
                            ,sqrt(2).*sind(GCoords(:,1))];
                    else
                        Rho = cotd(P(1)) + (P(1)- GCoords(:,1))*pi/180;
                        E = GCoords(:,2).*pi/180 .* cosd(GCoords(:,1))./Rho;
                        PCoords = [Rho.*sin(E)... 
                                    ,cotd(P(1)) - Rho.*cos(E)];
                    end;
                    % FOR THE FUTURE
%                 % AZIMUTHAL:
%                 case 'Orthographic'
%                     PCoords = [sind(Zenith).*cosd(Azimuth)...
%                                 ,sind(Zenith).*sind(Azimuth)];
%                 case 'Stereographic'
%                     PCoords = [2*tand(Zenith/2).*cosd(Azimuth)...
%                                 ,2*tand(Zenith/2).*sind(Azimuth)];
%                 case 'Lambert'
%                     PCoords = [2*sind(Zenith/2).*cosd(Azimuth)...
%                                 ,2*sind(Zenith/2).*sind(Azimuth)];
%                 case 'Postel'
%                     PCoords = [2*Zenith.*cosd(Azimuth)...
%                                 ,2*Zenith.*sind(Azimuth)];
            end;
%             PCoords(:,2) = -PCoords(:,2);
        end
            
        
        % =================================================================
        % Get the best geo sampling according to the map projection
        function [DeltaLat,DeltaLong] = GetGeoSampling(obj,NPoints)
            % Version: (03.11.2014)  
            % INPUT ARGUMENTS
            % NPoints:      [NLats,NLongs] Number of points for the
            %               geographical table
            % OUTPUT ARGUMENTS
            % DeltaLat      Sampling on latitudes
            % DeltaLong     Sampling on longitudes
            % -------------------------------------------------------------

            EquiLongs = linspace(-180,180,NPoints(2)+1)';
            DeltaLong = EquiLongs(2:end);
            ExtY = obj.Unfold([-90,0;90,0]);
            yy = linspace(ExtY(1,2),ExtY(2,2),NPoints(1)+1)';            
            switch obj.ProjNames{obj.Current}
                % CYLINDRICAL:
                case 'Mercator'
%                     Idx = (GCoords(:,1) <= P(1)) & (GCoords(:,1) > P(2));
%                     PCoords(Idx,:) = [GCoords(Idx,2), log(tand(45 + GCoords(Idx,1)/2))];
%                     PCoords(~Idx,:) = NaN;
                case {'Braun','Gall-Stereographic'}
                    DeltaLat = 2*atand(yy(2:end)/(1+sqrt(2)/2));
                case 'Miller'
                    DeltaLat = (atand(exp((4/5)*yy(2:end)))-45)*(5/2);
                case {'Lambert-Cylindrical','Gall-Peters'}
                    DeltaLat = asind(yy(2:end));
                % PSEUDO-CYLINDRICAL:
                case 'Sinusoidal'
                    DeltaLat = yy(2:end);
                case 'Mollweide'
                    Theta2 = 2*asin(yy(2:end)/sqrt(2));
                    DeltaLat = asind((Theta2 + sin(Theta2))/pi);
                case {'Wagner-VI','Kavrayskiy-VII'}
                    DeltaLat = yy(2:end);
                otherwise  % 'Equirectangular, Bonne
                    EquiLats = linspace(-90,90,NPoints(1)+1)';
                    DeltaLat = EquiLats(2:end);
            end;
        end
        
            
    end
end
