classdef EllipsoidReference
    % Class that control the projections
    %
    % Version:  0.7.0 (01.10.2014) Duplicate map

    properties (Dependent = true)
        
        CurrentApprox;
       
    end
    
    properties (SetAccess = public, GetAccess = public)
        
        % Ellipsoid parameters
        SemiAxis = zeros(2,3);      % SemiAxis of the fit ellipsoid [a+,b+,c+;a-,b-c-]
        Center = zeros(1,3);        % Center of the object/Ellipsoid
        AxisVectors = zeros(3,3);   % Matrix with the Axis vectors column-wise 
                                    % of the ellipsoid
        % Position parameters: Roto-translation matrix
        NormT
        InvNormT

        % Estimation method
        AutoFitting = 2;

        % Further rotation / manipulation of the ellipsoid
        Rotation = [1,2,3];         % Rotation vector (3x1) with the order
                                    % of the axis (x,y,z). Default = [3,2,1]
        SortedAxes                  % (8x3) matrix with the sorted axes 
                                    % where a>b>c for each octant
        UnsortedAxes                % (8x3) matrix with the information 
                                    % to unsort the axes of each octant
        LongShift = 0;              % Shift for the longitudes
        RefLong = 0;                % Reference Longitude
        
        
%         AxisOrder = zeros(8,3);     % Order semiaxis a>b>c for each octant
%         AxisInvOrder = zeros(8,3);  % Invert the order of the semiaxis        

        
    end
    
    
    properties (SetAccess = private, GetAccess = public)
        
        MainAxis = '+Z';
        RefLongAxis = '+X';
        
        Matrix
        Coeff
        
    end
    
    
    methods

        
        % -----------------------------------------------------------------
        % Constructor
        function obj = EllipsoidReference()
            
        end

        % -----------------------------------------------------------------
        % Current Approximation
        function Result = get.CurrentApprox(obj)
            if obj.AutoFitting == 0
                Result = 'Custom';
            else
                Result = obj.FittingMethods(obj.AutoFitting);
            end;
        end
        
        % -----------------------------------------------------------------
        % Estimate ellipsoid
        function obj = EstimateEllipsoid(obj,StackObj,MaskIndex,Frame)
            % Version: (08.10.2014) 
            % INPUT ARGUMENTS:
            % StackObj:     Stack5D object with the masks
            % MaskIndex:    mask to consider
            % Frame:        Frame(s) to consider in the mask
            % -------------------------------------------------------------
            if nargin < 4
                Frame = 1;
            end;
            if nargin < 3
                MaskIndex = 1;
            end;
            % -------------------------------------------------------------
            % Get XYZ coordinates of the mask pixels:
            XYZCoords = StackObj.GetMaskCoordsXYZ(MaskIndex,Frame);
            % Fitting:
            obj.SemiAxis = zeros(2,3);
            switch obj.AutoFitting
                % Case 4-5  (FUTURE!)
                case 3
                    obj = obj.ellipsoid_fit(XYZCoords,0);
%                     obj.CurrentApprox = obj.FittingMethods(3);
                    obj.AutoFitting = 3;
                case 2
                    obj = obj.ellipsoid_fit(XYZCoords,2);
%                     obj.CurrentApprox = obj.FittingMethods(2);
                    obj.AutoFitting = 2;
                case 1
                    obj = obj.ellipsoid_fit(XYZCoords,3);
%                     obj.CurrentApprox = obj.FittingMethods(1);
                    obj.AutoFitting = 1;
            end;
            % -------------------------------------------------------------
            % Calculatation of Transformation matrix:
            Tr = eye(4,4);
            Tr(1:3,4) = -obj.Center;
            % Rotation through quaternions (versors)
            Rm = eye(4,4);
            Rm(1:3,1:3) = obj.AxisVectors;
            % Roto-translation matrix for normalization:
            obj.NormT = Rm*Tr;
            % Inverted matrix
            obj.InvNormT = inv(obj.NormT);            
        end;

        % =================================================================
        % Set the rotation / flipped Axes
        function obj = DefineRotation(obj)
            % Version: (03.11.2014)  
            % OUTPUT ARGUMENTS
            % R:            Rotation vector (3x1) with the order of the axis (x,y,z)
            % -------------------------------------------------------------
            obj.Rotation = [0,0,0];
            switch upper(obj.MainAxis)
                case '+Z'
                    obj.Rotation(3) = 3;
                case '+Y'
                    obj.Rotation(3) = 2;
                case '+X'
                    obj.Rotation(3) = 1;
                case '-Z'
                    obj.Rotation(3) = -3;
                case '-Y'
                    obj.Rotation(3) = -2;
                case '-X'
                    obj.Rotation(3) = -1;
            end
            switch upper(obj.RefLongAxis)
                case '+Z'
                    obj.Rotation(1) = 3;
                case '+Y'
                    obj.Rotation(1) = 2;
                case '+X'
                    obj.Rotation(1) = 1;
                case '-Z'
                    obj.Rotation(1) = -3;
                case '-Y'
                    obj.Rotation(1) = -2;
                case '-X'
                    obj.Rotation(1) = -1;
            end
            switch sum(abs(obj.Rotation))
                case 3
                    obj.Rotation(2) = 3;
                case 4
                    obj.Rotation(2) = 2;
                case 5
                    obj.Rotation(2) = 1;
            end
%             if obj.Rotation(1) < 0
%                 obj.Rotation(2) = -obj.Rotation(2);
%             end;
            % Sort the axes
            obj = obj.SortAxes;
        end;        
        
        % =================================================================
        % Get the rotated/flipped semiaxes
        function SA = GetRotatedSemiAxis(obj)
            % Version: (03.11.2014) 
            % OUTPUT ARGUMENTS
            % R:        % Rotation vector (3x1) with the order of the axis (x,y,z)
            % -------------------------------------------------------------
            SA(1,:) = obj.SemiAxis(1,abs(obj.Rotation));
            SA(2,:) = obj.SemiAxis(2,abs(obj.Rotation));
            if obj.Rotation(1) < 0
                SA(:,1) = flipud(SA(:,1));
            end;
            if obj.Rotation(2) < 0
                SA(:,2) = flipud(SA(:,2));
            end
            if obj.Rotation(3) < 0
                SA(:,3) = flipud(SA(:,3));
            end
        end
        
        % =================================================================
        % Determine the sorted semiaxes  a > b > c for each octant
        function obj = SortAxes(obj)
            % Version: (03.11.2014) 
            % -------------------------------------------------------------
            % Get the rotated semi axis
            SA = obj.GetRotatedSemiAxis; %SA = obj.GetSemiAxis;
            obj.SortedAxes = zeros(8,3);
            obj.UnsortedAxes = zeros(8,3);            
            % Analysis of each octant
            for k = 1:2
                for j = 1:2
                    for i = 1:2
                        a = SA(i,1);
                        b = SA(j,2);
                        c = SA(k,3);
                        n = (k-1)*4 + (j-1)*2 + i; 
                        if a > b
                            if b > c
                                obj.SortedAxes(n,:) = [1,2,3];
                            elseif a > c
                                obj.SortedAxes(n,:) = [1,3,2];
                            else
                                obj.SortedAxes(n,:) = [3,1,2];
                            end;
                        else
                            if a > c
                                obj.SortedAxes(n,:) = [2,1,3];
                            elseif c > b
                                obj.SortedAxes(n,:) = [3,2,1];
                            else
                                obj.SortedAxes(n,:) = [2,3,1];
                            end;
                        end;
                        obj.UnsortedAxes(n,obj.SortedAxes(n,:)) = [1,2,3];                        
                    end;
                end;
            end;
            
        end
        
        % =================================================================
        % Returns the first octant normalized semiaxis
        function [S,Reflections] = FirstOctantSemiAxis(obj,Oct)
            % Version: (03.11.2014) 
            % -------------------------------------------------------------
            % INPUT ARGUMENTS:
            % Oct:          Octant number
            % OUTPUT ARGUMENTS:
            % S:            Octant semiaxis
            % Reflection:   1x3 vector that indicates whether the x, y, z values
            %               must be multiplied by -1, respectively
            % -------------------------------------------------------------
            SA = obj.GetRotatedSemiAxis;             
            switch Oct
                case 1
                    S = [SA(1,1) SA(1,2) SA(1,3)];
                    Reflections = [1,1,1];
                case 2
                    S = [SA(2,1) SA(1,2) SA(1,3)];
                    Reflections = [-1,1,1];
                case 3
                    S = [SA(1,1) SA(2,2) SA(1,3)];
                    Reflections = [1,-1,1];
                case 4
                    S = [SA(2,1) SA(2,2) SA(1,3)];
                    Reflections = [-1,-1,1];
                case 5
                    S = [SA(1,1) SA(1,2) SA(2,3)];
                    Reflections = [1,1,-1];
                case 6
                    S = [SA(2,1) SA(1,2) SA(2,3)];
                    Reflections = [-1,1,-1];
                case 7
                    S = [SA(1,1) SA(2,2) SA(2,3)];
                    Reflections = [1,-1,-1];
                case 8
                    S = [SA(2,1) SA(2,2) SA(2,3)];
                    Reflections = [-1,-1,-1];
            end;
        end

        % =================================================================
        % Set the main axis
        function obj = SetMainAxis(obj,Index)
            % Version: (04.11.2014) 
            % INPUT ARGUMENT:
            % Index:        It can be an index to the element of the list or
            %               the string: '+Z','+Y','+X','-Z','-Y','-X'
            % -------------------------------------------------------------
            if nargin < 2
                return;
            end
            L = obj.MainAxisList(true);            
            if isnumeric(Index) && (Index > 0) && (Index <= length(L))
                obj.MainAxis = L{Index};
            elseif ischar(Index) && ismember(Index,L)
                obj.MainAxis = Index;
            end
            if strcmp(obj.MainAxis(2),obj.RefLongAxis(2))
                obj = obj.SetRefLongAxis(1);
            end
        end
        % =================================================================
        % Get main axis index
        function Index = GetMainAxisIndex(obj)
            % Version: (04.11.2014) 
            % OUTPUT ARGUMENT:
            % Index:        Index of the axis in the main axis list
            % -------------------------------------------------------------
            L = obj.MainAxisList(true);
            Index = find(strcmp(obj.MainAxis,L),1);
        end
        % =================================================================
        % Return a list of reference axis for longitudes
        function L = RefLongAxisList(obj,Index)
            % Version: (04.11.2014)
            % INPUT ARGUMENT:
            % Index:        If it is empty, then the whole list is returned.
            %               If it has a value, only that element of the list is returned.
            %               If it is true, then the whole list is returned,
            %               but with only the first two characters.
            % -------------------------------------------------------------
            if nargin < 2
                Index = [];
            end
            switch upper(obj.MainAxis(2))
                case 'Z'
                    L = {'+X';'+Y'...
                        ;'-X (back)';'-Y (back)'};
                case 'Y'
                    L = {'+X';'+Z'...
                        ;'-X (back)';'-Z (back)'};
                case 'X'
                    L = {'+Y';'+Z'...
                        ;'-Y (back)';'-Z (back)'};
            end
            if islogical(Index) && Index
                for n = 1:length(L)
                    L{n} = L{n}(1:2);
                end
            elseif ~isempty(Index) && Index <= length(L) && Index > 0
                L = L{Index};
            end
        end; 
        % =================================================================
        % Set the ref long axis
        function obj = SetRefLongAxis(obj,Index)
            % Version: (04.11.2014) 
            % INPUT ARGUMENT:
            % Index:        It can be an index to the element of the list or
            %               the string: '+Z','+Y','+X','-Z','-Y','-X', but
            %               it should be contained in the list!
            % -------------------------------------------------------------
            if nargin < 2
                return;
            end
            L = obj.RefLongAxisList(true);
            if isnumeric(Index) && (Index > 0) && (Index <= length(L))
                obj.RefLongAxis = L{Index};
            elseif ischar(Index) && ismember(Index,L)
                obj.RefLongAxis = Index;
            end
        end        
        % =================================================================
        % Get Ref long axis index
        function Index = GetRefLongAxisIndex(obj)
            % Version: (04.11.2014) 
            % OUTPUT ARGUMENT:
            % Index:        Index of the axis in the main axis list
            % -------------------------------------------------------------
            L = obj.RefLongAxisList(true);
            Index = find(strcmp(obj.RefLongAxis,L),1);
        end

        
    end

    
    methods (Static = true, Access = public)

        
        % =================================================================
        % List of main axis possibilities
        function L = MainAxisList(Index)
            % Version: (04.11.2014)
            % INPUT ARGUMENT:
            % Index:        If it is empty, then the whole list is returned.
            %               If it has a value, only that element of the list is returned.
            %               If it is true, then the whole list is returned,
            %               but with only the first two characters.
            % -------------------------------------------------------------
            if nargin < 1
                Index = [];
            end
            L = {'+Z';'+Y';'+X'...
                ;'-Z (flip)';'-Y (flip)';'-X (flip)'};
            if islogical(Index) && Index
                for n = 1:length(L)
                    L{n} = L{n}(1:2);
                end
            elseif ~isempty(Index) && Index <= length(L) && Index > 0
                L = L{Index};
            end
        end;
        
        
        % -----------------------------------------------------------------
        % List of fitting methods
        function L = FittingMethods(Index)
            if nargin < 1
                Index = [];
            end
            L = {'Sphere (algebraic)' ...
                ;'Oblate Ellipsoid (algebraic)' ...
                ;'3-Axial Ellipsoid (algebraic)'...
                };
            if ~isempty(Index) && Index <= length(L) && Index > 0
                L = L{Index};
            end
        end
        
        
    end
    
    
    methods (Static = false, Access = private)
    
        % -----------------------------------------------------------------
        % Fitting algorithm
        function obj = ellipsoid_fit(obj, X, flag, equals )
            % Fit an ellispoid/sphere to a set of xyz data points:
            %
            %   [center, radii, evecs, A, pars ] = ellipsoid_fit( X )
            %   [center, radii, evecs, A, pars ] = ellipsoid_fit( [x y z] );
            %   [center, radii, evecs, A, pars ] = ellipsoid_fit( X, 1 );
            %   [center, radii, evecs, A, pars ] = ellipsoid_fit( X, 2, 'xz' );
            %   [center, radii, evecs, A, pars ] = ellipsoid_fit( X, 3 );
            %
            % Parameters:
            % * X, [x y z]   - Cartesian data, n x 3 matrix or three n x 1 vectors
            % * flag         - 0 fits an arbitrary ellipsoid (default),
            %                - 1 fits an ellipsoid with its axes along [x y z] axes
            %                - 2 followed by, say, 'xy' fits as 1 but also x_rad = y_rad
            %                - 3 fits a sphere
            %
            % Output:
            % * center    -  ellispoid center coordinates [xc; yc; zc]
            % * ax        -  ellipsoid radii [a; b; c]
            % * evecs     -  ellipsoid radii directions as columns of the 3x3 matrix
            % * A         -  ellipsoid matrix
            % * v         -  the 9 parameters describing the ellipsoid algebraically:
            %                Ax^2 + By^2 + Cz^2 + 2Dxy + 2Exz + 2Fyz + 2Gx + 2Hy + 2Iz = 1
            %
            % Author:
            % Yury Petrov, Northeastern University, Boston, MA
            %
            
            % error( nargchk( 1, 3, nargin ) );  % check input arguments
            if nargin == 2
                flag = 0;  % default to a free ellipsoid
            end
            if flag == 2 && nargin == 3
                equals = 'xy';
            end
            
            if size( X, 2 ) ~= 3
                error( 'Input data must have three columns!' );
            else
                x = X( :, 1 );
                y = X( :, 2 );
                z = X( :, 3 );
            end
            
            % need nine or more data points
            if length( x ) < 9 && flag == 0
                error( 'Must have at least 9 points to fit a unique ellipsoid' );
            end
            if length( x ) < 6 && flag == 1
                error( 'Must have at least 6 points to fit a unique oriented ellipsoid' );
            end
            if length( x ) < 5 && flag == 2
                error( 'Must have at least 5 points to fit a unique oriented ellipsoid with two axes equal' );
            end
            if length( x ) < 3 && flag == 3
                error( 'Must have at least 4 points to fit a unique sphere' );
            end
            
            if flag == 0
                % fit ellipsoid in the form Ax^2 + By^2 + Cz^2 + 2Dxy + 2Exz + 2Fyz + 2Gx + 2Hy + 2Iz = 1
                D = [ x .* x, ...
                    y .* y, ...
                    z .* z, ...
                    2 * x .* y, ...
                    2 * x .* z, ...
                    2 * y .* z, ...
                    2 * x, ...
                    2 * y, ...
                    2 * z ];  % ndatapoints x 9 ellipsoid parameters
            elseif flag == 1
                % fit ellipsoid in the form Ax^2 + By^2 + Cz^2 + 2Gx + 2Hy + 2Iz = 1
                D = [ x .* x, ...
                    y .* y, ...
                    z .* z, ...
                    2 * x, ...
                    2 * y, ...
                    2 * z ];  % ndatapoints x 6 ellipsoid parameters
            elseif flag == 2
                % fit ellipsoid in the form Ax^2 + By^2 + Cz^2 + 2Gx + 2Hy + 2Iz = 1,
                % where A = B or B = C or A = C
                if strcmp( equals, 'yz' ) || strcmp( equals, 'zy' )
                    D = [ y .* y + z .* z, ...
                        x .* x, ...
                        2 * x, ...
                        2 * y, ...
                        2 * z ];
                elseif strcmp( equals, 'xz' ) || strcmp( equals, 'zx' )
                    D = [ x .* x + z .* z, ...
                        y .* y, ...
                        2 * x, ...
                        2 * y, ...
                        2 * z ];
                else
                    D = [ x .* x + y .* y, ...
                        z .* z, ...
                        2 * x, ...
                        2 * y, ...
                        2 * z ];
                end
            else
                % fit sphere in the form A(x^2 + y^2 + z^2) + 2Gx + 2Hy + 2Iz = 1
                D = [ x .* x + y .* y + z .* z, ...
                    2 * x, ...
                    2 * y, ...
                    2 * z ];  % ndatapoints x 4 sphere parameters
            end
            
            % solve the normal system of equations
            v = ( D' * D ) \ ( D' * ones( size( x, 1 ), 1 ) );
            
            % find the ellipsoid parameters
            if flag == 0
                % form the algebraic form of the ellipsoid
                A = [ v(1) v(4) v(5) v(7); ...
                    v(4) v(2) v(6) v(8); ...
                    v(5) v(6) v(3) v(9); ...
                    v(7) v(8) v(9) -1 ];
                % find the center of the ellipsoid
                center = -A( 1:3, 1:3 ) \ [ v(7); v(8); v(9) ];
                % form the corresponding translation matrix
                T = eye( 4 );
                T( 4, 1:3 ) = center';
                % translate to the center
                R = T * A * T';
                % solve the eigenproblem
                [ evecs,evals ] = eig( R( 1:3, 1:3 ) / -R( 4, 4 ) );
                radii = sqrt( 1 ./ diag( evals ) );
            else
                if flag == 1
                    v = [ v(1) v(2) v(3) 0 0 0 v(4) v(5) v(6) ];
                elseif flag == 2
                    if strcmp( equals, 'xz' ) || strcmp( equals, 'zx' )
                        v = [ v(1) v(2) v(1) 0 0 0 v(3) v(4) v(5) ];
                    elseif strcmp( equals, 'yz' ) || strcmp( equals, 'zy' )
                        v = [ v(2) v(1) v(1) 0 0 0 v(3) v(4) v(5) ];
                    else % xy
                        v = [ v(1) v(1) v(2) 0 0 0 v(3) v(4) v(5) ];
                    end
                else
                    v = [ v(1) v(1) v(1) 0 0 0 v(2) v(3) v(4) ];
                end
                center = ( -v( 7:9 ) ./ v( 1:3 ) );
                gam = 1 + ( v(7)^2 / v(1) + v(8)^2 / v(2) + v(9)^2 / v(3) );
                radii = ( sqrt( gam ./ v( 1:3 ) ) );
                evecs = eye( 3 );
            end;
            obj.Matrix = [  v(1) v(4) v(5) v(7); ...
                            v(4) v(2) v(6) v(8); ...
                            v(5) v(6) v(3) v(9); ...
                            v(7) v(8) v(9) -1 ];
            obj.Coeff = v;
            obj.Center = center;
            obj.SemiAxis(1,:) = radii;
            obj.AxisVectors = evecs;
            obj.SemiAxis(2,:) = obj.SemiAxis(1,:);            
        end
        
    end
    
end
        
        
        
        
        