classdef Figure5D < handle % matlab.mixin.Copyable  
    % IMAGE5D class to encapsulate operation with 5D stacks
    %   Detailed explanation goes here
    %
    % Version:  0.5.2 (07.02.2014)  Roi Manager
    %           0.5.1 (20.01.2014)  Open/Save Tiff files
    %           0.3.3 (02.09.2013)  Add support for Sparse extra-channels
    %           0.3.2 (28.08.2013)  Save dialog & save function using bioformats
    %           0.3.1 (21.08.2013)  Figure toolbar: Pixel Position, Pan,
    %                               Zoom, Stretch, Mask
    %           0.3.0 (14.08.2013)  Use of sparse matrix for the mask
    %                               Use Extra Stack for the extra channels
    %           0.2.3 (26.06.2013)
    
    properties
        Filename = '';
        Folder = '';
        Name = 'Map32D';
        
        % Data
        Stack
        
        ShowComposite = false;          % Show composite image
        ShowMask = false;

        % Sub-figures
        ChildImages = {}                % {n} List of child images
        ParentImage = []                % Parent Image
        ChildDataTable = [];            % Measurements table
        ListenerDataTable = [];
        DialogListener                  % Handle to Listener objects
        
        % ROI selections
        CurrentSelection = [];          % Handle to the current selection
        CurrentSelectionType = '';      % Current selection type / to be changed later!
        Roi = struct('Selected',0           ...   % 0 = None
                    ,'List',ImageRoi.empty  ...
                    ,'Color',[255,255,0]    ...    
                    ,'DrawValue',255        ...        
                    ,'FillValue',255        ...        
                    ,'Operation',1          ...   
                    ,'SelectAll',false      ...
                    ,'KeepUpdated',true     ...
                    ,'Show2D',true          );
        RoiDrawColor = [255,255];
        
        % Graphical Interface
        hFig = [];                      % Figure handles
        hFigJava = [];                  % Java Figure handle
        hImage = struct('Panel',[]...   % Ex Im(1)
            ,'Axes',[]...               % Ex Im(2)
            ,'Image',[]...              % Ex Im(3)
            ,'LabelAxes',[]...          % Ex Im(4,6)
            ,'LabelText',[]...          % Ex Im(5,7)
            ,'Roi',[]);
        hSidebar = struct('TopPanel',[]...
            ,'TopElements',[]...
            ,'BottomPanel',[]...
            ,'RoiList',[]...
            ,'RoiButtons',[]);

        Slider = zeros(3,1);             % Slider handles
        SliderText = zeros(3,1);         % SliderText handles
        SliderEdit = zeros(3,1);         % SliderEdit handles
        Menubar
        
        hToolbar
        PixelInfo
        ImScroll
        PanZoomFlag = 0;                % Flag indicator: 
                                        % 0 = Nothing active
                                        % 1 = Pan
                                        % 2 = Zoom
        LogHistory

    end

    methods

        % =================================================================
        function obj = Figure5D(Stack,varargin)
            % Constructor
            % INPUT ARGUMENTS (optional):
            % Stack:    5D stack information
            % Properties (varargin):
            % 'DataClass':      Data Class (Default = '', calculated from Stack)
            % -------------------------------------------------------------
            if nargin < 1
                Stack = [];
            end;
            for k = 1:2:length(varargin)-1
                if isprop(obj,varargin{k})
                    obj.(varargin{k}) = varargin{k+1};
                end;
            end;
            
            if isempty(obj.LogHistory)
                obj.LogHistory = LogReg(obj);
            end;
            
            % Stack initialization
            obj.Stack = Stack5D;
            
            % Add listener to stack
            addlistener(obj.Stack,'ResizedStack',@(Source,Event)obj.SetImageSliders(true));
            
            % Load Stack
            if ~isempty(Stack)
                obj.Stack.Load(Stack);
            end
            
            % Solve problem with OpenGL
            gl = opengl('data');
            if strfind(gl.Vendor,'ATI')
                if ispc
                    opengl('software');
                end
            end
        end;

        function InitializeFigure(obj)
            % Initialize the figure;
            % -------------------------------------------------------------
            % Image sizes:
            set(obj.hToolbar(7),'Value',false,'Enable','off');
            obj.ShowMask = false;
            if isempty(obj.Stack.Data)
                % Disable Menubar & Toolbar & Roi Manager
                set(obj.Menubar{2}{3}{1},'Enable','off');
                for n = 3:length(obj.Menubar)
                    set(obj.Menubar{n}{1},'Enable','off');
                end;
                set(obj.hSidebar.RoiButtons(1:8),'Enable','off');                
                % Disable Axes
                set(obj.hImage.Axes,'Visible','off');
                set(obj.hToolbar(1),'Visible','off');
            else
                % Initialize Axes and images
                set(obj.hImage.Axes,'XLim',[0.5 obj.Stack.DataSize(2)+0.5],'YLim',[0.5 obj.Stack.DataSize(1)+0.5]);
                set(obj.hImage.Image,'XData',[1 obj.Stack.DataSize(2)],'YData',[1 obj.Stack.DataSize(1)]);
                % Roi image layer
                set(obj.hImage.Roi,'XData',[1 obj.Stack.DataSize(2)],'YData',[1 obj.Stack.DataSize(1)]);
                CData = zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),3,'uint8');
                CData(:,:,1) = obj.Roi.Color(1);
                CData(:,:,2) = obj.Roi.Color(2);
                CData(:,:,3) = obj.Roi.Color(3);
                set(obj.hImage.Roi,'CData',CData);
                set(obj.hImage.Roi,'AlphaData',zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),'uint8'));
                % Enable Menubar & Toolbar & RoiMeasure
                set(obj.Menubar{2}{3}{1},'Enable','on');
                for n = 3:length(obj.Menubar)
                    set(obj.Menubar{n}{1},'Enable','on');
                end;
                if isempty(obj.Roi.List)
                    set(obj.hSidebar.RoiButtons(5),'Enable','on');
                else
                    set(obj.hSidebar.RoiButtons(2:8),'Enable','on');                    
                end
                set(obj.hImage.Axes,'Visible','on');
                set(obj.hToolbar(1),'Visible','on');
            end;
        end;
        
        % =================================================================
        
        function Status = OpenImage(obj,~,~,IsNew,SearchFolder)
            % Open an image
            % -------------------------------------------------------------
            if nargin < 5
                SearchFolder = obj.Folder;
            end;
            Status = false;
            DeleteAllConnections = false;
            if ~isempty(obj.ChildImages)
                Button = questdlg({'There are other images/maps associated with this image!'...
                    ;'If you open a new image, then all the existing associations will be removed.'...
                    ;'Do you really want to continue?'}...
                    ,'Warning!!!','Yes','No','No');
                if strcmp(Button,'No')
                    return
                else
                    DeleteAllConnections = true;
                end;
            end
            [FileName,PathName] = uigetfile({'*.tif;*.tiff;*.btf','All Tiff files (*.tif,*.tiff,*.btf)';...
                '*.tif;*.tiff','Normal Tiff files (*.tif,*.tiff)';...
                '*.btf;*.tif','BigTiff files (*.btf,*.tif)';...
                '*.*','All files (*.*)'} ...
                ,'Open Tiff/BigTiff file...',SearchFolder);
            if ~isequal(FileName,0)
                if DeleteAllConnections
                    for n = 1:length(obj.ChildImages)
                        obj.ChildImages{n}.ParentImage = [];
                    end;
                    obj.ChildImages = {};
                end
                % S = bf_imread(fullfile(PathName,FileName));
                obj.LogHistory.NewTask('Loading Tiff image');
                [S,VoxelSize] = obj.LoadTiffStack(fullfile(PathName,FileName));
                obj.LogHistory.FinishAll;
                if strcmp(IsNew,'new')
                    NewObj = Map32D(S);
                    NewObj.Filename = FileName;
                    NewObj.Folder = PathName;
                    NewObj.Stack.PixelSize = VoxelSize;
                else
                    obj.Filename = FileName;
                    obj.Folder = PathName;
                    obj.ShowMask = false;
                    obj.Stack.PixelSize = VoxelSize;
                    obj.Stack.Load(S);
                    % Initialize figure:
                    obj.InitializeFigure;
%                     obj.ShowImage;
                    obj.VoxelSizeDialog;
                end;
                Status = true;
            end;
        end;

        % =================================================================

        % -----------------------------------------------------------------
        % Dialog to add masks
        function AddMaskCallback(obj,~,~,MI)
            % Version: (20.10.2014)
            % INPUT ARGUMENTS:
            % MI:        Mask index
            % -------------------------------------------------------------
            if nargin < 4
                MI = 1;
            end;
            % -------------------------------------------------------------
            % Check for child images (map)
            % -------------------------------------------------------------
            InformChildImages = false;
            if ~isempty(obj.ChildImages) && (length(obj.ChildImages) >= MI)
                Button = questdlg({'There are other images/maps associated with this image!'...
                    ;'If you add/change the mask then the associated images should be manually updated!'...
                    ;'Do you really want to continue?'}...
                    ,'Warning!!!','Yes','No','No');
                if strcmp(Button,'No')
                    return
                else
                    InformChildImages = true;
                end
            end
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 7* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 3*GUIP.NormalEditHeight + 2*GUIP.NormalTextHeight;
            Size(1) = mm2pt(70);
            %2 GUIP.DialogMargin(1) ...
             %   + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','Add Mask...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            Pos = [0,0,0,0];
            Offset = 0.5*Size(1);
            % Mask name
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Size(2) - GUIP.DialogMargin(2) - 1*GUIP.NormalEditHeight;
            Pos(3) = Offset - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Mask name:' ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Offset;
            Pos(3) = Size(1)-GUIP.DialogMargin(1)-Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            hD.MaskNameEdit = uicontrol(hD.Fig,'Style','edit'...
                ,'String',['Mask-',num2str(MI)] ... % num2str(obj.BackgroundValue) ...
                ,'BackgroundColor','white','Enable','off'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            % Subpanel for radiobuttons
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(4) = 1*0.75*GUIP.LargePanelMargin(2)+2*GUIP.NormalTextHeight;
            Pos(2) = Pos(2) - Pos(4) - 0.75*GUIP.LargePanelMargin(2);
            Pos(3) = Size(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            SubPanel = uibuttongroup('Parent',hD.Fig ...
                ,'Title','','BorderType','none'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Radiobutton: Load from file
            Pos2(1) = 0;
            Pos2(2) = Pos(4) - GUIP.NormalTextHeight;
            Pos2(3) = Pos(3);
            Pos2(4) = GUIP.NormalTextHeight;
            hD.LoadRadiobutton = uicontrol(SubPanel,'Style','radiobutton'...
                ,'String','Load from an image stack file'...
                ,'Value',true ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos2);            
            % Radiobutton: Use self Image
%             Pos2(1) = 2*GUIP.LargePanelMargin(1);            
%             Pos2(3) = Pos(3)-2*GUIP.LargePanelMargin(1);
            Pos2(2) = Pos2(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            hD.UseSelfRadiobutton = uicontrol(SubPanel,'Style','radiobutton'...
                ,'String','Use self image as mask'...
                ,'Value',false ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos2);       
            % Popupmenu: channel selection
            Pos(1) = 4*GUIP.LargePanelMargin(1);
            Pos(3) = Pos(3) - 3*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2) - GUIP.NormalEditHeight;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Channel:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Offset;
            Pos(3) = Size(1)-GUIP.DialogMargin(1)-Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
%             ChannelList = cellstr(int2str((1:obj.Stack.DataSize(4))'));
            ChannelList = cellstr(int2str((1:obj.Stack.FullZCT(2))'));
            hD.ReferenceChannel = uicontrol(hD.Fig,'Style','popupmenu'...
                ,'String',ChannelList,'HorizontalAlignment','center'...
                ,'Value',1 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1],'Enable','off'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Popupmenu: Frames selection
            Pos(1) = 4*GUIP.LargePanelMargin(1);
            Pos(3) = Pos(3) - 3*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Frames:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Offset;
            Pos(3) = Size(1)-GUIP.DialogMargin(1)-Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            FrameList = [{'-All-'};cellstr(int2str((1:obj.Stack.DataSize(5))'))];
            hD.ReferenceFrame = uicontrol(hD.Fig,'Style','popupmenu'...
                ,'String',FrameList,'HorizontalAlignment','center'...
                ,'Value',1 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1],'Enable','off'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Buttons
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','close(gcbf);');
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Continue'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','uiresume(gcbf);');
            hD.LoadRadiobutton.Callback = @(Source,Event)Figure5D.AddMaskDialogCallback(hD,1);
            hD.UseSelfRadiobutton.Callback = @(Source,Event)Figure5D.AddMaskDialogCallback(hD,2);
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                if get(hD.UseSelfRadiobutton,'Value') % Use self
                    Channel = get(hD.ReferenceChannel,'Value');
                    Frame = get(hD.ReferenceFrame,'Value') - 1;
                    if Channel > obj.Stack.DataSize(4)
                        Channel = Channel - obj.Stack.DataSize(4);
                        MaxFrame = size(obj.Stack.Extra{Channel},4);
                        MaskStack = reshape(obj.Stack.Extra{Channel}...
                            ,[obj.Stack.DataSize(1:3),1,MaxFrame]);
                        if Frame > MaxFrame
                            Frame = MaxFrame;
                        end
                        Channel = 1;
                    else
                        MaskStack = obj.Stack.Data;
                    end
                    N = 1;
                    if Frame == 0
                        N = size(MaskStack,5);
                        Frame = 1:N;
                    end
                    obj.Stack.MaskFileName{MI} = fullfile(obj.Folder,obj.Filename);                    
                else % Load
                    [FileName,PathName] = uigetfile({'*.tif;*.tiff;*.btf','All Tiff files (*.tif,*.tiff,*.btf)';...
                        '*.tif;*.tiff','Normal Tiff files (*.tif,*.tiff)';...
                        '*.btf;*.tif','BigTiff files (*.btf,*.tif)';...
                        '*.*','All files (*.*)'} ...
                        ,'Open Mask (Tiff/BigTiff) file...',obj.Folder);
                    if isequal(FileName,0)
                        close(hD.Fig);
                        % Enable every figure
                        obj.EnableEverything(true);
                        return;
                    end;
                    obj.LogHistory.NewTask(' - Opening Tiff file');
                    MaskStack = Figure5D.LoadTiffStack(fullfile(PathName,FileName));
                    obj.LogHistory.FinishTask;
                    % Check compatibility
                    if ~isequal([size(MaskStack,1),size(MaskStack,2),size(MaskStack,3)],obj.Stack.DataSize(1:3))
                        h = msgbox('The mask does not match with the current image (3D size)!'...
                            ,'Error','error','modal');
                        uiwait(h);
                        close(hD.Fig);
                        % Enable every figure
                        obj.EnableEverything(true);
                        return;
                    end;
                    Channel = 1;
                    % Determination of Mask frames
                    N = size(MaskStack,5);
                    if N >= obj.Stack.DataSize(5),
                        N = obj.Stack.DataSize(5);
                    end;
                    Frame = 1:N;
                    obj.Stack.MaskFileName{MI} = fullfile(PathName,FileName);
                end
                obj.Stack.MaskName{MI} = get(hD.MaskNameEdit,'String');
                obj.LogHistory.NewTask(' - Extracting coordinates');
                obj.Stack.MaskList{MI} = cell(N,1);
                obj.Stack.MaskPoints{MI} = zeros(N,1);
                % Determination of mask points
                for n = 1:N
                    obj.Stack.MaskList{MI}{n} = find(MaskStack(:,:,:,Channel,Frame(n)));
                    obj.Stack.MaskPoints{MI}(n) = length(obj.Stack.MaskList{MI}{n});
                end
                % Inform the child images
                if InformChildImages
                    for n = 1:length(obj.ChildImages)
                        if isprop(obj.ChildImages{n},'NextUpdate')
                            obj.ChildImages{n}.NextUpdate = 1;
                        end
                    end
                end
                % Update GUI mask related options
                if MI == 1
                    set(obj.hToolbar(7),'Visible','on','Enable','on','Value',1);
                    obj.ShowMask = true;
                end
                % Close and update image
                close(hD.Fig);
                obj.ShowImage;
                obj.LogHistory.FinishAll;
            end;
            % Enable every figure
            obj.EnableEverything(true);
        end
        
        % -----------------------------------------------------------------
        % Remove masks
        function RemoveMaskCallback(obj,~,~)
            obj.Stack.MaskList{1} = {};
            set(obj.hToolbar(7),'Visible','on','Enable','on','Value',1); 
        end
        
        % =================================================================

        function ExtraChannelsCallback(obj,~,~,Option)
            % Callback for manipulation of extrachannels
            % -------------------------------------------------------------
            if nargin < 4
                Option = [];
            end;
            if isempty(Option)
                return
            end;
            switch Option(1:3)
                case 'add'
                    obj.Stack.AddExtraChannel;
                case 'rem'
                    % Security questions
                    if obj.Stack.CurrentZCT(2) <= obj.Stack.DataSize(4)
                        if obj.Stack.DataSize(4) > 1
                            Button = questdlg({'This is not an extra channel!';'Are you sure?'}...
                                ,'Main Channel remove...','No','Yes','No');
                            if ~strcmp(Button,'Yes')
                                return;
                            end;
                        else
                            % No extra channel
                            h = msgbox({'There is only one main channel';'and cannot be removed!'}...
                                ,'Channel not removed','error');
                            uiwait(h);
                            return;
                        end;
                    end;
                    % Channel Remove
                    obj.Stack.RemoveChannel;
                case 'all'
                    % Security questions
                    if obj.Stack.FullZCT(2) > obj.Stack.DataSize(4)
                        Button = questdlg({'The information of the extra channels will be lost!';'Are you sure?'}...
                            ,'Remove all extra channels...','No','Yes','No');
                        if ~strcmp(Button,'Yes')
                            return;
                        end;
                    end;
                    obj.Stack.RemoveAllExtraChannels;
            end;
        end;

        function ShowCompositeCallback(obj,~,~)  % (~,~,obj)

            obj.ShowComposite = ~obj.ShowComposite;
            if obj.ShowComposite
                set(obj.Menubar{4}{end}{1},'Label','Show single channels');
                % Adjusting figure controls:
                set(obj.hToolbar(5),'Enable','off');
                set(obj.hToolbar(9),'Enable','off');
                set(obj.hToolbar(11),'Enable','off');
                set(obj.hImage.LabelText(1),'String','Composite');
                set(obj.hImage.LabelText(2),'String','Composite');                
            else
                set(obj.Menubar{4}{end}{1},'Label','Show composite');
                % Adjusting figure controls:
                set(obj.hToolbar(5),'Enable','on');
                set(obj.hToolbar(9),'Enable','on');
                set(obj.hToolbar(11),'Enable','on');
            end
            obj.ShowImage;
        end;
        
        % =================================================================

        function ShowImage(obj)
            % Update Image in the axes
            % -------------------------------------------------------------
            % Show the image
            set(obj.hImage.Image,'CData',obj.Stack.CurrentImage(obj.ShowMask,obj.ShowComposite));
            CM = obj.Stack.CurrentZCT(2);
            if CM > obj.Stack.DataSize(4)
                ShowComp = false;
            else
                ShowComp = true;
            end
            ShowComp = ShowComp & obj.ShowComposite;
            if ~ShowComp
                % Adjust Colormap
                set(obj.hFig,'ColorMap',obj.Stack.ChannelMap{CM});
                % Adjusting the display range:
                set(obj.hImage.Axes,'CLim',obj.Stack.ChannelDisplayRange(CM,:));            
                % Adjusting figure controls:
                set(obj.hToolbar(5),'Value',obj.Stack.ChannelMapValue(CM));
                % Update Edit controls
                if isequal(obj.Stack.ChannelDisplayRange(CM,:) - floor(obj.Stack.ChannelDisplayRange(CM,:)),[0,0])
                    set(obj.hToolbar(9),'String',num2str(obj.Stack.ChannelDisplayRange(CM,1),'%d'));
                    set(obj.hToolbar(11),'String',num2str(obj.Stack.ChannelDisplayRange(CM,2),'%d'));
                else
                    set(obj.hToolbar(9),'String',num2str(obj.Stack.ChannelDisplayRange(CM,1),'%.2f'));
                    set(obj.hToolbar(11),'String',num2str(obj.Stack.ChannelDisplayRange(CM,2),'%.2f'));
                end
                set(obj.hImage.LabelText(1),'String',obj.Stack.ChannelXLabel{CM});
                set(obj.hImage.LabelText(2),'String',obj.Stack.ChannelYLabel{CM});            
            end
            % Update ROI
            obj.Roi.KeepUpdated = get(obj.hSidebar.TopElements(2),'Value');
            obj.Roi.Show2D = get(obj.hSidebar.TopElements(3),'Value');
            if obj.Roi.KeepUpdated && obj.Roi.Selected(1) ~= 0
                obj.ShowRoi;
            end;            
        end;
        
        function ShowRoi(obj,Force,Jump)
            % Show the selected Rois in the current image
            % INPUT ARGUMENTS
            % Force:        (logical) Force showing ROIs
            % Jump:         Jump to the image where the selection was done
            %               (only valid for 2D Rois)
            % -------------------------------------------------------------
            if nargin < 3
                Jump = false;
            end
            if nargin < 2
                Force = false;
            end
            obj.Roi.KeepUpdated = get(obj.hSidebar.TopElements(2),'Value');
            obj.Roi.Show2D = get(obj.hSidebar.TopElements(3),'Value');            
            if ~Force
                if obj.Roi.KeepUpdated && obj.Roi.Selected(1) ~= 0
                    Force = true;
                end;
            end;
            if Force
                if obj.Roi.Selected(1) == 0
                    set(obj.hImage.Roi,'AlphaData',zeros(obj.Stack.DataSize(1),obj.Stack.DataSize(2),'uint8'));
                else
                    AlphaData = false(obj.Stack.DataSize(1)+2,obj.Stack.DataSize(2)+2);
                    AlphaAux = false(obj.Stack.DataSize(1)+2,obj.Stack.DataSize(2)+2);
                    Z = obj.Stack.CurrentZCT(1);
                    for n = 1:length(obj.Roi.Selected)
                        AlphaAux(:) = false;
                        AIndex = obj.Roi.List(obj.Roi.Selected(n)).GetRoi2DIndexesZ(Z,obj.Stack.DataSize(1:3),obj.Roi.Show2D,true);
                        obj.Roi.List(obj.Roi.Selected(n)).LastZ = Z;
                        AlphaAux(AIndex) = true;
                        AlphaAux = AlphaAux - imerode(AlphaAux,ones(3));
                        AlphaData = AlphaData | AlphaAux;
                    end;
                    set(obj.hImage.Roi,'AlphaData',uint8(AlphaData(2:end-1,2:end-1))*255);
                    if Jump && (length(obj.Roi.Selected) == 1) && obj.Roi.List(obj.Roi.Selected(1)).Is2D ...
                            && obj.Roi.List(obj.Roi.Selected(1)).Slices(1) > 0 ...
                            && obj.Roi.List(obj.Roi.Selected(1)).Frames(1) > 0 ...
                            && obj.Roi.List(obj.Roi.Selected(1)).Channels(1) > 0
                        obj.Stack.CurrentZCT(1) = obj.Roi.List(obj.Roi.Selected(1)).Slices(1);
                        obj.Stack.CurrentZCT(2) = obj.Roi.List(obj.Roi.Selected(1)).Channels(1);
                        obj.Stack.CurrentZCT(3) = obj.Roi.List(obj.Roi.Selected(1)).Frames(1);
                        obj.SetImageSliders(true);
                    end
                end;
            end;
        end;
                
        % =================================================================

        function SetImageSliders(obj,UpdateImageOption)
            % Update the slider associated objects
            % -------------------------------------------------------------
            if nargin < 2
                UpdateImageOption = false;
            end;
            for n = 1:3
                if obj.Stack.FullZCT(n) > 1
                    set(obj.Slider(n),'Enable','on'...
                        ,'Max',obj.Stack.FullZCT(n)...
                        ,'SliderStep',1/(obj.Stack.FullZCT(n)-1)*[1 1] ...
                        ,'Min',1,'Value',obj.Stack.CurrentZCT(n));
                    set(obj.SliderEdit(n),'Enable','on');
                else
                    set(obj.Slider(n),'Enable','off'...
                        ,'Max',1 ...
                        ,'Min',0,'Value',1);
                    set(obj.SliderEdit(n),'Enable','off');
                end;
            end;
            if UpdateImageOption
                obj.ShowImage;
            end;
        end;
        
        
        % =================================================================

        function CreateGUI(obj)
            % CreateGUI
            % -------------------------------------------------------------
            % Read GUI Proportions
            if ~isempty(obj.hFig)
                return;
            end;
            GUIP = GetGUIProp('maingui');
            GUIS = GetGUIProp('guisizes');
            % ---------------------------------------------------------
            % Figure creation
            % ---------------------------------------------------------
            RootD = groot;
            Size = GUIS.FigureSize;
            OldUnits = get(RootD,'Units');
            set(RootD,'Units',GUIS.Units);
            SC = get(RootD,'ScreenSize');
            set(RootD,'Units',OldUnits);
            obj.hFig = figure('Visible','off'...
                ,'Name',obj.Name...
                ,'NumberTitle','off'...
                ,'MenuBar','none'...
                ,'Toolbar','none'...
                ,'CloseRequestFcn',@(Source,Event)obj.CloseFigure...
                ,'Resize','on'... 
                ,'SizeChangedFcn',@(Source,Event)obj.UpdateFigure(Source,Event)...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2),round((SC(4)-Size(2))/2),Size(1),Size(2)]...
                ,'Interruptible','off'...
                ,'UserData',[]...
                );
            BkgColor = get(obj.hFig,'Color');
            % -------------------------------------------------------------
            % Slider bars creation
            % -------------------------------------------------------------
            TextStrings = {'T:','Z:','C:'};
            SliderIndex = [3,1,2];
            for n = 1:3
                obj.SliderText(SliderIndex(n)) = uicontrol(obj.hFig,'Style','text'...
                    ,'String',TextStrings{n},'FontSize',GUIP.LargeFontSize ...
                    ,'FontWeight','bold','FontAngle','Italic' ...
                    ,'BackgroundColor',BkgColor,'Units','points'...
                    ,'Position',[0,(n-1)*GUIP.NormalEditHeight,GUIP.NormalEditHeight,GUIP.LargeTextHeight]);
                obj.SliderEdit(SliderIndex(n)) = uicontrol(obj.hFig,'Style','edit'...
                    ,'String','1','BackgroundColor','white'...
                    ,'FontSize',GUIP.NormalFontSize,'Enable','off'...
                    ,'Callback',@(Source,Event)obj.UpdateSliderEdit(Source,Event,SliderIndex(n))...
                    ,'Units','points' ...
                    ,'Position',[GUIP.NormalEditHeight,(n-1)*GUIP.NormalEditHeight,3*GUIP.NormalEditHeight,GUIP.NormalEditHeight] ...
                    );
                obj.Slider(SliderIndex(n)) = uicontrol(obj.hFig,'Style','slider'...
                    ,'Max',1,'Min',0,'Value',1   ...
                    ,'Enable','off'...
                    ,'Callback',@(Source,Event)obj.UpdateSlider(Source,Event,SliderIndex(n))...
                    ,'Units','points' ...
                    ,'Position',[4*GUIP.NormalEditHeight,(n-1)*GUIP.NormalEditHeight,Size(1)-4*GUIP.NormalEditHeight,GUIP.NormalEditHeight] ...
                    );
            end;
            % ---------------------------------------------------------
            % Image handles creation
            % ---------------------------------------------------------
            % Panel that have the image     Ex Im(1)
            obj.hImage.Panel = uipanel('Parent',obj.hFig ...
                ,'Units',GUIS.Units ...
                ,'Position',[GUIS.LabelHeight,GUIS.SlidersHeight + GUIS.LabelHeight ...
                            ,Size(1)-GUIS.LabelHeight-GUIS.SidebarWidth ...
                            ,Size(2)-GUIS.SlidersHeight-GUIS.LabelHeight-GUIS.ToolbarHeight]);
            % Image axes handle             Ex Im(2)
            obj.hImage.Axes = axes('Parent',obj.hImage.Panel...
                ,'Visible','off','Units','normalized'...
                ,'Position',[0,0,1,1]...
                ,'XTick',[],'YTick',[]);
            % Image handle                  Ex Im(3)
            obj.hImage.Image = imshow([],'Parent',obj.hImage.Axes);
            set(obj.hImage.Image,'CDataMapping','scaled');
            set(obj.hImage.Axes,'XTick',[],'YTick',[]);
            % Roi Image handle
            obj.hImage.Roi = image('Parent',obj.hImage.Axes...
                ,'XData',[1,0],'YData',[1,0],'CDataMapping','direct','CData',[]...
                ,'Interruptible','off');
            set(obj.hImage.Roi,'AlphaDataMapping','none','CDataMapping','scaled');
            set(obj.hImage.Axes,'XTick',[],'YTick',[]);
            % Axes for X Labels             Ex Im(4)
            obj.hImage.LabelAxes(1) = axes('Parent',obj.hFig...
                ,'XTick',[],'YTick',[]...
                ,'Units',GUIS.Units...
                ,'Position',[GUIS.LabelHeight,GUIS.SlidersHeight ...
                           ,Size(1)-GUIS.LabelHeight-GUIS.SidebarWidth ...
                           ,GUIS.LabelHeight] ...
                ,'Box','off'...
                ,'Color',BkgColor...
                ,'XColor',BkgColor...
                ,'YColor',BkgColor...
                );
            % Text for X Labels             Ex Im(5)
            obj.hImage.LabelText(1) = text('Parent',obj.hImage.LabelAxes(1)...
                ,'String',''...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','center','Rotation',0 ...
                ,'Units','normalized','Position',[0.5,0.5]...
                );
            % Axes for Y Labels             Ex Im(6)
            obj.hImage.LabelAxes(2) = axes('Parent',obj.hFig...
                ,'XTick',[],'YTick',[]...
                ,'Units',GUIS.Units...
                ,'Position',[0,GUIS.SlidersHeight+GUIS.LabelHeight ...
                           ,GUIS.LabelHeight ...
                            ,Size(2)-GUIS.SlidersHeight-GUIS.LabelHeight-GUIS.ToolbarHeight] ...
                ,'Box','off'...
                ,'Color',BkgColor...
                ,'XColor',BkgColor...
                ,'YColor',BkgColor...
                );
            % Text for Y Labels             Ex Im(7)
            obj.hImage.LabelText(2) = text('Parent',obj.hImage.LabelAxes(2)...
                ,'String',''...
                ,'FontSize',GUIP.LargeFontSize+1,'FontWeight','bold'...
                ,'HorizontalAlignment','center','Rotation',90 ...
                ,'Units','normalized','Position',[0.5,0.5]...
                );         
            % -------------------------------------------------------------
            % Sidebar creation
            % -------------------------------------------------------------
            GTB = GetGUIProp('toolbar');    
            % TopPanel
            obj.hSidebar.TopPanel = uipanel(obj.hFig,'Title','' ...
                ,'BorderType','none','BorderWidth',0 ...
                ,'Units','points','Visible','off'...
                );
            % TopElements
            obj.hSidebar.TopElements(1) = uicontrol(obj.hSidebar.TopPanel,'Style','text'...
                ,'String','ROI Manager','FontWeight','bold','FontAngle','Italic' ...
                ,'Units',GUIS.Units,'FontSize',GTB.FontSize ...
                ,'Position',[0,2*GUIS.SidebarYTabs,GUIS.SidebarWidth,GUIS.SidebarYTabs*0.78] ...
                );  
            obj.hSidebar.TopElements(2) = uicontrol(obj.hSidebar.TopPanel,'Style','checkbox'...
                ,'String','Keep updated','FontWeight','normal','FontAngle','normal' ...
                ,'Value',obj.Roi.KeepUpdated ...
                ,'Units',GUIS.Units,'FontSize',GTB.FontSize ...
                ,'Position',[0,GUIS.SidebarYTabs,GUIS.SidebarWidth,GUIS.SidebarYTabs] ...
                );  
            obj.hSidebar.TopElements(3) = uicontrol(obj.hSidebar.TopPanel,'Style','checkbox'...
                ,'String','2D selections','FontWeight','normal','FontAngle','normal' ...
                ,'Value',obj.Roi.Show2D ...
                ,'Units',GUIS.Units,'FontSize',GTB.FontSize ...
                ,'Position',[0,0,GUIS.SidebarWidth,GUIS.SidebarYTabs] ...
                );  
            PanelHeight = length(obj.hSidebar.TopElements)*GUIS.SidebarYTabs;
            set(obj.hSidebar.TopPanel,'Position'...
                ,[Size(1)-GUIS.SidebarWidth,Size(2)-PanelHeight...
                ,GUIS.SidebarWidth,PanelHeight] ...
                ,'Visible','on');
            % BottomPanel
            PanelHeight = Size(2)-GUIS.SlidersHeight-PanelHeight-GUIS.ToolbarHeight;
            obj.hSidebar.BottomPanel = uipanel(obj.hFig,'Title','' ...
                ,'BorderType','none','BorderWidth',0 ...
                ,'Units','points','Visible','off'...
                ,'Position',[Size(1)-GUIS.SidebarWidth,GUIS.SlidersHeight...
                ,GUIS.SidebarWidth,PanelHeight] ...
                );
            % Roi buttons
            TextStrings = {'Add','Delete','Properties','Select all','Measure'...
                ,'Operations','Draw','Transfer'};
            CallbackStrings = {'add','del','pro','all','mea','ope','dra','tra'};
            NE = length(TextStrings);
            for n = 1:length(TextStrings)
                obj.hSidebar.RoiButtons(n) = uicontrol(obj.hSidebar.BottomPanel,'Style','pushbutton'...
                    ,'String',TextStrings{n},'FontSize',GTB.FontSize ...
                    ,'Callback',@(Source,Event)obj.RoiManager(Source,Event,CallbackStrings(n))...
                    ,'Enable','off','Units',GUIS.Units...
                    ,'Position',[0,GUIS.SidebarYTabs*(NE-n),GUIS.SidebarWidth,GUIS.SidebarYTabs] ...
                    );            
            end;
            set(obj.hSidebar.RoiButtons(5),'TooltipString',...
                sprintf('Press "Ctrl" to consider only current image.\nPress "Shift" to exclude 2D selections.\nPress "Alt" to consider all frames.'));
            % Roi List
            obj.hSidebar.RoiList = uicontrol(obj.hSidebar.BottomPanel,'Style','listbox'...
                ,'String',{'None'},'FontSize',GUIP.SmallFontSize ...
                ,'Max',2,'Min',0,'Value',[] ...
                ,'Callback',@(Source,Event)obj.RoiManager(Source,Event,'')...
                ,'TooltipString',sprintf('Press "Alt" to jump to the original image position\nwhen selecting one Roi')...
                ,'Enable','off','Units',GUIS.Units...
                ,'Position',[0,GUIS.SidebarYTabs*NE...
                ,GUIS.SidebarWidth,PanelHeight-GUIS.SidebarYTabs*NE] ...
                );
            set(obj.hSidebar.BottomPanel,'Visible','on');            
            % -------------------------------------------------------------
            % Menu creation
            % -------------------------------------------------------------
            obj.CreateMenubar;
            % -------------------------------------------------------------
            % Toolbar creation
            % -------------------------------------------------------------
            obj.hToolbar(1) = uipanel(obj.hFig,'Title',''...
                ,'BorderType','none','BorderWidth',0 ...
                ,'Visible','off','Units','points'...
                ,'Position',[0,Size(2)+1-GUIS.ToolbarHeight,Size(1)-GUIS.SidebarWidth,GUIS.ToolbarHeight] ...
                );
            % Cursor Position
            obj.hToolbar(2) = impixelinfoval(obj.hToolbar(1),obj.hImage.Image);
            % Pan button            % Ex 4
            obj.hToolbar(3) = uicontrol(obj.hToolbar(1),'Style','togglebutton'...
                ,'String','Pan','Value',0 ...
                ,'Callback',@(Source,Event)obj.PanCallback(Source,Event)...
                ,'TooltipString','Press "Ctrl/Shift" for only horizontal/vertical pan'...
                );
            % Zoom button           % Ex 5
            obj.hToolbar(4) = uicontrol(obj.hToolbar(1),'Style','togglebutton'...
                ,'String','Zoom','Value',0 ...
                ,'Callback',@(Source,Event)obj.ZoomCallback(Source,Event)...
                ,'TooltipString','Press "Ctrl/Shift" for only horizontal/vertical zoom'...
                );
            % Colormap popup menu   % Ex 7
            obj.hToolbar(5) = uicontrol(obj.hToolbar(1),'Style','popupmenu'...
                ,'String',{'Gray','Red','Green','Blue','Cyan','Magenta','Yellow'...
                ,'Jet','Hot','Cool','Spring','Summer','Autumn','Winter'...
                ,'Bone','Cooper','Pink'}...
                ,'Value',1 ...
                ,'Callback',@(Source,Event)obj.ChannelMapCallback(Source,Event)...
                );
            % Mask checkbox         % Ex 9
            obj.hToolbar(6) = uicontrol(obj.hToolbar(1),'Style','checkbox'...
                ,'String','Stretch'...
                ,'Value',0 ...
                ,'Callback',@(Source,Event)obj.StretchCallback(Source,Event)...
                ,'TooltipString','Stretch image to fill the panel'...
                );
            % Stretch checkbox      % Ex 10
            obj.hToolbar(7) = uicontrol(obj.hToolbar(1),'Style','checkbox'...
                ,'String','Masked'...
                ,'Value',0 ...
                ,'Callback',@(Source,Event)obj.MaskedCallback(Source,Event)...
                ,'Enable','off'...
                ,'TooltipString','Show the masked image'...
                );
            % Min display range text    % Ex 12
            obj.hToolbar(8) = uicontrol(obj.hToolbar(1),'Style','text'...
                ,'String','Min:');
            % Min display range edit    % Ex 13
            obj.hToolbar(9) = uicontrol(obj.hToolbar(1),'Style','edit'...
                ,'String','0' ...
                ,'BackgroundColor','white'...
                ,'Callback',@(Source,Event)obj.DisplayRangeCallback(Source,Event,1)...
                ,'TooltipString','Low limit of the channel display range'...
                );
            % Max display range text    % Ex 14
            obj.hToolbar(10) = uicontrol(obj.hToolbar(1),'Style','text'...
                ,'String','Max:');
            % Max display range edit    % Ex 15
            obj.hToolbar(11) = uicontrol(obj.hToolbar(1),'Style','edit'...
                ,'String','1' ...
                ,'BackgroundColor','white'...
                ,'Callback',@(Source,Event)obj.DisplayRangeCallback(Source,Event,2)...
                ,'TooltipString','High limit of the channel display range'...
                );
            % Auto Display range button % Ex 16
            obj.hToolbar(12) = uicontrol(obj.hToolbar(1),'Style','pushbutton'...
                ,'String','Auto' ...
                ,'Callback',@(Source,Event)obj.DisplayRangeCallback(Source,Event,3)...
                ,'TooltipString',sprintf('Auto-adjust the display range.\nPress "Ctrl" to consider all slices.\nPress "Shift" to consider all time points.')...
                );
            % Reset Display range button % Ex 17
            obj.hToolbar(13) = uicontrol(obj.hToolbar(1),'Style','pushbutton'...
                ,'String','Reset' ...
                ,'Callback',@(Source,Event)obj.DisplayRangeCallback(Source,Event,4)...
                ,'TooltipString','Reset the display range to the full-range'...
                );
            % Selection popup menu   % Ex 7
            obj.hToolbar(14) = uicontrol(obj.hToolbar(1),'Style','popupmenu'...
                ,'String',{'Point','Line','Rect','Ellipse','Polygon','Free hand'}...
                ,'Callback',@(Source,Event)obj.SelectionCallback(Source,Event,'',false)...
                ,'Value',1 ... 
                );
            % Reset Display range button % Ex 17
            obj.hToolbar(15) = uicontrol(obj.hToolbar(1),'Style','pushbutton'...
                ,'String','Select','Value',0 ...
                ,'TooltipString','Press "Ctrl" to deselect'...
                ,'Callback',@(Source,Event)obj.SelectionCallback(Source,Event,'',true)...
                );
            % Adjust positions
            for n = 1:length(GTB.Widths)-1
                Height = GTB.Height;
                Bottom = GTB.Margin(2);
                Left = GTB.Margin(1);
                Width = GTB.Widths(n+1);
                switch n
                    case {1,7,9}
                        Height = Height * GTB.TextHeightFactor;
                    case {4,13}
                        Bottom = Bottom*GTB.PopupMarginFactor;
                    case 5
                        Left = Left*GTB.CheckboxMarginFactor;
                        Width = Width - (Left-GTB.Margin(1));
                end;
                set(obj.hToolbar(n+1),'Units',GTB.Units ...
                    ,'FontSize',GTB.FontSize ...
                    ,'Position',[sum(GTB.Widths(1:n))+n*Left,Bottom,Width,Height] ...
                    );
            end;
%             set(obj.hToolbar(1),'Visible','on');
            % ---------------------------------------------------------
            obj.InitializeFigure;

            Pos = obj.hFig.OuterPosition;
            obj.hFig.Units = 'normalized';
            obj.hFig.OuterPosition = [0,0,1,1];
            obj.hFig.Units = GUIP.Units;
            obj.hFig.UserData = min(0.8*obj.hFig.Position(3:4),Size);
            obj.hFig.OuterPosition = Pos; 
            obj.hFig.Visible = 'on';
            % Determine the JAVA handle of the figure
            obj.GetJavaHandle;
            
%             % Minimum figure size:
%             drawnow;
%             jFrame = get(handle(obj.hFig),'JavaFrame');
%             try     % R2008a and later
%                 jClient = jFrame.fHG1Client;
%             catch   % R2007b and earlier
%                 jClient = jFrame.fFigureClient;
%             end;
%             jWindow = jClient.getWindow;
%             if isempty(jWindow)
%                 drawnow;
%                 pause(0.02);
%                 jWindow = jClient.getWindow;
%             end;
%             jWindow.setMinimumSize(java.awt.Dimension(Size(1),Size(2)));

%             jAxis = jFrame.getAxisComponent;
%             set(jAxis,'ComponentResizedCallback',{@UpdateFigure,obj});

        end;

        % =================================================================

        function MenuStruct = DefineMenubar(obj)
            % Define standard menubar structure
            % OUTPUT ARGUMENTS
            % Struct to create the maps
            % -------------------------------------------------------------
            % Define standard menu
            % -------------------------------------------------------------
            MenuStruct = struct(...
                'Label',{{'File';'Selection';'Image'}}...
                ,'Separator',   false(3,1) ...
                ,'Enable',      logical([1,0,0]') ...
                ,'Callback',    {cell(3,1)} ...
                ,'hMenu',       zeros(3,1)...
                );
            % File menu
            MenuStruct.Callback{1} = struct(...
                'Label',{{'Open';'Save';'Quit'}}...
                ,'Separator',   logical([0,0,1]') ...
                ,'Enable',      logical([1,0,1]') ...
                ,'Callback',    {{{@obj.OpenImage,''} ...
                                ;{@obj.SaveFigure} ...
                                ;{@obj.CloseFigure}}} ...
                ,'hMenu',       zeros(3,1)...
                );
            % Selection menu
            MenuStruct.Callback{2} = struct(...
                'Label',       {{'Make selection...'  ... %1
                                ;'Clear selection'    ...
                                ;'Repeat selection'   ...
                                ;'Add to ROI Manager' ...
                                ;'Delete ROI'         ...
                                ;'ROI properties...'  ... %6
                                ;'Select/Deselect all ROIs'    ...
                                ;'Measure...'   ...
                                ;'ROI operations...'  ...
                                ;'Draw ROI...'        ...
                                ;'Transfer ROI...'    ... %11
                                ;'Load ROIs...'       ...
                                ;'Save ROIs...'       ...
                                ;'ROI color'          ...
                                }}...
                ,'Separator',   logical([0,0,0,0,1, 0,0,0,0,0, 1,1,0,1]') ...
                ,'Enable',      logical([1,0,1,0,0, 0,1,1,0,0, 0,1,0,1]') ...
                ,'Callback',    {{{}            ...
                                ;{@obj.SelectionCallback,'cle'} ...
                                ;{@obj.SelectionCallback,'',true} ...
                                ;{@obj.RoiManager,'add'} ...
                                ;{@obj.RoiManager,'del'} ...
                                ;{@obj.RoiManager,'pro'} ... %6
                                ;{@obj.RoiManager,'all'} ...
                                ;{@obj.RoiManager,'mea'} ...
                                ;{@obj.RoiManager,'ope'} ...
                                ;{@obj.RoiManager,'dra'} ...
                                ;{@obj.RoiManager,'tra'} ... %10
                                ;{@obj.RoiManager,'loa'} ...
                                ;{@obj.RoiManager,'sav'} ...
                                ;{@obj.RoiManager,'col'} ...
                                }}...
                ,'hMenu',       zeros(14,1)                 ...
                );
            % Image menu
            MenuStruct.Callback{3} = struct(...
                'Label',       {{'Voxel size...'  ... %1
                                ;'Channel background value...'     ...
                                ;'Add Mask 1 (object/surface)' ...
                                ;'Add Mask 2 (reference)' ...
                                ;'Add extra channel'       ...
                                ;'Remove current channel'  ...
                                ;'Remove all extra channels'...
                                ;'Show Composite'...
                                }}...
                ,'Separator',   logical([0,1,1,0,1,0,0,1]') ...
                ,'Enable',      logical([1,1,1,1,1,1,1,1]') ...
                ,'Callback',    {{{@obj.VoxelSizeDialog}             ...
                                ;{@obj.BackgroundDialog}             ...
                                ;{@obj.AddMaskCallback,1}  ...
                                ;{@obj.AddMaskCallback,2}  ...
                                ;{@obj.ExtraChannelsCallback,'add'}  ...
                                ;{@obj.ExtraChannelsCallback,'rem'}  ...
                                ;{@obj.ExtraChannelsCallback,'all'}  ...
                                ;{@obj.ShowCompositeCallback}  ...
                                }}...
                ,'hMenu',       zeros(8,1) ...
                );
            % Selection submenu
            MenuStruct.Callback{2}.Callback{1} = struct(...
                'Label',       {{'Point'  ... %1
                                ;'Line'    ...
                                ;'Rect'   ...
                                ;'Ellipse' ...
                                ;'Polygon'         ...
                                ;'Free hand'  ... %6
                                }}...
                ,'Separator',   logical([0,0,0,0,0,0]') ...
                ,'Enable',      logical([1,1,1,1,1,1]') ...
                ,'Callback',    {{{@obj.SelectionCallback,'poi'} ...
                                ;{@obj.SelectionCallback,'lin'} ...
                                ;{@obj.SelectionCallback,'rec'} ...
                                ;{@obj.SelectionCallback,'ell'} ...
                                ;{@obj.SelectionCallback,'pol'} ...
                                ;{@obj.SelectionCallback,'fre'} ...
                                }}...
                ,'hMenu',       zeros(6,1)                 ...
                );
        end;
        
        
        function CreateMenubar(obj,MenuStruct)
            % Create MenuBar
            % INPUT ARGUMENTS
            % MenuStruct:       Structure of the menubar
            % -------------------------------------------------------------
            if nargin < 2
                MenuStruct = [];
            end;
            if isempty(MenuStruct)
                MenuStruct = obj.DefineMenubar;
            end;
            % -------------------------------------------------------------
            % Delete any existing menu
            % -------------------------------------------------------------
            if ~isempty(obj.Menubar)
                for n = 1:length(obj.Menubar)
                    for m = 1:length(obj.Menubar{n})
                        if ~iscell(obj.Menubar{n}{m})
                            delete(obj.Menubar{n}{m})
                        end;
                    end;
                end;
            end;
            % -------------------------------------------------------------
            % Create menu (recursive function)
            % -------------------------------------------------------------
            obj.Menubar = AddMenu(MenuStruct,obj.hFig);
        end;
        
        % =================================================================

        function UpdateFigure(obj,~,~)
            % Update the content of the figure
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIS = GetGUIProp('guisizes');
            % -------------------------------------------------------------
            % Figure size update
            % -------------------------------------------------------------            
            FigH = gcbo;
            OldUnits = FigH.Units;
            FigH.Units = GUIS.Units;
            
%             if isempty(FigH.UserData)
%                 pause(0.01);
%                 jFrame = get(handle(obj.hFig),'JavaFrame');
%                 set(jFrame,'Maximized',true);
%                
%                 FigH.UserData = FigH.Position(3:4);
%                 if FigH.Position(3) < GUIS.FigureSize(1)
%                     FigH.UserData(1) = FigH.Position(3);
%                 end
%                 if FigH.Position(4) < GUIS.FigureSize(2)
%                     FigH.UserData(2) = FigH.Position(4);
%                 end
%             end        
            
%             OldUnits = get(FigH,'Units');
%             set(FigH,'Units',GUIS.Units);
%             Pos = get(FigH,'Position');
%             if Pos(3) < GUIS.FigureSize(1)
%                 Pos(3) = GUIS.FigureSize(1);
%             end;
%             if Pos(4) < GUIS.FigureSize(2)
%                 Pos(4) = GUIS.FigureSize(2);
%             end;
%             Size = Pos(3:4);
%             set(FigH,'Position',Pos);
            if FigH.Position(3) < FigH.UserData(1)
                FigH.Position(3) = FigH.UserData(1);
            end;
            if FigH.Position(4) < FigH.UserData(2)
                FigH.Position(4) = FigH.UserData(2);
            end;
            Size = FigH.Position(3:4);
%             set(FigH,'Units',OldUnits);
            FigH.Units = OldUnits;
            % -------------------------------------------------------------
            % Sliders update
            % -------------------------------------------------------------
            for n = 1:3
                Pos = get(obj.Slider(n),'Position');
                Pos(3) = Size(1)-Pos(1);
                set(obj.Slider(n),'Position',Pos);
            end;
            % -------------------------------------------------------------
            % Image handles
            % -------------------------------------------------------------   
            % Panel that have the image     Ex Im(1)
            set(obj.hImage.Panel ...
                ,'Position',[GUIS.LabelHeight,GUIS.SlidersHeight + GUIS.LabelHeight ...
                            ,Size(1)-GUIS.LabelHeight-GUIS.SidebarWidth ...
                            ,Size(2)-GUIS.SlidersHeight-GUIS.LabelHeight-GUIS.ToolbarHeight]);
            % Axes for X Labels             Ex Im(4)
            set(obj.hImage.LabelAxes(1) ...
                ,'Position',[GUIS.LabelHeight,GUIS.SlidersHeight ...
                           ,Size(1)-GUIS.LabelHeight-GUIS.SidebarWidth ...
                           ,GUIS.LabelHeight]);
            % Axes for Y Labels             Ex Im(6)
            set(obj.hImage.LabelAxes(2) ...
                ,'Position',[0,GUIS.SlidersHeight+GUIS.LabelHeight ...
                           ,GUIS.LabelHeight ...
                            ,Size(2)-GUIS.SlidersHeight-GUIS.LabelHeight-GUIS.ToolbarHeight]);
            % ---------------------------------------------------------
            % Sidebar update
            % ---------------------------------------------------------
            % Top Panel
            Pos = get(obj.hSidebar.TopPanel,'Position');
            Pos(1) = Size(1)-GUIS.SidebarWidth;
            Pos(2) = Size(2)-Pos(4);  % GUIS.ToolbarHeight-Pos(4);
            set(obj.hSidebar.TopPanel,'Position',Pos);
            % Bottom Panel
            set(obj.hSidebar.BottomPanel,'Position'...
                ,[Size(1)-GUIS.SidebarWidth,GUIS.SlidersHeight...
                ,GUIS.SidebarWidth,Pos(2)-GUIS.SlidersHeight]);
            % ROI List
            set(obj.hSidebar.RoiList,'Position',...
                [0,GUIS.SidebarYTabs*length(obj.hSidebar.RoiButtons)...
                ,GUIS.SidebarWidth,Pos(2)-GUIS.SlidersHeight-GUIS.SidebarYTabs*length(obj.hSidebar.RoiButtons)] ...
                );
            % ---------------------------------------------------------
            % Toolbar update
            % ---------------------------------------------------------
            set(obj.hToolbar(1),'Position' ...
                ,[0,Size(2)+1-GUIS.ToolbarHeight,Size(1)-GUIS.SidebarWidth,GUIS.ToolbarHeight]);
        end;        
        
        % =================================================================

        function UpdateSlider(obj,Source,~,SliderNumber)
            % Update the slider associated objects
            % -------------------------------------------------------------
            Val = round(get(Source,'Value'));
            if Val ~= obj.Stack.CurrentZCT(SliderNumber)
                obj.Stack.CurrentZCT(SliderNumber) = Val;
                set(Source,'Value',Val);
                set(obj.SliderEdit(SliderNumber),'String',num2str(Val));
                obj.ShowImage;
            end;
        end;
        
        % =================================================================

        function UpdateSliderEdit(obj,Source,~,SliderNumber)
            % Update the sliderEdit associated objects
            % -------------------------------------------------------------
            Num = round(str2double(get(Source,'String')));
            if isfinite(Num) && ...
                    Num >= get(obj.Slider(SliderNumber),'Min') && Num <= get(obj.Slider(SliderNumber),'Max')
                obj.Stack.CurrentZCT(SliderNumber) = Num;
                set(Source,'String',num2str(Num,'%u'));
                set(obj.Slider(SliderNumber),'Value',Num);
                obj.ShowImage;
            else
                set(Source,'String',num2str(obj.Stack.CurrentZCT(SliderNumber),'%u'));
            end;
        end;
      
        % =================================================================
        
        function SelectionCallback(obj,~,~,Type,SelectButton)
            % Make a selection over the image
            % INPUT ARGUMENTS
            % Type:         Selection type: 'Point','Line','Rect','Ellipse',
            %                   'Poligon','Free hand',
            %                   'Clear' (clear current selection)
            %               if Type is empty or not defined then it reads from the
            %                   toolbar
            % SelectButton: Is this a callback from the selection button?
            % -------------------------------------------------------------
            if nargin < 5
                SelectButton = false;
            end;
            if nargin < 4
                Type = '';
            end;
            if isempty(obj.Stack.Data)
                return;
            end
            % Check for an already active selection and delete it
            if ~isempty(obj.CurrentSelection)
                delete(obj.CurrentSelection);
                pause(0.1);
            elseif ~isempty(obj.CurrentSelectionType)
                return;
            end;
            % Check if callback comes from the button + key
            if SelectButton
                Keys = get(obj.hFig,'CurrentModifier');
                if ismember('control',Keys)
                    Type = 'cle';
                else
                    Type = '';
                end;
            end;
            % Read the type from the Popup Selection menu in case is empty
            if isempty(Type)
                Val = get(obj.hToolbar(14),'Value');
                S = get(obj.hToolbar(14),'String');
                Type = lower(S{Val}(1:3));
            end;
            % Disable Menu, toolbar, and Roi Manager
            for n = 2:length(obj.Menubar)
                set(obj.Menubar{n}{1},'Enable','off');
            end
            ToolbarDisable = [3,4,5,6,14,15];
            set(obj.hToolbar(ToolbarDisable),'Enable','off');
            if strcmp(get(obj.hSidebar.RoiButtons(2),'Enable'),'on')
                for n = [2,3,5,6,7,8]
                    set(obj.hSidebar.RoiButtons(n),'Enable','off');
                end;
                EnableRoiButtons = true;
            else
                EnableRoiButtons = false;
            end
            % Make selection
            switch Type
                case 'poi'
                    obj.CurrentSelectionType = 'impoint';
                    obj.CurrentSelection = impoint(obj.hImage.Axes);
                    Val = 1;
                case 'lin'
                    obj.CurrentSelectionType = 'imline';
                    obj.CurrentSelection = imline(obj.hImage.Axes);
                    Val = 2;
                case 'rec'
                    obj.CurrentSelectionType = 'imrect';
                    obj.CurrentSelection = imrect(obj.hImage.Axes);
                    Val = 3;
                case 'ell' 
                    obj.CurrentSelectionType = 'imellipse';
                    obj.CurrentSelection = imellipse(obj.hImage.Axes);
                    Val = 4;
                case 'pol'
                    obj.CurrentSelectionType = 'impoly';
                    obj.CurrentSelection = impoly(obj.hImage.Axes);
                    Val = 5;
                case 'fre'
                    obj.CurrentSelectionType = 'imfreehand';
                    obj.CurrentSelection = imfreehand(obj.hImage.Axes);
                    Val = 6;
                otherwise
                    obj.CurrentSelectionType = '';
                    obj.CurrentSelection = [];
                    % Enable Menu, toolbar & Roi Manager
                    for n = 2:length(obj.Menubar)
                        set(obj.Menubar{n}{1},'Enable','on');
                    end
                    set(obj.hToolbar(ToolbarDisable),'Enable','on');
                    if EnableRoiButtons
                        for n = [2,3,5,6,7,8]
                            set(obj.hSidebar.RoiButtons(n),'Enable','on');
                        end
                    end
                    % Disable "Clear selection"
                    set(obj.Menubar{1+2}{1+2}{1},'Enable','off');
                    % Disable "Add to Roi Manager"
                    set(obj.hSidebar.RoiButtons(1),'Enable','off');
                    set(obj.Menubar{1+2}{1+4}{1},'Enable','off');
                    return
            end;
            % Enable Menu, toolbar & Roi Manager
            for n = 2:length(obj.Menubar)
                set(obj.Menubar{n}{1},'Enable','on');
            end
            set(obj.hToolbar(ToolbarDisable),'enable','on');
            if EnableRoiButtons
                for n = [2,3,5,6,7,8]
                    set(obj.hSidebar.RoiButtons(n),'Enable','on');
                end
            end
            % Update selection popup in toolbar
            set(obj.hToolbar(14),'Value',Val);
            % Enable "Clear selection"
            set(obj.Menubar{1+2}{1+2}{1},'Enable','on');
            % Enable "Add to Roi Manager"
            set(obj.hSidebar.RoiButtons(1),'Enable','on');
            set(obj.Menubar{1+2}{1+4}{1},'Enable','on');
            % -------------------------------------------------------------
            % Limitation of the dragging to the Axes
            % -------------------------------------------------------------
            Fcn = makeConstrainToRectFcn(obj.CurrentSelectionType...
                ,get(obj.hImage.Axes,'XLim'),get(obj.hImage.Axes,'YLim'));
            api = iptgetapi(obj.CurrentSelection);
            api.setPositionConstraintFcn(Fcn);
        end;    
        
        function ListenerCloseCallback(obj,~,~,Option)
            % Listener callback for closing events
            % -------------------------------------------------------------
            switch Option
                case 'DataTable'
                    obj.ChildDataTable = [];
                    delete(obj.ListenerDataTable);
                    obj.ListenerDataTable = [];
            end;
        end;        
        
        % =================================================================

        function RoiManager(obj,~,~,Option)
            % Make a selection over the image
            % -------------------------------------------------------------
            Jump = false;
            switch char(Option)
                case ''   % ROI Selection callback
                    V = get(obj.hSidebar.RoiList,'Value')-1;
                    if isempty(V) || (V(1) == 0) || (length(V) == 1) && isequal(V,obj.Roi.Selected)
                        % Remove selection
                        set(obj.hSidebar.RoiList,'Value',1);
                        obj.Roi.Selected = 0;
                        % Disable ROI Menu buttons & Menu
                        for n = 2:length(obj.hSidebar.RoiButtons)
                            set(obj.hSidebar.RoiButtons(n),'Enable','off');
                        end;
                        for n = [5,6,7,8,9,10,11,13]
                            set(obj.Menubar{1+2}{1+n}{1},'Enable','off');
                        end
                        if ~isempty(obj.Roi.List)
                            set(obj.Menubar{1+2}{1+7}{1},'Enable','on');
                            set(obj.hSidebar.RoiButtons(4),'Enable','on');
                        end
                    else
                        obj.Roi.Selected = V;
                        % Read list of keys (Ctrl, Shift, Alt)
                        Keys = get(obj.hFig,'CurrentModifier');
                        % Check Keys
                        if ismember('alt',Keys)
                            Jump = true;
                        end
                        % Enable ROI Menu buttons & Menu
                        for n = 2:length(obj.hSidebar.RoiButtons)
                            set(obj.hSidebar.RoiButtons(n),'Enable','on');
                        end;
                        for n = [5,6,7,8,9,10,11,13]
                            set(obj.Menubar{1+2}{1+n}{1},'Enable','on');
                        end

                    end;
                    if ~isempty(obj.CurrentSelection)
                        obj.SelectionCallback([],[],'cle');
                    end;
                    obj.Roi.SelectAll = false;
                case 'add'   % Add current ROI
                    % Create ImageRoi object
                    if isempty(obj.Roi.List)
                        N = 1;
                        obj.Roi.List(1) = ImageRoi(obj.Stack.DataSize(1:3));
                    else
                        N = length(obj.Roi.List)+1;
                        obj.Roi.List(N) = ImageRoi(obj.Stack.DataSize(1:3));
                    end;
                    % Add Roi information to the ImageRoi
%                     AddSelection(obj.Roi.List(N),obj.hImage.Roi,obj.CurrentSelection,obj.Stack.DataSize(3),obj.Stack.CurrentZCT);
                    obj.Roi.List(N) = AddSelection(obj.Roi.List(N),obj.hImage.Roi,obj.CurrentSelection,obj.Stack.CurrentZCT);
                    % Remove imRoi selection
                    obj.SelectionCallback([],[],'cle');
                    % RoiName and addition to the Sidebar.Roi.List
                    RoiManager_Update(obj);
                    % Activate current selection
                    obj.Roi.Selected = N;
                    obj.Roi.SelectAll = false;
                    set(obj.hSidebar.RoiList,'Value',obj.Roi.Selected+1);
                    % Enable ROI Menu buttons & Menu 
                    for n = 2:length(obj.hSidebar.RoiButtons)
                        set(obj.hSidebar.RoiButtons(n),'Enable','on');
                    end;
                    for n = [5,6,7,8,9,10,11,13]
                        set(obj.Menubar{1+2}{1+n}{1},'Enable','on');
                    end
                case 'del'  % Delete current ROI
                    RIndex = get(obj.hSidebar.RoiList,'Value');
                    if RIndex(1) ~= 1
                        for n = length(RIndex):-1:1
%                             % Delete ImageROI objects
%                             delete(obj.Roi.List(RIndex(n)-1)); % Not a handle class anymore
                            % Delete Roi.List items
                            obj.Roi.List = [obj.Roi.List(1:RIndex(n)-2) obj.Roi.List(RIndex(n):end)];
                        end;
                        % Disable Roi Manager menu & buttons
                        for n = 2:length(obj.hSidebar.RoiButtons)
                            set(obj.hSidebar.RoiButtons(n),'Enable','off');
                        end;
                        for n = [5,6,7,8,9,10,11,13]
                            set(obj.Menubar{1+2}{1+n}{1},'Enable','off');
                        end
                        if ~isempty(obj.Roi.List)
                            set(obj.Menubar{1+2}{1+7}{1},'Enable','on');
                            set(obj.hSidebar.RoiButtons(4),'Enable','on');
                        end
                    end;
                    % RoiName and addition to the Sidebar.Roi.List
                    obj.Roi.SelectAll = false;
                    obj.Roi.Selected = 0;
                    RoiManager_Update(obj);                    
                case 'pro'  % Properties
                    % Dialog
                    RoiManager_Properties(obj);
                case 'all'  % Select All
                    if obj.Roi.SelectAll
                        obj.Roi.Selected = 0;
                        set(obj.hSidebar.RoiList,'Value',1);
                        obj.Roi.SelectAll = false;
                        % Disable Roi Manager menu & buttons
                        for n = [2,3,5,6,7,8]
                            set(obj.hSidebar.RoiButtons(n),'Enable','off');
                        end;
                        for n = [5,6,8,9,10,11,13]
                            set(obj.Menubar{1+2}{1+n}{1},'Enable','off');
                        end                        
                    else
                        N = length(obj.Roi.List);
                        obj.Roi.Selected = 1:N;
                        set(obj.hSidebar.RoiList,'Value',2:N+1);
                        obj.Roi.SelectAll = true;
                        % Enable Roi Manager menu & buttons
                        for n = [2,3,5,6,7,8]
                            set(obj.hSidebar.RoiButtons(n),'Enable','on');
                        end;
                        for n = [5,6,8,9,10,11,13]
                            set(obj.Menubar{1+2}{1+n}{1},'Enable','on');
                        end                        
                    end;
                case 'mea'  % Measure
                    % Read list of keys (Ctrl, Shift, Alt)
                    Keys = get(obj.hFig,'CurrentModifier');
                    % Check Keys
                    if ismember('control',Keys)
                        OnlyCurrentImage = true;
                    else
                        OnlyCurrentImage = false;
                    end;
                    if ismember('shift',Keys)
                        ExcludeSelections = true;
                    else
                        ExcludeSelections = false;
                    end;
                    if ismember('alt',Keys)
                        AllFrames = true;
                    else
                        AllFrames = false;
                    end;
                    % Create Table GUI
                    if isempty(obj.ChildDataTable)
                        FigName = strcat(obj.Name,' - Measurements');
                        obj.ChildDataTable = DataTable(FigName,'Measurements');
                        obj.ChildDataTable.Folder = obj.Folder;
                        obj.ListenerDataTable = addlistener(obj.ChildDataTable,'ClosingTable',...
                            @(Source,Event)obj.ListenerCloseCallback(Source,Event,'DataTable'));
%                                   function ListenerCloseCallback(~,~,obj,Option)                        
                    end;
                    % Perform measurements
                    RoiManager_Measure(obj,OnlyCurrentImage,ExcludeSelections,AllFrames);
                case 'ope'  % Operations
                    % Dialog
                    if length(obj.Roi.Selected) > 1
                        RoiManager_Operations(obj);
                        obj.Roi.SelectAll = false;
                    else
                        h = msgbox('At least 2 ROIs must be selected!'...
                            ,'Error','error','modal');
                        uiwait(h);
                        return;
                    end
                case 'dra'  % Draw
                    % Dialog
                    RoiManager_Draw(obj);
                case 'tra'  % Transfer
                    % Dialog
                    RoiManager_Transfer(obj);
                case 'loa'  % Load
                    RoiManager_Load(obj);
                case 'sav'  % Save
                    RoiManager_Save(obj);
                case 'col'  % Color
                    % Dialog
                    RoiManager_ColorDialog(obj);
            end;
            obj.ShowRoi(true,Jump);
        end;

        function RoiManager_Update(obj)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Create ImageRoi object
            if isempty(obj.Roi.List)
                set(obj.hSidebar.RoiList,'String','None'...
                    ,'Value',1,'Enable','off');
            else
                Val = get(obj.hSidebar.RoiList,'Value');
                N = length(obj.Roi.List);
                if max(Val) > N+1
                    obj.Roi.Selected = 0;
                    set(obj.hSidebar.RoiList,'Value',1);
                end;
                S = cell(N+1,1);
                S{1} = 'None';
                for n = 1:N
                    S{n+1} = obj.Roi.List(n).RoiName;
                end;
                set(obj.hSidebar.RoiList,'String',S,'Enable','on');
                %                         obj.Roi.List(N).ItemInsertAfter(obj.Roi.List(N-1));
            end;
        end;
        
        
        function RoiManager_Properties(obj)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 3*GUIP.NormalEditHeight;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','ROI properties...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );    
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            YT = Size(2) - GUIP.DialogMargin(2);
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Roi name:' ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',[GUIP.DialogMargin(1),YT-GUIP.NormalEditHeight...
                    ,Size(1)-2*GUIP.DialogMargin(1),GUIP.NormalTextHeight]);
            % Edits:
            V = get(obj.hSidebar.RoiList,'Value')-1;
%             S = get(obj.hSidebar.RoiList,'String');
            hD.Edit = uicontrol(hD.Fig,'Style','edit'...
                ,'String',obj.Roi.List(V(1)).RoiName ...
                ,'BackgroundColor','white'...
                ,'HorizontalAlignment','left'...
                ,'Enable','on'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',[GUIP.DialogMargin(1),YT - 2*GUIP.NormalEditHeight ...
                    ,Size(1)-2*GUIP.DialogMargin(1),GUIP.NormalEditHeight]);
            % Buttons
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);            
            PosE(2) = GUIP.DialogMargin(2);
            PosE(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','close(gcbf);');
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
%                 S{V(1)} = get(hD.Edit,'String');
                obj.Roi.List(V(1)).RoiName = get(hD.Edit,'String');
%                 RoiManager_Update([],[],obj);
                RoiManager_Update(obj);
%                 set(obj.hSidebar.RoiList,'String',S);
                close(hD.Fig);
            end;
            % Enable every figure
            obj.EnableEverything(true);            
        end;        
 
        function RoiManager_Measure(obj,OnlyCurrent2D,AvoidSelections,AllFrames)
            % Perform measurements on the selected ROIS
            % Version: 0.7 (10.11.2014)
            % INPUT ARGUMENTS
            % OnlyCurrent2D:    Only measure the current image in the stack?
            % AvoidSelections:  Omit measurements on selections
            % AllFrames:        Perform the measurements on every frame
            % -------------------------------------------------------------
            % Offset calculation
            AvoidSelections = ~AvoidSelections;
            Data = cell(1,15);
            Data{2} = obj.Stack.CurrentZCT(2);
            if AllFrames
                MinT = 1;
                MaxT = obj.Stack.FullZCT(3);
            else
                MinT = obj.Stack.CurrentZCT(3);
                MaxT = MinT;
            end
            Z = 0;
            if obj.Roi.Selected(1) == 0
                for t = MinT:MaxT
                    Data{3} = t;
                    if OnlyCurrent2D
                        PlaneIndexLimit = obj.Stack.DataSize(1)*obj.Stack.DataSize(2)...
                            *[obj.Stack.CurrentZCT(1)-1,obj.Stack.CurrentZCT(1)];
                        if obj.ShowMask
                            First = find(obj.Stack.MaskList{1}{Data{3}} > PlaneIndexLimit(1),1,'first');
                            Last = find(obj.Stack.MaskList{1}{Data{3}} < (PlaneIndexLimit(2)+1),1,'Last');
                            Indexes = obj.Stack.MaskList{1}{Data{3}}(First:Last);
                            Data{1} = 'Image-Mask-Z';
                        else
                            Indexes = PlaneIndexLimit(1)+1:PlaneIndexLimit(2);
                            Data{1} = 'Image-Z';
                        end
                        Data{1} = [Data{1},num2str(obj.Stack.CurrentZCT(1))];
                    else
                        if obj.ShowMask
                            Indexes = obj.Stack.MaskList{1}{Data{3}}(:);
                            Data{1} = 'Stack-Mask';
                        else
                            Indexes = 1:prod(obj.Stack.DataSize(1:3));
                            Data{1} = 'Stack';
                        end
                    end
                    if obj.Stack.CurrentZCT(2) > obj.Stack.DataSize(4)
                        Indexes = Indexes + prod(obj.Stack.DataSize(1:3))*(Data{3}-1);
                        Values = double(obj.ExtraStack{obj.Stack.CurrentZCT(2)-obj.Stack.DataSize(4)}(Indexes));
                    else
                        % Measurements
                        Indexes= Indexes + prod(obj.Stack.DataSize(1:4)) * (Data{3}-1) ...
                            + prod(obj.Stack.DataSize(1:3))*(obj.Stack.CurrentZCT(2)-1);
                        Values = double(obj.Stack.Data(Indexes));
                    end
                    % Fill table
                    % Intensity measurements
                    Data{4} = mean(Values);
                    Data{5} = std(Values);
                    Data{6} = min(Values);
                    Data{7} = max(Values);
                    Data{8} = sum(Values);
                    Data{9} = median(Values);
                    % Geometrical measurements
                    Data{10} = length(Indexes);
                    [PosY,PosX,PosZ,~,~] = ind2sub(obj.Stack.DataSize,Indexes);
                    Pos = double([PosX,PosY,PosZ]);
                    Data{11} = mean(Pos(:,1));
                    Data{12} = mean(Pos(:,2));
                    Data{13} = mean(Pos(:,3));
                    % Center of mass
                    Data{14} = mean(Pos(:,1).*Values);
                    Data{15} = mean(Pos(:,2).*Values);
                    Data{16} = mean(Pos(:,3).*Values);
                    % Add to DataTable
                    obj.ChildDataTable.AddRow(Data);
                end
            else  % From ROI
                for t = MinT:MaxT
                    Data{3} = t;
                    if obj.Stack.CurrentZCT(2) > obj.Stack.DataSize(4)
                        % {c}(r,c,z,t) Additional 5D stack
                        Offset = prod(obj.Stack.DataSize(1:3))*(Data{3}-1);
                    else
                        Offset = prod(obj.Stack.DataSize(1:4)) * (Data{3}-1) ...
                            + prod(obj.Stack.DataSize(1:3))*(obj.Stack.CurrentZCT(2)-1);
                    end;
                    for ii = 1:length(obj.Roi.Selected);
                        % Selection of indexes
                        if OnlyCurrent2D
                            Z = obj.Roi.List(obj.Roi.Selected(ii)).LastZ;
                        end;
                        Indexes = obj.Roi.List(obj.Roi.Selected(ii)).GetRoiIndexesZ(Z,AvoidSelections);
                        if isempty(Indexes)
                            continue;
                        end;
                        Indexes = uint64(Indexes) + Offset;
                        % Decide whether is Stack or ExtraStack
                        if obj.Stack.CurrentZCT(2) > obj.Stack.DataSize(4)
                            Values = double(obj.Stack.Extra{obj.Stack.CurrentZCT(2)-obj.Stack.DataSize(4)}(Indexes));
                        else
                            % Measurements
                            Values = double(obj.Stack.Data(Indexes));
                        end;
                        % Fill table
                        Data{1} = obj.Roi.List(obj.Roi.Selected(ii)).RoiName;
                        % Intensity measurements
                        Data{4} = mean(Values);
                        Data{5} = std(Values);
                        Data{6} = min(Values);
                        Data{7} = max(Values);
                        Data{8} = sum(Values);
                        Data{9} = median(Values);
                        % Geometrical measurements
                        Data{10} = length(Indexes);
                        [PosY,PosX,PosZ,~,~] = ind2sub(obj.Stack.DataSize,Indexes);
                        Pos = double([PosX,PosY,PosZ]);
                        Data{11} = mean(Pos(:,1));
                        Data{12} = mean(Pos(:,2));
                        Data{13} = mean(Pos(:,3));
                        % Center of mass
                        Data{14} = mean(Pos(:,1).*Values);
                        Data{15} = mean(Pos(:,2).*Values);
                        Data{16} = mean(Pos(:,3).*Values);
                        % Add to DataTable
                        obj.ChildDataTable.AddRow(Data);
                    end
                end
            end
        end
        
        function RoiManager_Operations(obj)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            TextStrings = {'And';'Or';'Exclusive Or'};
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + (length(TextStrings)+1)*GUIP.NormalEditHeight;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 2*GUIP.NormalButtonSize(1) + 1*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','ROI operation...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            
            hD.BG = uibuttongroup('Parent',hD.Fig...
                ,'BorderType','none','BorderWidth',0 ...
                ,'Title','');
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            Pos = [GUIP.DialogMargin(1),0,Size(1)-2*GUIP.DialogMargin(1),GUIP.NormalEditHeight];
            YT = Size(2) - GUIP.DialogMargin(2) - (1:length(TextStrings))*GUIP.NormalEditHeight;
            for n = 1:length(TextStrings)
                Pos(2) = YT(n);
                % Radiobuttons:
                hD.Option(n) = uicontrol(hD.BG,'Style','radiobutton'...
                    ,'String',TextStrings{n} ...
                    ,'Enable','on'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',Pos...
                    ,'UserData',n);
            end;
            set(hD.Option(obj.Roi.Operation),'Value',1);
            % Buttons
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);            
            PosE(2) = GUIP.DialogMargin(2);
            PosE(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.BG,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','close(gcbf);');
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);                        
            uicontrol(hD.BG,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                obj.Roi.Operation = get(get(hD.BG,'SelectedObject'),'UserData');
                switch obj.Roi.Operation
                    case 1  % And
                        N = length(obj.Roi.List)+1;     % Point to new Roi.List
                        obj.Roi.List(N) = ImageRoi.AndN(obj.Roi.List(obj.Roi.Selected));
                    case 2  % Or
                        N = length(obj.Roi.List)+1;     % Point to new Roi.List
                        obj.Roi.List(N) = ImageRoi.OrN(obj.Roi.List(obj.Roi.Selected));
                    case 3  % Exclusive Or
                        N = length(obj.Roi.List)+1;     % Point to new Roi.List
                        obj.Roi.List(N) = ImageRoi.ExOrN(obj.Roi.List(obj.Roi.Selected));
                end;
%                 RoiManager_Update([],[],obj);
                RoiManager_Update(obj);
                set(obj.hSidebar.RoiList,'Value',N+1);
                obj.Roi.Selected = N;
                close(hD.Fig);                
            end;
            % Enable every figure
            obj.EnableEverything(true);
            
        end;

        function RoiManager_Draw(obj)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 9* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 2*GUIP.NormalEditHeight + 5*GUIP.NormalTextHeight;
            Size(1) = mm2pt(70);
            %2 GUIP.DialogMargin(1) ...
             %   + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','Draw Roi...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );    
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            Pos = [0,0,0,0];
            Offset = 0.7*Size(1);
            % Border value
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Size(2) - GUIP.DialogMargin(2) - 1*GUIP.NormalEditHeight;
            Pos(3) = Offset - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Border value:' ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Offset;
            Pos(3) = Size(1)-GUIP.DialogMargin(1)-Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Rounded = false;
            S.MinMax = [-inf,inf];
            S.Old = obj.RoiDrawColor(1);
            hD.BorderValueEdit = uicontrol(hD.Fig,'Style','edit'...
                ,'String',num2str(S.Old) ... % num2str(obj.BackgroundValue) ...
                ,'UserData',S ... % obj.BackgroundValue ...
                ,'BackgroundColor','white','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Callback',@(Source,Event)obj.EditNumCallback(Source)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Foreground Checkbox
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            Pos(3) = Size(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            hD.ForegroundCheckbox = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Draw also foreground','Value',1 ...
                ,'FontSize',GUIP.NormalFontSize ... %                ,'Callback',@(Source,Event)obj.RoiManager_DrawForeground(hD,1)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Foreground same Checkbox
            Pos(1) = 3*GUIP.DialogMargin(1);
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            Pos(4) = GUIP.NormalTextHeight;
            hD.SameForegroundCheckbox = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Same value as border','Value',1 ...
                ,'FontSize',GUIP.NormalFontSize ... %                 ,'Callback',@(Source,Event)obj.RoiManager_DrawForeground(hD,2)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Foreground value
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalEditHeight;
            Pos(3) = Offset - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Foreground value:' ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Offset;
            Pos(3) = Size(1) - GUIP.DialogMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.RoiDrawColor(2);
            hD.ForegroundEdit = uicontrol(hD.Fig,'Style','edit'...
                ,'String',num2str(S.Old) ... % num2str(obj.BackgroundValue) ...
                ,'UserData',S ... % obj.BackgroundValue ...
                ,'BackgroundColor','white','Enable','off'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Callback',@(Source,Event)obj.EditNumCallback(Source)...
                ,'Units',GUIP.Units,'Position',Pos);
            % 3D ROIs in every slice
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            Pos(3) = Size(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            hD.AllSlices3D = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Draw full 3D ROIs','Value',1 ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            % 2D Rois repeated for each slice
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            hD.AllSlices2D = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Draw 2D ROIs in every slice','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            if obj.Stack.DataSize(3) == 1
                set(hD.AllSlices3D,'Enable','off','Value',false);
                set(hD.AllSlices2D,'Enable','off','Value',false);
            end;
            % All frames
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            hD.AllFrames = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Draw ROIs in every frame','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            if obj.Stack.DataSize(5) == 1
                set(hD.AllFrames,'Enable','off','Value',false);
            end;
            % Callbacks
            set(hD.ForegroundCheckbox,'Callback',@(Source,Event)obj.RoiManager_DrawForeground(hD,1));
            set(hD.SameForegroundCheckbox,'Callback',@(Source,Event)obj.RoiManager_DrawForeground(hD,2));
            % Buttons
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','close(gcbf);');
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                obj.RoiDrawColor(1) = str2double(get(hD.BorderValueEdit,'String'));
                obj.RoiDrawColor(2) = str2double(get(hD.ForegroundEdit,'String'));
                % Define Foreground color
                if get(hD.ForegroundCheckbox,'Value')
                    if get(hD.SameForegroundCheckbox,'Value')
                        ForegroundColor = obj.RoiDrawColor(1);
                    else
                        ForegroundColor = obj.RoiDrawColor(2);
                    end;
                else
                    ForegroundColor = [];
                end;
                % Define Slices for 3D ROIs
                if get(hD.AllSlices3D,'Value')
                    Z3D = [];                    
                else
                    Z3D = obj.Stack.CurrentZCT(1);
                end;
                % Define Slices for 2D ROIs
                if get(hD.AllSlices2D,'Value')
                    Z2D = 1:obj.Stack.DataSize(3);                    
                else
                    Z2D = obj.Stack.CurrentZCT(1);
                end;
                % Define frames
                if get(hD.AllFrames,'Value')
                    T = 1:obj.Stack.DataSize(5);
                else
                    T = obj.Stack.CurrentZCT(3);
                end;
                % Analyze each selected ROI
                for ii = 1:length(obj.Roi.Selected);
                    RI = obj.Roi.Selected(ii);
                    if obj.Roi.List(RI).Is2D
                        obj.Stack.Draw2DIndexes(obj.Roi.List(RI).RoiIndexes ...
                            ,Z2D,obj.Stack.CurrentZCT(2),T,obj.RoiDrawColor(1),ForegroundColor);
                    else
                        obj.Stack.DrawIndexes(obj.Roi.List(RI).RoiIndexes ...
                            ,Z3D,obj.Stack.CurrentZCT(2),T,obj.RoiDrawColor(1),ForegroundColor);
                    end;                            
                end;
                close(hD.Fig);
                obj.ShowImage;
            end;
            % Enable every figure
            obj.EnableEverything(true);
            
        end;
        
        % =================================================================
        
        function RoiManager_Transfer(obj)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            
            ListChildImages = cell(length(obj.ChildImages),1);
            k = 0;
            for n = 1:length(obj.ChildImages)
                if obj.ChildImages{n}.NextUpdate == 0
                    k = k + 1;
                    ListChildImages{k} = obj.ChildImages{n};
                end
            end
            if k > 0
                ListChildImages = ListChildImages(1:k);
            end
            if isempty(ListChildImages) && isempty(obj.ParentImage)            
                msgbox({'The image does not have connection with other images...'...
                    ;'Nothing can be transfered!'}...
                    ,'Error','error','modal');
                return;
            else
                % Read GUI Proportions
                GUIP = GetGUIProp('dialog');
                % -------------------------------------------------------------
                % Create Dialog
                % -------------------------------------------------------------
                Size = [0,0];
                if isempty(obj.ParentImage)
                    % Calculation of size:
                    Size(2) = 2* GUIP.DialogMargin(2) ...
                        + GUIP.NormalButtonSize(2) + 10*GUIP.NormalEditHeight;
                    Size(1) = 2* GUIP.DialogMargin(1) ...
                        + 2*GUIP.NormalButtonSize(1) + 1*GUIP.NormalButtonMargin(1);
                    IsParent = true;
                else
                    % Calculation of size:
                    Size(2) = 2* GUIP.DialogMargin(2) ...
                        + GUIP.NormalButtonSize(2) + 3*GUIP.NormalEditHeight;
                    Size(1) = 2* GUIP.DialogMargin(1) ...
                        + 2*GUIP.NormalButtonSize(1) + 1*GUIP.NormalButtonMargin(1);
                    IsParent = false;
                end;
                % Relative position:
                OldUnits = get(obj.hFig,'Units');
                set(obj.hFig,'Units','points');
                SC = get(obj.hFig,'Position');
                set(obj.hFig,'Units',OldUnits);
                hD.Fig = dialog('Name','Transfer ROIs...'...
                    ,'Units','points'...
                    ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                    ,'Visible','off'...
                    ,'WindowStyle','normal'...
                    );
                % Disable every figure
                obj.EnableEverything(false,hD.Fig);
                hD.Fig.Visible = 'on';
                % -------------------------------------------------------------
                % Dialog content
                % -------------------------------------------------------------
                YT = Size(2) - GUIP.DialogMargin(2);
                uicontrol(hD.Fig,'Style','text'...
                    ,'String','Destination of the ROIs:' ...
                    ,'HorizontalAlignment','left'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',[GUIP.DialogMargin(1),YT-GUIP.NormalEditHeight...
                    ,Size(1)-2*GUIP.DialogMargin(1),GUIP.NormalTextHeight]);
               % Create Listbox 
               if IsParent
                    % List of strings
                    N = length(ListChildImages);
                    S = cell(N,1);
                    for n = 1:N
                        S{n} = ListChildImages{n}.Name;
                    end;
                    % Listbox
                    hD.List = uicontrol(hD.Fig,'Style','listbox'...
                        ,'String',S ...
                        ,'BackgroundColor','white'...
                        ,'HorizontalAlignment','left'...
                        ,'Enable','on','Max',2 ...
                        ,'FontSize',GUIP.NormalFontSize ...
                        ,'Units','points'...
                        ,'Position',[GUIP.DialogMargin(1),GUIP.DialogMargin(2)+GUIP.NormalButtonSize(2)+GUIP.NormalEditHeight ...
                        ,Size(1)-2*GUIP.DialogMargin(1),8*GUIP.NormalEditHeight]);
                else
                    S = 'Main Image';
                    % Listbox
                    hD.List = uicontrol(hD.Fig,'Style','text'...
                        ,'String',S ...
                        ,'BackgroundColor','white'...
                        ,'HorizontalAlignment','center'...
                        ,'Enable','on'...
                        ,'FontSize',GUIP.NormalFontSize ...
                        ,'Units','points'...
                        ,'Position',[GUIP.DialogMargin(1),GUIP.DialogMargin(2)+GUIP.NormalButtonSize(2)+GUIP.NormalEditHeight ...
                        ,Size(1)-2*GUIP.DialogMargin(1),GUIP.NormalTextHeight]);
                end;
                % Buttons
                PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);
                PosE(2) = GUIP.DialogMargin(2);
                PosE(3:4) = GUIP.NormalButtonSize;
                uicontrol(hD.Fig,'Style','pushbutton'...
                    ,'String','Cancel'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',PosE...
                    ,'Callback','close(gcbf);');
                PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);
                uicontrol(hD.Fig,'Style','pushbutton'...
                    ,'String','Apply'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',PosE...
                    ,'Callback','uiresume(gcbf);');
                uiwait(hD.Fig);
                % Saving process
                if ishghandle(hD.Fig)
                    if IsParent
                        Val = get(hD.List,'Value');
                        for n = 1:length(Val)
                            for ii = 1:length(obj.Roi.Selected);
                                RI = obj.Roi.Selected(ii);
%                                 T = obj.Roi.List(ii).Frames;
                                T = obj.Roi.List(RI).Frames;
%                                 L = obj.ChildImages{Val(n)}.Parent2Child(obj.Roi.List(RI).RoiIndexes,T);
                                if obj.Roi.List(RI).Is2D
                                    Offset = obj.Roi.List(RI).RoiImageSize(1)*obj.Roi.List(RI).RoiImageSize(2)...
                                        *(obj.Stack.CurrentZCT(1)-1);
                                    L = ListChildImages{Val(n)}.ReceiveRoiIndexes(obj.Roi.List(RI).RoiIndexes+Offset,T);                                    
                                else
                                    L = ListChildImages{Val(n)}.ReceiveRoiIndexes(obj.Roi.List(RI).RoiIndexes,T);
                                end;
                                ListChildImages{Val(n)}.Roi.List(end+1) = ImageRoi(obj.ChildImages{Val(n)}.Stack.DataSize(1:3),L);
                                ListChildImages{Val(n)}.Roi.List(end).Channels = obj.Roi.List(RI).Channels;
                                ListChildImages{Val(n)}.Roi.List(end).Frames = obj.Roi.List(RI).Frames;
                                ListChildImages{Val(n)}.Roi.List(end).RoiName = obj.Roi.List(RI).RoiName;
                                ListChildImages{Val(n)}.Roi.List(end).Is2D = true;
                            end;
%                             RoiManager_Update([],[],obj.ChildImages{Val(n)});
                            RoiManager_Update(ListChildImages{Val(n)});
                        end;
                    else
                        for ii = 1:length(obj.Roi.Selected);
                            RI = obj.Roi.Selected(ii);
%                             T = obj.Roi.List(ii).Frames;
                            T = obj.Roi.List(RI).Frames;
%                             L = obj.Child2Parent(obj.Roi.List(RI).RoiIndexes,T);
                            L = obj.SendRoiIndexes(obj.Roi.List(RI).RoiIndexes,T);
                            obj.ParentImage.Roi.List(end+1) = ImageRoi(obj.ParentImage.Stack.DataSize(1:3),L);
                            obj.ParentImage.Roi.List(end).Channels = obj.Roi.List(RI).Channels;
                            obj.ParentImage.Roi.List(end).Frames = obj.Roi.List(RI).Frames;
                            obj.ParentImage.Roi.List(end).RoiName = obj.Roi.List(RI).RoiName;
                        end;
%                         RoiManager_Update([],[],obj.ParentImage);
                        RoiManager_Update(obj.ParentImage);
                    end;
                    close(hD.Fig);
                end
                % Enable every figure
                obj.EnableEverything(true);

            end;
        end
        
        % =================================================================
        % Save all selected ROIs into a file
        function RoiManager_Save(obj)
            % Version: 0.7 (11.11.2014)
            % -------------------------
            Version = 0.7;
            if ~isempty(obj.Roi.List)
                NewFolder = false;
                if ~exist(obj.Folder,'dir') && ~isempty(obj.Folder)
                    mkdir(obj.Folder);
                    NewFolder = true;
                end
                % Disable every figure
                obj.EnableEverything(false);
                [FileName,PathName] = uiputfile({'*.rsm'},'Save ROI file...',obj.Folder);
                % Enable every figure
                obj.EnableEverything(true);
                if NewFolder && ~strcmpi(obj.Folder,PathName)
                    rmdir(obj.Folder);
                end
                if ~isequal(FileName,0)
                    obj.Folder = PathName;
                    [~,FileName,FileExt] = fileparts(FileName);                    
                    if isempty(FileExt)
                        FileExt = '.rsm';
                    end;
                    FullFilename = fullfile(PathName,[FileName,FileExt]);
                    % Open binary file
                    fid = fopen(FullFilename,'w+','l','UTF-8');
                    % Write Header Version
                    fwrite(fid,Version,'float64',0,'l');
                    % Write Header Number of ROIs
                    if obj.Roi.Selected(1) == 0
                        Idx = 1:length(obj.Roi.List);
                    else
                        Idx = obj.Roi.Selected;
                    end
                    N = length(Idx);
                    fwrite(fid,N,'uint64',0,'l');
                    % Leave space for the ROIs Offset
                    RoiPosition = ftell(fid);
                    fwrite(fid,zeros(N,1),'uint64',0,'l');
                    % End of Header:
                    for n = 1:N
                        CurrentPos = ftell(fid);
                        fseek(fid,RoiPosition+(n-1)*8,'bof');
                        fwrite(fid,CurrentPos,'uint64',0,'l');
                        fseek(fid,CurrentPos,'bof');
                        % Write name
                        FieldLength = length(obj.Roi.List(Idx(n)).RoiName);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).RoiName,'char',0,'l');
                        % Write Imagesize
                        FieldLength = length(obj.Roi.List(Idx(n)).RoiImageSize);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).RoiImageSize,'uint64',0,'l');
                        % Write Slices
                        FieldLength = length(obj.Roi.List(Idx(n)).Slices);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).Slices,'uint64',0,'l');
                        % Write Channels
                        FieldLength = length(obj.Roi.List(Idx(n)).Channels);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).Channels,'uint64',0,'l');
                        % Write Frames
                        FieldLength = length(obj.Roi.List(Idx(n)).Frames);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).Frames,'uint64',0,'l');
                        % Write Is2D
                        FieldLength = length(obj.Roi.List(Idx(n)).Is2D);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).Is2D,'uint64',0,'l');
                        % Write RoiIndexes
                        FieldLength = length(obj.Roi.List(Idx(n)).RoiIndexes);
                        fwrite(fid,FieldLength,'uint64',0,'l');
                        fwrite(fid,obj.Roi.List(Idx(n)).RoiIndexes,'uint64',0,'l');
                    end
                    fclose(fid);
                end
            end
        end

        % =================================================================
        % Load ROIs from a file
        function RoiManager_Load(obj)
            % Version: 0.7 (11.11.2014)
            % -------------------------
            NRoiList = length(obj.Roi.List);
            % Disable every figure
            obj.EnableEverything(false);
            [FileName,PathName] = uigetfile({'*.rsm'},'Load ROI file...',obj.Folder);
            % Enable every figure
            obj.EnableEverything(true);
            if ~isequal(FileName,0)
                obj.Folder = PathName;
                [~,FileName,FileExt] = fileparts(FileName);
                if isempty(FileExt)
                    FileExt = '.rsm';
                end;
                FullFilename = fullfile(PathName,[FileName,FileExt]);
                % Open binary file
                fid = fopen(FullFilename,'r','l','UTF-8');
                % Read Header Version
                Version = fread(fid,1,'*float64',0,'l');
                if Version >= 0.7
                    % Read number of ROIs
                    N = fread(fid,1,'uint64',0,'l');
                    RoiPosition = fread(fid,N,'*uint64',0,'l');
                    % Read each ROI
                    for n = 1:N
                        fseek(fid,RoiPosition(n),'bof');
                        % Read name
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        RoiName = fread(fid,[1,FieldLength],'*char',0,'l');
                        % Read Imagesize
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        RoiImageSize = fread(fid,[1,FieldLength],'*uint64',0,'l');
                        % Create ROI
                        NRoiList = NRoiList + 1;
                        obj.Roi.List(NRoiList) = ImageRoi(RoiImageSize);
                        obj.Roi.List(NRoiList).RoiName = RoiName;
                        % Read Slices
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        obj.Roi.List(NRoiList).Slices = fread(fid,FieldLength,'*uint64',0,'l');
                        % Read Channels
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        obj.Roi.List(NRoiList).Channels = fread(fid,FieldLength,'*uint64',0,'l');
                        % Read Frames
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        obj.Roi.List(NRoiList).Frames = fread(fid,FieldLength,'*uint64',0,'l');
                        % Read Is2D
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        Is2D = fread(fid,FieldLength,'*uint64',0,'l');
                        obj.Roi.List(NRoiList).Is2D = Is2D > 0;
                        % Read RoiIndexes
                        FieldLength = fread(fid,1,'*uint64',0,'l');
                        obj.Roi.List(NRoiList).RoiIndexes = fread(fid,FieldLength,'*uint64',0,'l');
                    end
                end
                fclose(fid);
                RoiManager_Update(obj);
            end
        end
        
        
        % =================================================================
        
        function PanCallback(obj,~,~)
            % Pan on the image
            % -------------------------------------------------------------
            Keys = get(obj.hFig,'CurrentModifier');
            ButtonState = get(obj.hToolbar(3),'Value');
            if ButtonState == get(obj.hToolbar(3),'Max')
                obj.PanZoomFlag = 1;
                set(obj.hToolbar(4),'Value',get(obj.hToolbar(4),'Min'));
                if ismember('control',Keys)
                    pan(obj.hFig,'xon');
                elseif ismember('shift',Keys)
                    pan(obj.hFig,'yon');
                else
                    pan(obj.hFig,'on');
                end;
            elseif ButtonState == get(obj.hToolbar(3),'Min')
                obj.PanZoomFlag = 0;
                pan(obj.hFig,'off');
            end;
        end;

        % =================================================================
        
        function ZoomCallback(obj,~,~)
            % Pan on the image
            % -------------------------------------------------------------
            Keys = get(obj.hFig,'CurrentModifier');
            ButtonState = get(obj.hToolbar(4),'Value');
            if ButtonState == get(obj.hToolbar(4),'Max')
                obj.PanZoomFlag = 2;
                set(obj.hToolbar(3),'Value',get(obj.hToolbar(3),'Min'));
                if ismember('control',Keys)
                    zoom(obj.hFig,'xon');
                elseif ismember('shift',Keys)
                    zoom(obj.hFig,'yon');
                else
                    zoom(obj.hFig,'on');
                end;
            elseif ButtonState == get(obj.hToolbar(4),'Min')
                obj.PanZoomFlag = 0;
                zoom(obj.hFig,'off');
            end;
        end;

        % =================================================================
        
        function ChannelMapCallback(obj,~,~)
            % Pan on the image
            % -------------------------------------------------------------
            Val = get(obj.hToolbar(5),'Value');
            ListColorMaps = get(obj.hToolbar(5),'String');
            c = obj.Stack.CurrentZCT(2);
            obj.Stack.ChannelMapValue(c) = Val;
            switch ListColorMaps{Val}
                case 'Gray'
                    obj.Stack.ChannelMap{c} = gray(256);
                case 'Red'
                    obj.Stack.ChannelMap{c}(:,1) = (1:256);
                    obj.Stack.ChannelMap{c}(:,2:3) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Green'
                    obj.Stack.ChannelMap{c}(:,2) = (1:256);
                    obj.Stack.ChannelMap{c}(:,[1,3]) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Blue'
                    obj.Stack.ChannelMap{c}(:,3) = (1:256);
                    obj.Stack.ChannelMap{c}(:,[1,2]) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Cyan'
                    obj.Stack.ChannelMap{c}(:,2) = (1:256);
                    obj.Stack.ChannelMap{c}(:,3) = (1:256);
                    obj.Stack.ChannelMap{c}(:,1) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Yellow'
                    obj.Stack.ChannelMap{c}(:,1) = (1:256);
                    obj.Stack.ChannelMap{c}(:,2) = (1:256);
                    obj.Stack.ChannelMap{c}(:,3) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Magenta'
                    obj.Stack.ChannelMap{c}(:,1) = (1:256);
                    obj.Stack.ChannelMap{c}(:,3) = (1:256);
                    obj.Stack.ChannelMap{c}(:,2) = 0;
                    obj.Stack.ChannelMap{c} = obj.Stack.ChannelMap{c} / 256;
                case 'Jet'
                    obj.Stack.ChannelMap{c} = jet(256);
                case 'Hot'
                    obj.Stack.ChannelMap{c} = hot(256);
                case 'Cool'
                    obj.Stack.ChannelMap{c} = cool(256);
                case 'Spring'
                    obj.Stack.ChannelMap{c} = spring(256);
                case 'Summer'
                    obj.Stack.ChannelMap{c} = summer(256);
                case 'Autumn'
                    obj.Stack.ChannelMap{c} = autumn(256);
                case 'Winter'
                    obj.Stack.ChannelMap{c} = winter(256);
                case 'Bone'
                    obj.Stack.ChannelMap{c} = bone(256);
                case 'Cooper'
                    obj.Stack.ChannelMap{c} = copper(256);
                case 'Pink'
                    obj.Stack.ChannelMap{c} = pink(256);
            end;
%             obj.UpdateImage;
            obj.ShowImage;
        end;
        
        % =================================================================
        
        function StretchCallback(obj,~,~)
            % Pan on the image
            % -------------------------------------------------------------
            if get(obj.hToolbar(6),'Value') == get(obj.hToolbar(6),'Max')
                set(obj.hImage.Axes,'DataAspectRatioMode','auto');
            else
                set(obj.hImage.Axes,'DataAspectRatio',[1,1,1]);                
            end;
        end;

        % =================================================================
        
        function MaskedCallback(obj,~,~)
            % Show the masked image
            % -------------------------------------------------------------
            if get(obj.hToolbar(7),'Value') == get(obj.hToolbar(7),'Max')
                obj.ShowMask = true;
            else
                obj.ShowMask = false;
            end;
            obj.ShowImage;
        end;

        % =================================================================
        
        function DisplayRangeCallback(obj,Source,~,n)
            % Show the masked image
            % -------------------------------------------------------------
            if nargin < 4
                n = 4;
            end;
            Keys = get(obj.hFig,'CurrentModifier');
            I = obj.Stack.CurrentZCT;
            ZRange = I(1);
            c = I(2);
            TRange = I(3);
            % Calculation of the limit values
            if any(strcmp(obj.Stack.DataClass,{'uint8','uint16'}))
                Limits = [0,intmax(obj.Stack.DataClass)];
            else
                Limits = [min(obj.Stack.Data(:)), max(obj.Stack.Data(:))];
                if any(isnan(obj.Stack.ChannelDisplayRange(c,:)))
                    Limits = [0,1];
                end;
            end;
            % Determination of the source of callback
            switch n
                case 1
                    Num = str2double(get(Source,'String'));                
                    if isfinite(Num) 
                        obj.Stack.ChannelDisplayRange(c,1) = Num;
                        Num2 = str2double(get(obj.hToolbar(11),'String'));
                        if Num >= Num2
                            if Num < Limits(2)
                                obj.Stack.ChannelDisplayRange(c,2) = Limits(2);
                            else
                                obj.Stack.ChannelDisplayRange(c,2) = Num + 1;
                            end
                        end
                    else
                        obj.Stack.ChannelDisplayRange(c,1) = Limits(1);
                    end
                case 2
                    Num = str2double(get(Source,'String'));                
                    if isfinite(Num) 
                        obj.Stack.ChannelDisplayRange(c,2) = Num;
                        Num0 = str2double(get(obj.hToolbar(9),'String'));
                        if Num <= Num0
                            if Num > Limits(1)
                                obj.Stack.ChannelDisplayRange(c,1) = Limits(1);
                            else
                                obj.Stack.ChannelDisplayRange(c,1) = Num - 1;
                            end
                        end
                    else
                        obj.Stack.ChannelDisplayRange(c,2) = Limits(2);
                    end
                case 3
                    if ismember('control',Keys)
                        ZRange = 1:obj.Stack.DataSize(3);
                    end;
                    if ismember('shift',Keys)
                        TRange = 1:obj.Stack.DataSize(5);
                    end;
                    if c > obj.Stack.DataSize(4)
                        A = [min(min(min(min(obj.Stack.Extra{c-obj.Stack.DataSize(4)}(:,:,ZRange,TRange))))) ...
                            max(max(max(max(obj.Stack.Extra{c-obj.Stack.DataSize(4)}(:,:,ZRange,TRange)))))];
                    else
                        A = [min(min(min(min(obj.Stack.Data(:,:,ZRange,c,TRange))))) ...
                            max(max(max(max(obj.Stack.Data(:,:,ZRange,c,TRange)))))];
                    end;
                    if all(~isnan(A))
                        if A(1) == A(2)
                            obj.Stack.ChannelDisplayRange(c,:) = Limits;
                        else
                            obj.Stack.ChannelDisplayRange(c,:) = A;
                        end
                    end
                case 4
                    obj.Stack.ChannelDisplayRange(c,:) = Limits;
            end
            % Update Edit controls
            if isequal(obj.Stack.ChannelDisplayRange(c,:) - floor(obj.Stack.ChannelDisplayRange(c,:)),[0,0])
                set(obj.hToolbar(9),'String',num2str(obj.Stack.ChannelDisplayRange(c,1),'%d'));
                set(obj.hToolbar(11),'String',num2str(obj.Stack.ChannelDisplayRange(c,2),'%d'));
            else
                set(obj.hToolbar(9),'String',num2str(obj.Stack.ChannelDisplayRange(c,1),'%.2f'));
                set(obj.hToolbar(11),'String',num2str(obj.Stack.ChannelDisplayRange(c,2),'%.2f'));
            end
            % Update image
            obj.ShowImage;
        end;
        
        % =================================================================
        
        function GetJavaHandle(obj)
            % Function to get the Java Frame handle of the figure
            % -------------------------------------------------------------
            obj.hFigJava = [];
            % Check that Java Virtual Machine is available
            if ~usejava('jvm')
                disp('This software requires Java Virtual Machine to run...');
                delete(obj);
            end
            MaxChances = 15;
            while (MaxChances > 0) && isempty(obj.hFigJava);
                try
                    JavaFigure = get(handle(obj.hFig),'JavaFrame');
                    try
                        obj.hFigJava = JavaFigure.fHG2Client.getWindow;
                    catch
                        obj.hFigJava = JavaFigure.fHG1Client.getWindow;
%                         obj.hFigJava = JavaFigure.fFigureClient.getWindow;
                    end;
                catch
                    obj.hFigJava = [];
                end;
                MaxChances = MaxChances - 1;
                pause(0.1);
            end;
        end;

        % =================================================================
        
        function State = EnableFigure(obj,Flag)
            % Function to enable/disable the figure
            % Flag: can be true,false,1,0
            % -------------------------------------------------------------
            State = [];
            if ~isempty(obj.hFigJava)
                State = get(obj.hFigJava,'Enabled');
                if nargin > 1
                    try
                        WarningMsg = warning('off','MATLAB:hg:JavaSetHGProperty');
                        obj.hFigJava.setEnabled(Flag);
                        warning(WarningMsg)
                    catch
                    end;
                end;
            end;
        end;

        % =================================================================
        
        function SaveFigure(obj,~,~,SearchFolder)
            % Save figure contents
            % -------------------------------------------------------------
            if nargin < 4
                SearchFolder = obj.Folder;
            end;
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + (2*6+2)*GUIP.NormalTextHeight ...
                + (4*GUIP.NormalTextHeight + 2*GUIP.PanelMargin(2)) ;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 4*GUIP.NormalButtonSize(1) + 3*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','Saving...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );    
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            % Checkboxes:
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Size(2);
            Pos(3) = Size(1) - GUIP.DialogMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            TextStrings = {'Save main (original) channels' ...
                          ;'Apply mask to channels (slower)' ...
                          ;'Save additional channels'};
            OnOff = {'on','off'};
            EnableList = [1,1,2];
            ValueList = [1,0,1];
            if isempty(obj.Stack.MaskList{1})
                EnableList(2) = 2;
            end;
            for n = 1:3
                % Edits:
                Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
                hD.Checkbox(n) = uicontrol(hD.Fig,'Style','checkbox'...
                    ,'String',TextStrings{n} ...
                    ,'Value',ValueList(n) ...
                    ,'Enable',OnOff{EnableList(n)}...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos);
            end;
            % Additional channels listbox
            Pos(2) = Pos(2) - 2*GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Additional channels:'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'FontAngle','normal' ... % ,'FontWeight','bold' ...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                );            
            Pos(2) = Pos(2) -0.5*GUIP.DialogMargin(2) -6*GUIP.NormalTextHeight ;
            Pos(4) = 6*GUIP.NormalTextHeight;
            hD.AdditionalChannels = uicontrol(hD.Fig,'style','listbox'...
                ,'Min',0,'Max',2 ...
                ,'Value',1 ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'BackGroundColor',[1,1,1] ...
                ,'Enable','off' ...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                );           
            % Listbox items:
            N = length(obj.Stack.Extra);
            if N > 0
                set(hD.Checkbox(3),'Enable','on');
                ListStrings = cell(1,N);
                for n = 1:N
                    ListStrings{n} = ['Channel '...
                        ,num2str(obj.Stack.DataSize(4)+n),': '...
                        ,obj.Stack.ChannelYLabel{obj.Stack.DataSize(4)+n}...
                        ,' / '...
                        ,obj.Stack.ChannelXLabel{obj.Stack.DataSize(4)+n}...
                        ];
                end;
                set(hD.AdditionalChannels,'Enable','on'...
                    ,'String',ListStrings,'Value',1:N);
            else
                set(hD.Checkbox(1),'Enable','off');
            end;
            % Multi channel saving options---------------------------------
            Pos(1) = GUIP.DialogMargin(1);
            Pos(3) = Size(1)-2*GUIP.DialogMargin(1);
            Pos(2) = Pos(2) - 2*GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Channel distribution in files:'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'FontAngle','normal' ... % ,'FontWeight','bold' ...
                ,'HorizontalAlignment','left'...
                ,'Units','points'...
                ,'Position',Pos...
                );            
            Pos(2) = Pos(2) - 0.5*GUIP.DialogMargin(2) - GUIP.NormalEditHeight;
            Pos(4) = GUIP.NormalEditHeight;
            TextStrings = {'Multiple files for additional channels'...
                          ;'Extra channels in one separate file' ...
                          ;'Original & additional channels in one file'...
                          };
            hD.ChannelsFileDistribution = uicontrol(hD.Fig,'Style','popupmenu'...
                ,'String','  '...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'FontAngle','normal' ... % ,'FontWeight','bold' ...
                ,'HorizontalAlignment','left'...
                ,'Units','points'...
                ,'Position',Pos...
                ,'UserData',TextStrings...
                );            
            if obj.Stack.FullZCT(2) == obj.Stack.DataSize(4)
                hD.ChannelsFileDistribution.Enable = 'off';
            else
                hD.ChannelsFileDistribution.String = TextStrings;
            end
            % Buttons
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',Pos...
                ,'Callback','close(gcbf);');
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Save'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',Pos...
                ,'Callback','uiresume(gcbf);');
            for n = 1:3
                set(hD.Checkbox(n),'Callback'...
                    ,@(Source,Event)obj.SaveFigureCheckboxCallback(hD,n));
            end
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                
                NewFolder = false;
                if ~exist(SearchFolder,'dir') && ~isempty(SearchFolder)
                    mkdir(SearchFolder);
                    NewFolder = true;
                end
                [FileName,PathName] = uiputfile({'*.btf;*.tif'},'Save file...',SearchFolder);
                if NewFolder && ~strcmpi(SearchFolder,PathName)
                    rmdir(SearchFolder);
                end
                if ~isequal(FileName,0)
                    obj.Folder = PathName;
                    [~,FileName,FileExt] = fileparts(FileName);
                    FileName2 = FileName;
                    if isempty(FileExt)
                        FileExt = '.btf';
                    end;
                    obj.LogHistory.NewTask('Saving image');
                    % Analyze if there are extra channels
                    if strcmpi(hD.ChannelsFileDistribution.Enable,'on')
                        ExistExtraChannels = true;
                        ExtraChannels = get(hD.AdditionalChannels,'Value');
                        NExtraChannels = length(ExtraChannels);
                        % Analyze Extra channels biggest datatype
                        ExtraCast = 'uint8';
                        for n = 1:NExtraChannels
                            if isfloat(obj.Stack.Extra{ExtraChannels(n)})
                                ExtraCast = 'single';
                                break;
                            elseif intmax(class(obj.Stack.Extra{ExtraChannels(n)})) == 65535
                                ExtraCast = 'uint16';
                            end;
                        end;
                    else
                        ExistExtraChannels = false;
                        NExtraChannels = 0;
                    end
                    % Save Original Data:
                    if get(hD.Checkbox(1),'Value')
                        % Initialization of SuperCast:
                        if get(hD.Checkbox(2),'Value')
                            MainCast = 'single';
                        else
                            if ExistExtraChannels 
                                if strcmp(ExtraCast,'double') || strcmp(obj.Stack.DataClass,'double')
                                    MainCast = 'double';
                                elseif strcmp(ExtraCast,'single') || strcmp(obj.Stack.DataClass,'single')
                                    MainCast = 'single';
                                elseif strcmp(ExtraCast,'uint16') || strcmp(obj.Stack.DataClass,'uint16')
                                    MainCast = 'uint16';
                                else
                                    MainCast = 'uint8';
                                end;
                            else
                                MainCast = obj.Stack.DataClass;
                            end;
                        end;
                        % Initialization of SuperStack
                        if ExistExtraChannels && hD.ChannelsFileDistribution.Value == 3
                            MainStack = zeros(obj.Stack.DataSize(1)...
                                ,obj.Stack.DataSize(2)...
                                ,obj.Stack.DataSize(3)...
                                ,obj.Stack.DataSize(4) + NExtraChannels...
                                ,obj.Stack.DataSize(5),MainCast);
                            FileName = [FileName,'-full'];
                        else                            
                            MainStack = zeros(obj.Stack.DataSize(1)...
                                ,obj.Stack.DataSize(2)...
                                ,obj.Stack.DataSize(3)...
                                ,obj.Stack.DataSize(4)...
                                ,obj.Stack.DataSize(5),MainCast);
                        end
                        % Apply the mask:
                        if get(hD.Checkbox(2),'Value')
                            FileName = [FileName,'-masked'];
                            if iscell(obj.Stack.MaskList{1})
                                TMask = length(obj.Stack.MaskList{1});
                                MaskCellTimes = true;
                            else
                                TMask = 1;
                                MaskCellTimes = false;
                            end;
                            MaskStack = NaN(obj.Stack.DataSize(1:3),'single');
                            for t = 1:obj.Stack.DataSize(5)
                                if t > TMask
                                    t2 = TMask;
                                else
                                    t2 = t;
                                end;
                                MaskStack(:) = NaN;
                                if MaskCellTimes
                                    MaskStack(obj.Stack.MaskList{1}{t2}) = 1;
                                else
                                    MaskStack(obj.Stack.MaskList{1}) = 1;
                                end;
                                for c = 1:obj.Stack.DataSize(4)
                                    MainStack(:,:,:,c,t) = cast(obj.Stack.Data(:,:,:,c,t),MainCast) ...
                                        .* MaskStack;
                                end;
                            end;
                        else
                            MainStack(:,:,:,1:obj.Stack.DataSize(4),:) = obj.Stack.Data;
                        end;
                        % Add extra channels
                        if ExistExtraChannels && hD.ChannelsFileDistribution.Value == 3
                            for n = 1:NExtraChannels
                                T = size(obj.Stack.Extra{ExtraChannels(n)},4);
                                MainStack(:,:,:,obj.Stack.DataSize(4)+n,1:T) = cast(obj.Stack.Extra{ExtraChannels(n)},MainCast);
                                if T < obj.Stack.DataSize(5)
                                    for t = T+1:obj.Stack.DataSize(5)
                                        MainStack(:,:,:,obj.Stack.DataSize(4)+n,t) = ...
                                            MainStack(:,:,:,obj.Stack.DataSize(4)+n,T);
                                    end;
                                end;
                            end;
                        end;
                        % Save the image
                        FullFileName = [PathName,FileName,FileExt];
                        Figure5D.SaveStack2BigTiff(FullFileName,MainStack,obj.Stack.PixelSize);
                    end;
                    % Save Additional Channels independently from the stack:
                    if ExistExtraChannels 
                        % Save all additional channels in one separate file
                        switch hD.ChannelsFileDistribution.Value
                            case 1
                                T = obj.Stack.DataSize(5);
                                MainStack = zeros(obj.Stack.DataSize(1)...
                                    ,obj.Stack.DataSize(2)...
                                    ,obj.Stack.DataSize(3)...
                                    ,1,T,ExtraCast);
                                for n = 1:NExtraChannels
                                    T2 = size(obj.Stack.Extra{ExtraChannels(n)},4);
                                    MainStack(:,:,:,1,1:T2) = reshape(cast(obj.Stack.Extra{ExtraChannels(n)},ExtraCast)...
                                        ,obj.Stack.DataSize(1)...
                                        ,obj.Stack.DataSize(2)...
                                        ,obj.Stack.DataSize(3)...
                                        ,1,T2);
                                    if T2 < T
                                        for t = T2+1:T
                                            MainStack(:,:,:,1,t) = ...
                                                MainStack(:,:,:,1,T2);
                                        end;
                                    end;
                                    FileName = [FileName2,'-extra-Channel_',num2str(ExtraChannels(n),'%u')];
                                    FullFileName = [PathName,filesep,FileName,FileExt];
                                    Figure5D.SaveStack2BigTiff(FullFileName,cast(MainStack,'like',obj.Stack.Extra{ExtraChannels(n)})...
                                        ,obj.Stack.PixelSize);
                                end                            
                            case 2
                                % Suffix to filename
                                FileName = [FileName2,'-extra'];
                                % Initialization of the new MainStack
                                T = obj.Stack.DataSize(5);
                                MainStack = zeros(obj.Stack.DataSize(1)...
                                    ,obj.Stack.DataSize(2)...
                                    ,obj.Stack.DataSize(3)...
                                    ,NExtraChannels...
                                    ,T,ExtraCast);
                                % Add extra channels
                                for n = 1:NExtraChannels
                                    T2 = size(obj.Stack.Extra{ExtraChannels(n)},4);
                                    MainStack(:,:,:,n,1:T2) = reshape(cast(obj.Stack.Extra{ExtraChannels(n)},ExtraCast)...
                                        ,obj.Stack.DataSize(1)...
                                        ,obj.Stack.DataSize(2)...
                                        ,obj.Stack.DataSize(3)...
                                        ,1,T2);
                                    if T2 < T
                                        for t = T2+1:T
                                            MainStack(:,:,:,n,t) = ...
                                                MainStack(:,:,:,n,T2);
                                        end;
                                    end;
                                end;
                                % Save the image
                                FullFileName = [PathName,filesep,FileName,FileExt];
                                Figure5D.SaveStack2BigTiff(FullFileName,MainStack,obj.Stack.PixelSize);
                        end
                    end
                end;
                obj.LogHistory.FinishAll;
                close(hD.Fig);
            end;
            % Enable every figure
            obj.EnableEverything(true);
        end;

        % =================================================================
        % Save Figure Dialog - Checkbox callback
        function SaveFigureCheckboxCallback(obj,hD,n)
            % INPUT ARGUMENTS:
            % n:             checkbox index
            % --------------------------------------------------------------
            switch n
                case 1
                    if get(hD.Checkbox(n),'Value')
                        hD.ChannelsFileDistribution.String = hD.ChannelsFileDistribution.UserData;
                    else
                        if hD.ChannelsFileDistribution.Value > 2
                            hD.ChannelsFileDistribution.Value = 1;
                        end
                        hD.ChannelsFileDistribution.String = hD.ChannelsFileDistribution.UserData(1:2);
                    end
                case 3
                    if get(hD.Checkbox(n),'Value')
                        hD.AdditionalChannels.Enable = 'on';
                        hD.ChannelsFileDistribution.Enable ='on';
                    else
                        hD.AdditionalChannels.Enable = 'off';
                        hD.ChannelsFileDistribution.Enable ='off';
                    end
            end
            
        end
        
        % =================================================================
        
        function RoiManager_ColorDialog(obj)
            % Dialog to change the ROI selections color
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 6*GUIP.NormalEditHeight;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','ROI''s color...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );    
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            PosT = [GUIP.DialogMargin(1),0,Size(1)/2-GUIP.DialogMargin(1),GUIP.NormalTextHeight];
            PosE = [Size(1)/2,0,Size(1)/2-GUIP.DialogMargin(1),GUIP.NormalEditHeight];
            YT = Size(2) - GUIP.DialogMargin(2) - (0:2)*2*GUIP.NormalEditHeight;
            TextStrings = {'Red (0..255):';'Green (0..255):';'Blue (0..255):'};
            S.MinMax = [0,255];
            S.Rounded = true;
            for n = 1:3
                % Texts:
                PosT(2) = YT(n)-GUIP.NormalEditHeight;
                uicontrol(hD.Fig,'Style','text'...
                    ,'String',TextStrings{n}...
                    ,'HorizontalAlignment','left'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',PosT);
                % Edits:
                S.Old = obj.Roi.Color(n);
                PosE(2) = YT(n)-GUIP.NormalEditHeight;
                hD.Edit(n) = uicontrol(hD.Fig,'Style','edit'...
                    ,'String',num2str(S.Old) ...
                    ,'UserData',S ...
                    ,'BackgroundColor','white'...
                    ,'Enable','on'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Callback',@(Source,Event)obj.EditNumCallback(Source)...
                    ,'Units','points'...
                    ,'Position',PosE);
            end;
            % Buttons
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 2*GUIP.NormalButtonMargin(1) - 3*GUIP.NormalButtonSize(1);
            PosE(2) = GUIP.DialogMargin(2);
            PosE(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Remove'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback',@(Source,Event)obj.VoxelSizeDialog_Remove(hD));
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);            
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','close(gcbf);');
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                for n = 1:3
                    S = get(hD.Edit(n),'UserData');
                    obj.Roi.Color(n) = S.Old;
                end
                CData = get(obj.hImage.Roi,'CData');
                CData(:,:,1) = obj.Roi.Color(1);
                CData(:,:,2) = obj.Roi.Color(2);
                CData(:,:,3) = obj.Roi.Color(3);
                set(obj.hImage.Roi,'CData',CData);
                close(hD.Fig);
            end;
            % Enable every figure
            obj.EnableEverything(true);
        end;

        % =================================================================
        
        function VoxelSizeDialog(obj,~,~)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Check for child images (map)
            % -------------------------------------------------------------            
            InformChildImages = false;
            if ~isempty(obj.ChildImages)
                Button = questdlg({'There are other images/maps associated with this image!'...
                    ;'If you change the voxel size, then the associated images should be manually updated!'...
                    ;'Do you really want to continue?'}...
                    ,'Warning!!!','Yes','No','No');
                if strcmp(Button,'No')
                    return
                else
                    InformChildImages = true;
                end
            end
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 6*GUIP.NormalEditHeight;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','Voxel size...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            PosT = [GUIP.DialogMargin(1),0,Size(1)/2-GUIP.DialogMargin(1),GUIP.NormalTextHeight];
            PosE = [Size(1)/2,0,Size(1)/2-GUIP.DialogMargin(1),GUIP.NormalEditHeight];
            YT = Size(2) - GUIP.DialogMargin(2) - (0:2)*2*GUIP.NormalEditHeight;
            TextStrings = {'Voxel width (X):';'Voxel height (Y):';'Voxel depth (Z):'};
            S.Rounded = false;
            S.MinMax = [0,Inf];
            for n = 1:3
                % Texts:
                PosT(2) = YT(n)-GUIP.NormalEditHeight;
                uicontrol(hD.Fig,'Style','text'...
                    ,'String',TextStrings{n}...
                    ,'HorizontalAlignment','left'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Units','points'...
                    ,'Position',PosT);
                % Edits:
                PosE(2) = YT(n)-GUIP.NormalEditHeight;
                S.Old = obj.Stack.PixelSize(n);
                hD.Edit(n) = uicontrol(hD.Fig,'Style','edit'...
                    ,'String',num2str(S.Old) ...
                    ,'UserData',S ...
                    ,'BackgroundColor','white'...
                    ,'Enable','on'...
                    ,'FontSize',GUIP.NormalFontSize ...
                    ,'Callback',@(Source,Event)obj.EditNumCallback(Source)...
                    ,'Units','points'...
                    ,'Position',PosE);
            end;
            % Buttons
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 2*GUIP.NormalButtonMargin(1) - 3*GUIP.NormalButtonSize(1);
            PosE(2) = GUIP.DialogMargin(2);
            PosE(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Remove'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback',@(Source,Event)obj.VoxelSizeDialog_Remove(hD));
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);            
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','close(gcbf);');
            PosE(1) = Size(1) - GUIP.DialogMargin(1) - 1*GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units','points'...
                ,'Position',PosE...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                for n = 1:3
                    S = get(hD.Edit(n),'UserData');
                    obj.Stack.PixelSize(n) = S.Old;
                end;
                % Inform the child images
                if InformChildImages
                    for n = 1:length(obj.ChildImages)
                        if isprop(obj.ChildImages{n},'NextUpdate') && ~isa(obj.ChildImages{n},'DistanceMap')
                            obj.ChildImages{n}.NextUpdate = 1;
                        end
                    end
                end
                close(hD.Fig);                
            end;
            % Enable every figure
            obj.EnableEverything(true);
        end;
        
        % =================================================================

        function BackgroundDialog(obj,~,~)
            % Dialog to adjust pixel size
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 4* GUIP.DialogMargin(2) ...
                + GUIP.NormalButtonSize(2) + 2*GUIP.NormalEditHeight;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 3*GUIP.NormalButtonSize(1) + 2*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(obj.hFig,'Units');
            set(obj.hFig,'Units','points');
            SC = get(obj.hFig,'Position');
            set(obj.hFig,'Units',OldUnits);
            hD.Fig = dialog('Name','Background...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                );
            % Disable every figure
            obj.EnableEverything(false,hD.Fig);
            hD.Fig.Visible = 'on';
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            Pos = [0,0,0,0];
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Size(2) - GUIP.DialogMargin(2) - GUIP.NormalEditHeight;
            Pos(3) = Size(1)/2 - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(hD.Fig,'Style','text'...
                ,'String','Background value:' ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            % Edits:
            Pos(1) = Size(1)/2;
            Pos(4) = GUIP.NormalEditHeight;
            S.Rounded = false;
            S.MinMax = [-inf,inf];
            S.Old = obj.Stack.ChannelBackground(obj.Stack.CurrentZCT(2));
            hD.Edit = uicontrol(hD.Fig,'Style','edit'...
                ,'String',num2str(S.Old) ... % num2str(obj.BackgroundValue) ...
                ,'UserData',S ... % obj.BackgroundValue ...
                ,'BackgroundColor','white','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Callback',@(Source,Event)obj.EditNumCallback(Source)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Checkbox
            Pos(1) = Size(1)/4;
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalTextHeight;
            Pos(3) = Size(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            hD.Checkbox = uicontrol(hD.Fig,'Style','checkbox'...
                ,'String','Apply to all channels','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos);
            % Buttons
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonMargin(1) - 2*GUIP.NormalButtonSize(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3:4) = GUIP.NormalButtonSize;
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Cancel'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','close(gcbf);');
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - GUIP.NormalButtonSize(1);                        
            uicontrol(hD.Fig,'Style','pushbutton'...
                ,'String','Apply'...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback','uiresume(gcbf);');
            uiwait(hD.Fig);
            % Saving process
            if ishghandle(hD.Fig)
                S = get(hD.Edit,'UserData');
                if get(hD.Checkbox,'Value')
                    obj.Stack.ChannelBackground(:) = S.Old;
                else
                    obj.Stack.ChannelBackground(obj.Stack.CurrentZCT(2)) = S.Old;
                end;
                close(hD.Fig);
                obj.ShowImage;
            end;
            % Enable every figure
            obj.EnableEverything(true);
        end;
        
        % =================================================================        
        
        function EnableEverything(obj,Flag,DialogHandle)
            % Enable/Disable all the figures
            % Flag:             can be true,false,1,0            
            % DialogHandle:     inform dialog handle
            % -------------------------------------------------------------
            if nargin < 3 || ~ishghandle(DialogHandle)
                DialogHandle = [];
            end
            obj.EnableFigure(Flag);
            if Flag
                set(obj.LogHistory.hButtons(end),'Enable','off');
                obj.LogHistory.CurrentDialog = {};
            else
                set(obj.LogHistory.hButtons(end),'Enable','on');
                obj.LogHistory.CurrentDialog{1} = DialogHandle;
            end
            for n = 1:length(obj.ChildImages)
                obj.ChildImages{n}.EnableFigure(Flag);
            end;
            if ~isempty(obj.ParentImage)
                obj.ParentImage.EnableFigure(Flag);
            end
        end;
        
        % =================================================================
        
        function CloseFigure(obj,~,~)
            % Function to close the figure
            % -------------------------------------------------------------
            Button = questdlg({'Are you sure?';'All dependant figures/objects will also be closed!'}...
                ,'Warning!!!','Yes','Cancel','Cancel');
            if strcmp(Button,'Yes')
                delete(obj);
            end;
        end;

        % =================================================================
        
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            delete(obj.hFig);
            obj.hFig = [];
            obj.hImage.Axes = [];
            if obj.LogHistory.Parent == obj
                delete(obj.LogHistory);
            end;
            if ~isempty(obj.ChildDataTable)
                delete(obj.ChildDataTable);
            end
        end;
        
        
    end
    
    
    methods (Static = true)
        
        function VoxelSizeDialog_Remove(hD)
            % Reset Pixel Size to (1,1,1)
            % -------------------------------------------------------------
            S = get(hD.Edit(1),'UserData');
            S.Old = 1;
            set(hD.Edit(1),'String','1','UserData',S);
            set(hD.Edit(2),'String','1','UserData',S);
            set(hD.Edit(3),'String','1','UserData',S);
        end;
        
        function RoiManager_DrawForeground(hD,Option)
            % RoiManager Pixel Size to (1,1,1)
            % -------------------------------------------------------------
            if Option == 1
                if get(hD.ForegroundCheckbox,'Value')
                    set(hD.SameForegroundCheckbox,'Enable','on');
                    Option = 2;
                else
                    set(hD.SameForegroundCheckbox,'Enable','off');
                    set(hD.ForegroundEdit,'Enable','off');                    
                end;
            end;
            if Option == 2
                if get(hD.SameForegroundCheckbox,'Value')
                    set(hD.ForegroundEdit,'Enable','off');
                else                    
                    set(hD.ForegroundEdit,'Enable','on');
                end;
            end;
        end
        
        % -----------------------------------------------------------------
        % Add Mask Dialog Callback
        function AddMaskDialogCallback(hD,n)
            if n == 1
                hD.ReferenceChannel.Enable = 'off';
                hD.ReferenceFrame.Enable = 'off';
            else
                hD.ReferenceChannel.Enable = 'on';
                hD.ReferenceFrame.Enable = 'on';
            end
        end        
        
    end

    methods (Access = public, Static = true)    

        function SaveStack2BigTiff(Filename,I,VoxelSize)
            % Save Stack to a BigTiff file 
            % INPUT ARGUMENTS
            % Filename:     Target file
            % I:            5D Stack
            % -------------------------------------------------------------
            if nargin < 3
                VoxelSize = [1,1,1];
            end
            Channels = size(I,4);
            Slices = size(I,3);
            Frames  = size(I,5);
            % -------------------------------------------------------------
            % Create Tag structure
            % -------------------------------------------------------------
            TagStruct.ImageLength = size(I,1);
            TagStruct.ImageWidth = size(I,2);
            TagStruct.Photometric = Tiff.Photometric.MinIsBlack;
            switch class(I)
                case 'uint8'
                    TagStruct.BitsPerSample = 8;
                    TagStruct.SampleFormat = Tiff.SampleFormat.UInt;
                case 'uint16'
                    TagStruct.BitsPerSample = 16;
                    TagStruct.SampleFormat = Tiff.SampleFormat.UInt;
                case 'uint32'
                    TagStruct.BitsPerSample = 32;
                    TagStruct.SampleFormat = Tiff.SampleFormat.UInt;
                case 'single'
                    TagStruct.BitsPerSample = 32;
                    TagStruct.SampleFormat = Tiff.SampleFormat.IEEEFP;
                case {'double'}
                    I = single(I);  % Compatibility with ImageJ
                    TagStruct.BitsPerSample = 32;
                    TagStruct.SampleFormat = Tiff.SampleFormat.IEEEFP;
            end;
            TagStruct.PlanarConfiguration = Tiff.PlanarConfiguration.Chunky;
            TagStruct.Software = 'MATLAB';
%             TagStruct.ResolutionUnit = Tiff.ResolutionUnit.None;
%             TagStruct.XResolution = 11.2994;
%             TagStruct.YResolution = 11.2994;
            TagStruct.ResolutionUnit = 1;%Tiff.ResolutionUnit.None;
            TagStruct.XResolution = 1/ VoxelSize(1);
            TagStruct.YResolution = 1/ VoxelSize(2);
            Images = Channels*Slices*Frames;
            Desc = ['ImageJ=X' ...  
                    '\nimages=' num2str(Images) ...
                    '\nchannels=' num2str(Channels) ...
                    '\nslices=' num2str(Slices) ...
                    '\nframes=' num2str(Frames) ...
                    '\nhyperstack=true' ...
                    '\nmode=grayscale' ...
                    '\nspacing=' num2str(VoxelSize(3)) ...
                    '\nloop=false' ...
                    '\n'];
            TagStruct.ImageDescription = sprintf(Desc);
            % -------------------------------------------------------------
            % Create image
            % -------------------------------------------------------------
            TiffObj = Tiff(Filename,'w8');
            % -------------------------------------------------------------
            % Write images
            % -------------------------------------------------------------
            i = 1;
            for t = 1:Frames
                for z = 1:Slices
                    for c = 1:Channels
                        TiffObj.setTag(TagStruct);
                        TiffObj.write(I(:,:,z,c,t));
                        if i < Images
                            TiffObj.writeDirectory;
                            i = i + 1;
                        end;
                    end;
                end;
            end;
            % -------------------------------------------------------------
            % Close file
            % -------------------------------------------------------------
            TiffObj.close();
        end;
        
        function [I,VoxelSize] = LoadTiffStack(Filename,czt)
            % Read an image or the whole Stack from a Tiff/BigTiff file 
            % INPUT ARGUMENTS
            % Filename:     It can be:
            %                   Source Filename, or
            %                   Object to an opened file
            % czt:          3-el vector with [Channel, Slice, Frame] of one
            %               image
            % -------------------------------------------------------------
            if nargin < 2
                czt = [];
            end;
            % -------------------------------------------------------------
            % Open Tiff file & get basic information
            % -------------------------------------------------------------
            wstate = warning('off','MATLAB:imagesci:tiffmexutils:libtiffWarning');
%             [TiffObj,TiffInfo] = Image5D.OpenTiff(Filename);
            [TiffObj,TiffInfo] = Figure5D.OpenTiff(Filename);
            VoxelSize = TiffInfo.VoxelSize;
            % -------------------------------------------------------------
            % Read full stack or single image?
            % -------------------------------------------------------------
            if isempty(czt)
                ReadAll = true;
            else
                if length(czt) ~= 3
                    ReadAll = true;
                elseif (czt(1) < 1) || (czt(1) > TiffInfo.Channels*TiffInfo.SamplesPerPixel)
                    ReadAll = true;
                elseif (czt(2) < 1) || (czt(2) > TiffInfo.Slices)
                    ReadAll = true;
                elseif (czt(3) < 1) || (czt(3) > TiffInfo.Frames)
                    ReadAll = true;
                else
                    ReadAll = false;
                end;
            end;
            % -------------------------------------------------------------
            % Read Images
            % -------------------------------------------------------------
            if ReadAll
                Images = TiffInfo.Channels*TiffInfo.Slices*TiffInfo.Frames;
                TiffObj.setDirectory(1);
                I = zeros(TiffInfo.Height,TiffInfo.Width,TiffInfo.Slices ...
                    ,TiffInfo.Channels*TiffInfo.SamplesPerPixel,TiffInfo.Frames,TiffInfo.Dataclass);
                i = 1;
                for t = 1:TiffInfo.Frames
                    for s = 1:TiffInfo.Slices
                        for c = 1:TiffInfo.Channels
                            I(:,:,s,(c-1)*TiffInfo.SamplesPerPixel+1:c*TiffInfo.SamplesPerPixel,t) = TiffObj.read();
                            if i ~= Images
                                TiffObj.nextDirectory();
                            end;
                            i = i + 1;
                        end;
                    end;
                end;
            else
                I = zeros(TiffInfo.Height,TiffInfo.Width,TiffInfo.Dataclass);
                czt(1) = floor(czt(1)-1/TiffInfo.SamplesPerPixel);
                Offset = rem(czt(1)-1,TiffInfo.SamplesPerPixel)+1;
                ii = Figure5D.Tiff_czt2n(TiffInfo,czt);
                TiffObj.setDirectory(ii);
                IRGB = TiffObj.read();
                I(:,:) = IRGB(:,:,Offset);
            end;
            % -------------------------------------------------------------
            % Close Tiff file
            % -------------------------------------------------------------
            TiffObj.close();
            warning(wstate);
        end;
        
        function [TiffObj,TiffInfo] = OpenTiff(Filename)
            % Read an image or the whole Stack from a Tiff/BigTiff file 
            % INPUT ARGUMENTS
            % Filename:     It can be:
            %                   Source Filename, or
            %                   Object to an opened file
            % czt:          3-el vector with [Channel, Slice, Frame] of one
            %               image
            % OUTPUT ARGUMENTS
            % TiffInfo:
            %   .Width          Width in pixels
            %   .Height         Height in pixels
            %   .Channels:      Number of channels in the file
            %   .Slices:        Number of channels in the file
            %   .Frames:        Number of channels in the file
            %   .Dataclass:     String with the class of the image data
            % -------------------------------------------------------------
            TiffObj = [];
            TiffInfo = [];
            if ~exist(Filename,'file')
                disp('Error: Filename not found!...');
                return;
            end;
            try
                TiffObj = Tiff(Filename,'r');
            catch
                disp('Error: the file cannot be recognized as Tiff/BigTiff...');
                return;
            end;
            % -------------------------------------------------------------
            % Determine basic information from the file header
            % -------------------------------------------------------------
            [TiffInfo] = Figure5D.getTiffInfo(TiffObj);
            if isempty(TiffInfo)
                TiffObj.close();
                TiffObj = [];
            end;
        end;        
        
        function [TiffInfo] = getTiffInfo(TiffObj)
            % Read an image or the whole Stack from a Tiff/BigTiff file 
            % INPUT ARGUMENTS
            % TiffObj:      Tiff object
            % OUTPUT ARGUMENTS
            % TiffInfo:
            %   .Width          Width in pixels
            %   .Height         Height in pixels
            %   .Channels:      Number of channels in the file
            %   .Slices:        Number of channels in the file
            %   .Frames:        Number of channels in the file
            %   .Dataclass:     String with the class of the image data
            %   .RGB            True if it is an RGB image
            % -------------------------------------------------------------
            % Number of images
            % -------------------------------------------------------------
            while ~TiffObj.lastDirectory()
                TiffObj.nextDirectory();
            end;
            Images = TiffObj.currentDirectory();
            % -------------------------------------------------------------
            % Read image size & class
            % -------------------------------------------------------------
            TiffObj.setDirectory(1);
            try
                TiffInfo.Width = TiffObj.getTag('ImageWidth');
                TiffInfo.Height = TiffObj.getTag('ImageLength');
                BitsPerSample = TiffObj.getTag('BitsPerSample');
                SampleFormat = TiffObj.getTag('SampleFormat');
                TiffInfo.SamplesPerPixel = TiffObj.getTag('SamplesPerPixel');
            catch
                disp('Error: Tiff dimensions not recognized...');
                TiffInfo = [];
                return;
            end;
            % -------------------------------------------------------------
            % Read image size & class
            % -------------------------------------------------------------            
            TiffInfo.VoxelSize = [1,1,1];
            try
                TiffInfo.VoxelSize(1) = 1/TiffObj.getTag('XResolution');
                TiffInfo.VoxelSize(2) = 1/TiffObj.getTag('YResolution');
            catch
            end;            
            % -------------------------------------------------------------
            % Read stack information
            % -------------------------------------------------------------
            try
                ImageDesc = TiffObj.getTag('ImageDescription');
                DescCell = textscan(ImageDesc,'%s %s','Delimiter','=');
            catch
                DescCell = [];
            end;
            if isempty(ImageDesc)
                TiffInfo.Channels = 1;
                TiffInfo.Slices = Images;
                TiffInfo.Frames = 1;
            else
                try
                    % Slices spacing
                    Pos = find(strcmpi('spacing',DescCell{1}),1,'first');
                    if ~isempty(Pos)
                        TiffInfo.VoxelSize(3)= str2double(DescCell{2}{Pos});
                    end;
                catch
                end
                % Number of channels
                Pos = find(strcmpi('channels',DescCell{1}),1,'first');
                if ~isempty(Pos)
                    TiffInfo.Channels = str2double(DescCell{2}{Pos});
                else
                    TiffInfo.Channels = 1;
                end;
                % Number of slices
                Pos = find(strcmpi('slices',DescCell{1}),1,'first');
                if ~isempty(Pos)
                    TiffInfo.Slices = str2double(DescCell{2}{Pos});
                else
                    TiffInfo.Slices = 1;
                end;
                % Number of frames
                Pos = find(strcmpi('frames',DescCell{1}),1,'first');
                if ~isempty(Pos)
                    TiffInfo.Frames = str2double(DescCell{2}{Pos});
                else
                    TiffInfo.Frames = 1;
                end;
                % Check consistency between Images and Frames, Slices & Channels
                if TiffInfo.Frames*TiffInfo.Slices*TiffInfo.Channels ~= Images
                    TiffInfo.Channels = 1;
                    TiffInfo.Slices = Images;
                    TiffInfo.Frames = 1;
                end;
                
            end;
            % -------------------------------------------------------------
            % Determine Matlab class
            % -------------------------------------------------------------
            TiffInfo.Dataclass = '';
            switch SampleFormat
                case TiffObj.SampleFormat.UInt
                    switch BitsPerSample
                        case 8
                            TiffInfo.Dataclass = 'uint8';
                        case 16
                            TiffInfo.Dataclass = 'uint16';
                        case 32
                            TiffInfo.Dataclass = 'uint32';
                    end;
                case TiffObj.SampleFormat.Int
                    switch BitsPerSample
                        case 8
                            TiffInfo.Dataclass = 'int8';
                        case 16
                            TiffInfo.Dataclass = 'int16';
                        case 32
                            TiffInfo.Dataclass = 'int32';
                    end;
                case TiffObj.SampleFormat.IEEEFP
                    switch BitsPerSample
                        case 32
                            TiffInfo.Dataclass = 'single';
                        case 64
                            TiffInfo.Dataclass = 'double';
                    end;
                otherwise
                    disp('Error: Unsuported data type...');
            end;
        end;        

        
        function [n] = Tiff_czt2n(TiffInfo,czt)
            % Convert czt image index to linear Tiff index
            % INPUT ARGUMENTS
            % TiffInfo:
            %   .Width          Width in pixels
            %   .Height         Height in pixels
            %   .Channels:      Number of channels in the file
            %   .Slices:        Number of channels in the file
            %   .Frames:        Number of channels in the file
            %   .Dataclass:     String with the class of the image data
            % czt:              3-el vector with [Channel, Slice, Frame] 
            %                   of one image
            % OUTPUT ARGUMENTS
            % n:                Linear index
            % -------------------------------------------------------------
            n = (czt(3)-1)*TiffInfo.Slices*TiffInfo.Channels ...
                + (czt(2)-1)*TiffInfo.Channels + czt(1);        
        end;
        
        function [I] = ReadTiffImage(TiffObj,TiffInfo,czt)
            % Read an image or the whole Stack from a Tiff/BigTiff file 
            % INPUT ARGUMENTS
            % TiffObj:          Tiff object
            % TiffInfo:
            %   .Width          Width in pixels
            %   .Height         Height in pixels
            %   .Channels:      Number of channels in the file
            %   .Slices:        Number of channels in the file
            %   .Frames:        Number of channels in the file
            %   .Dataclass:     String with the class of the image data
            % czt:              3-el vector with [Channel, Slice, Frame] 
            %                   of one image
            % OUTPUT ARGUMENTS
            % I:                Output image
            % -------------------------------------------------------------
            if nargin < 2
                czt = [1,1,1];
            end;
            % -------------------------------------------------------------
            % Read Image
            % -------------------------------------------------------------
            I = zeros(TiffInfo.Height,TiffInfo.Width,TiffInfo.Dataclass);
            ii = Figure5D.Tiff_czt2n(TiffInfo,czt);
            TiffObj.setDirectory(ii);
            I(:,:) = TiffObj.read();
        end;

        function Changed = EditNumCallback(Source)
            % Update Edits in Property Dialog function  
            % UserData: S.Old       (numeric)
            %           S.Rounded   (logical)
            %           S.MinMax    [Min,Max]  (numeric)
            % -------------------------------------------------------------
            S = get(Source,'UserData');
            Num = str2double(get(Source,'String'));
            if S.Rounded
                Num = round(Num);
            end;
            if isfinite(Num) && Num >= S.MinMax(1) && Num <= S.MinMax(2)
                if Num ~= S.Old
                    Changed = true;
                    S.Old = Num;
                    set(Source,'String',num2str(Num),'UserData',S);
                else
                    Changed = false;
                end;
            else
                Changed = false;
                set(Source,'String',num2str(S.Old),'UserData',S);
            end;
        end;          
        
        
       function RemoveMenubarItem(MenuStruct,Index)
            % Remove item from the Menubar structure
            % -------------------------------------------------------------
            MenuStruct.Label =      MenuStruct.Label([1:Index-1,Index+1:end]);
            MenuStruct.Separator =  MenuStruct.Separator([1:Index-1,Index+1:end]);
            MenuStruct.Enable =     MenuStruct.Enable([1:Index-1,Index+1:end]);
            MenuStruct.Callback =   MenuStruct.Callback([1:Index-1,Index+1:end]);
            MenuStruct.hMenu =      MenuStruct.hMenu([1:Index-1,Index+1:end]);            
        end;
        
        
        function MenuStruct = InsertMenubarItem(MenuStruct,Index,Label,Separator,Enable,Callback,hMenu)
            % Insert item in the Menubar structure
            % -------------------------------------------------------------
            if nargin < 7
                hMenu = 0;
            end;
            if nargin < 6
                Callback = {};
            end;
            if nargin < 5
                Enable = true;
            end;
            if nargin < 4
                Separator = false;
            end;
            if nargin < 3
                Label = '';
            end;
            N = length(MenuStruct.Label);
            MenuStruct.Label(Index+1:N+1) = MenuStruct.Label(Index:N);
            MenuStruct.Separator(Index+1:N+1) = MenuStruct.Separator(Index:N);
            MenuStruct.Enable(Index+1:N+1) = MenuStruct.Enable(Index:N);
            MenuStruct.Callback(Index+1:N+1) = MenuStruct.Callback(Index:N);
            MenuStruct.hMenu(Index+1:N+1) = MenuStruct.hMenu(Index:N);
            % Default values
            MenuStruct.Label{Index} = Label;
            MenuStruct.Separator(Index) = Separator;
            MenuStruct.Enable(Index) = Enable;
            MenuStruct.Callback{Index} = Callback;
            MenuStruct.hMenu(Index) = hMenu;
        end;        
        
        
    end
    
    
end


% -------------------------------------------------------------------------
% Recursive function to create menubar
% -------------------------------------------------------------------------
function MB = AddMenu(MStruct,Parent)
    L = length(MStruct.Label);
    MB = cell(L+1,1);
    MB{1} = Parent;
    State = {'off','on'};
    for k = 1:L
        MB{k+1}{1} = uimenu(Parent...
            ,'Label',       MStruct.Label{k}...
            ,'Separator',   State{MStruct.Separator(k)+1} ...
            ,'Enable',      State{MStruct.Enable(k)+1});
        if isstruct(MStruct.Callback{k})
            MB{k+1} = AddMenu(MStruct.Callback{k},MB{k+1}{1});
        else
            set(MB{k+1}{1},'Callback',MStruct.Callback{k});
        end;
    end;
end
