classdef ImageRoi
    % ImageRoi class to encapsulate operation with ROIs
    %   Detailed explanation goes here
    %
    % Version:  0.0.1 (27.01.2014)
    
    properties
        RoiName
        RoiImageSize                     % [Height,Width,Pages]
        RoiIndexes
        Slices = uint16(0)               % Slices information
        Channels = uint16(1)             % Channels information
        Frames = uint16(1)               % Frames information
        Is2D = false;
        LastZ = 0;
    end
    
    properties (Dependent)
        RoiCoords                       % [r,Channels,Slices] list of pixels (uint16);
    end
    
    methods
        
        % =================================================================
        function obj = ImageRoi(ImageSize,L,Channel,Frame,Slice)
            % Constructor
            % INPUT ARGUMENTS:
            % ImageSize:        Size of the original image, needed always
            % L:                List of pixels (optional). It can be:
            %                       n-by-1: Assumed as 3D linear indexes
            %                       n-by-3: Assumed as 3D ROI
            % Channel:          Channel(s) (optional)
            % Frame:            Frame(s) (optional)
            % Slice:            Slice(s) (optional)
            % -------------------------------------------------------------
            if nargin < 5
                Slice = [];
            end
            if nargin < 4
                Frame = 1;
            end;
            if nargin < 3
                Channel = 1;
            end;
            if nargin < 1
                delete(obj);
            end;
            obj.RoiImageSize = ImageSize(1:3);
            if nargin > 1 && ~isempty(L);
                if isvector(L)
%                     ImageRoi.AddIndexes(L,Channel,Frame);
                    obj = obj.AddIndexes(L,Channel,Frame,Slice);
                else
                    Indexes = sub2ind(obj.RoiImageSize,L(:,1),L(:,2),L(:,3));
%                     ImageRoi.AddIndexes(Indexes,Channel,Frame,unique(L(:,3)));
                    obj = obj.AddIndexes(Indexes,Channel,Frame,unique(L(:,3)));
                end;
            end;
        end;
        
        % =================================================================
        function L = get.RoiCoords(obj)
            % Calculate de current list of coordinates (Row,Col,Pag)
            % OUTPUT ARGUMENTS:
            % L                 If Slices is specified
            %                       n-by-2 [Row,Col] pixel coordinates
            %                   otherwise
            %                       n-by-3 [Row,Col,Pag] pixel coordinates
            % -------------------------------------------------------------
            L = uint16(zeros(length(obj.RoiIndexes),3));
            [L(:,1),L(:,2),L(:,3)] = ind2sub(obj.RoiImageSize,obj.RoiIndexes);
            L = uint16(L);
        end;
        
        function L = GetRoiIndexesZ(obj,Z,Show2D)
            % Extract the list of Indexes for the current Z.
            % It is faster because it assumes same image size 
            % INPUT ARGUMENTS (optional):
            % Z:                Desired slice (Z plane)
            % Show2D:           Also return Indexes from 2D selections even 
            %                   if they were not in the same plane
            % OUTPUT ARGUMENTS:
            % L                 List of linear indexes
            % -------------------------------------------------------------
            if nargin < 3
                Show2D = true;
            end;
            if nargin < 2
                Z = 0;
            end;
            L = [];
            ImageSize = uint32(obj.RoiImageSize);
            if (Z ~= 0)
                if obj.Slices(1) == 0 || any(Z==obj.Slices);
                    Z = uint32(Z);
                    Limits = sub2ind(obj.RoiImageSize,[1,ImageSize(1)],[1,ImageSize(2)],[Z,Z]);
                    SubList = (obj.RoiIndexes >= Limits(1)) & (obj.RoiIndexes <= Limits(2));
                    L = obj.RoiIndexes(SubList);
                elseif obj.Is2D && Show2D
                    %                     L = obj.RoiIndexes;
                    L = obj.RoiIndexes + (Z-1)*obj.RoiImageSize(1)*obj.RoiImageSize(2);
                else
                    return;
                end;
            else
                if obj.Is2D && Show2D
                    if Show2D
                        L = obj.RoiIndexes;
                    else
                        return;
                    end;
                else
                    L = obj.RoiIndexes;
                end;
            end;
        end;
        
        function L = GetRoi2DIndexesZ(obj,Z,OutputImageSize,Show2D,Extended)
            % Calculate de current list of coordinates (Row,Col,Pag)
            % Version: 0.7 (10.11.2014)
            % INPUT ARGUMENTS (optional):
            % Z:                Desired slice (Z plane)
            % OutputImageSize:  1-by-3 (Rows,Cols,Pags) of the output image
            %                   Default( OutputImageSize = obj.RoiImageSize
            % Show2D:           Also return Indexes from 2D selections even 
            %                   if they were not in the same plane
            %                   (default true)
            % Extended:         Consider a one pixel image margin on every
            %                   direction
            % OUTPUT ARGUMENTS:
            % L                 List of linear indexes
            % -------------------------------------------------------------
            if nargin < 5
                Extended = false;
            end
            if nargin < 4
                Show2D = true;
            end;
            if (nargin < 3) || isempty(OutputImageSize)
%                 OutputImageSize = [];
                OutputImageSize = obj.RoiImageSize;
            end;
            L = [];
            if nargin < 2
                return;
            end;
%             if isempty(OutputImageSize)
%                 OutputImageSize = obj.RoiImageSize;
%             end;
            L = [];
            ImageSize = uint32(obj.RoiImageSize);
            if obj.Is2D
                if (Z == obj.Slices) || Show2D
                    [Row,Col,~] = ind2sub(ImageSize,obj.RoiIndexes);
                else
                    return
                end;
            elseif obj.Slices(1) == 0 || any(Z==obj.Slices);
                Z = uint32(Z);
                LimitIndex = sub2ind(obj.RoiImageSize,[1,ImageSize(1)],[1,ImageSize(2)],[Z,Z]);
                SubList = (obj.RoiIndexes >= LimitIndex(1)) & (obj.RoiIndexes <= LimitIndex(2));
                [Row,Col,~] = ind2sub(ImageSize,obj.RoiIndexes(SubList));
            else
                return;
            end;
            if Extended
                L = sub2ind(OutputImageSize(1:2)+2,Row+1,Col+1);
            else
                L = sub2ind(OutputImageSize(1:2),Row,Col);
            end
        end

        
        function L = GetFirstRoiCoord(obj)
            % Calculate de current list of coordinates (Row,Col,Pag)
            % OUTPUT ARGUMENTS:
            % L                 1-by-3 (Rows,Cols,Pags) of the first pixel
            % -------------------------------------------------------------
            if isempty(obj.RoiIndexes)
                L = [];
            else
                [L(1,1),L(1,2),L(1,3)] = ind2sub(obj.RoiImageSize,obj.RoiIndexes(1));
                L = uint16(L);
            end
        end;
        
        
        
        % =================================================================
        function obj = AddIndexes(obj,Indexes,Channel,Time,Slices)
            % Add the current 2D imroi selection to the Roi Manager
            % INPUT ARGUMENTS (optional):
            % Indexes:          List of 3D (linear) indexes
            % Channel:          ROI channel(s)
            % Time:             ROI frame(s)
            % Slices:           (optional) list of known slices to speed up
            % -------------------------------------------------------------
            if nargin < 5
                Slices = [];
            end;
            if nargin < 4
                Time = 0;
            end;
            if nargin < 3
                Channel = 0;
            end;
            if nargin < 2
                return;
            end;
            obj.Channels = uint16(Channel);
            obj.Frames = uint16(Time);
            obj.RoiIndexes = Indexes;
            if isempty(Slices)
                [~,~,Pag] = ind2sub(obj.RoiImageSize,Indexes);
                obj.Slices = uint16(unique(Pag));
            else
                obj.Slices = uint16(Slices);
            end;
        end;
        
        function obj = AddSelection(obj,hIm,hRoi,ZCT)
            % Add the current 2D imroi selection to the Roi Manager
            % INPUT ARGUMENTS (optional):
            % hIm:              handle to the "image" object containing the "roi"
            % hRoi:             handle to a "roi" object
            % ZCT:              1-by-3 [Slices,Channels,Frames] values (Default = [1,1,1])
            % -------------------------------------------------------------
            if nargin < 4
                ZCT = [1,1,1];
            end;
            ZCT = uint16(ZCT);
            obj.Slices = ZCT(1);
            obj.Channels = ZCT(2);
            obj.Frames = ZCT(3);
            % Determination of RoiImageSize (if not defined)
            Data = get(hIm,'YData');
            obj.RoiImageSize(1,1) = uint16(Data(2));
            Data = get(hIm,'XData');
            obj.RoiImageSize(1,2) = uint16(Data(2));
            % Getting the Image ROI
            BW = createMask(hRoi,hIm);
            R = regionprops(BW,'PixelIdxList');
            % Subscripts to indexes
%             ZOffset = uint32(ZCT(1)-1)*uint32(obj.RoiImageSize(1))*uint32(obj.RoiImageSize(2));
%             obj.RoiIndexes = uint32(R.PixelIdxList)+ZOffset;
            obj.RoiIndexes = uint32(R.PixelIdxList);
            obj.Is2D = true;
            % Roi Name
            obj = obj.RoiGenericName;
        end;
        
        function obj = RoiGenericName(obj,Prefix)
            % Assign the Generic name to the ROI
            % Version 0.7 (10.11.2014)
            % INPUT ARGUMENTS:
            % Prefix:       Prefix String to the name
            % -------------------------------------------------------------
            if nargin < 2 
                Prefix = '';
            end
            XYZ = obj.GetFirstRoiCoord;
            if isempty(XYZ)
                obj.RoiName = [Prefix,'-Empty'];
            else
                obj.RoiName = [Prefix...
                    ,'T',num2str(obj.Frames(1))...
                    ,'-Z',num2str(XYZ(1,3))...
                    ,'-C',num2str(obj.Channels(1))...
                    ,'-X',num2str(XYZ(1,2))...
                    ,'-Y',num2str(XYZ(1,1))];
            end
        end;        
        
        function NewRoi = Clone(obj)
            % Create a duplicate clone of the object
            % INPUT ARGUMENTS (optional):
            % obj:            Roi handle to be cloned
            % -------------------------------------------------------------
            NewRoi = ImageRoi(obj.RoiIndexes,obj.RoiImageSize);
            NewRoi.RoiName = obj.RoiName;
            NewRoi.Slices = obj.Slices;
            NewRoi.Channels = obj.Channels;
            NewRoi.Frames = obj.Frames;
            NewRoi.Is2D = obj.Is2D;
        end;
        
    end
        
        
    methods (Access = public, Static = true)

        % =================================================================
        % Operations between ROIs
        % -----------------------------------------------------------------

        function NewRoi = AndN(RoiArray)
            % AND operation between multiple ROIs
            % INPUT ARGUMENTS:
            % RoiArray:            Array of Rois to be considered
            % OUTPUT ARGUMENTS:
            % NewRoi:              Output ImageRoi object
            % -----------------------------------------------------------------
            % Check maximum size and compatibility (ROIs belonging to same image)
            Compatibles = true;
            All2D = RoiArray(1).Is2D;
            MaxSize = RoiArray(1).RoiImageSize;
            for n = 2:length(RoiArray)
                if ~isequal(MaxSize,RoiArray(n).RoiImageSize);
                    Compatibles = false;
                    MaxSize = max(MaxSize,RoiArray(n).RoiImageSize);
                end;
                All2D = All2D & RoiArray(n).Is2D;
            end;            
            % AND operation between compatible ROIs, all belonging to the same image
            if Compatibles      
                L = RoiArray(1).RoiIndexes;
                for n = 2:length(RoiArray)
                    N = length(L);
                    LIndex = false(N,1);
                    for ii = 1:N
                        if any(RoiArray(n).RoiIndexes == L(ii))
                            LIndex(ii) = true;
                        end;
                    end;
                    L = L(LIndex);
                end;
            % AND operation between incompatible ROIs
            else
                if isequal(MaxSize,RoiArray(1).RoiImageSize)
                    L = RoiArray(1).RoiIndexes;
                else
                    [Row,Col,Pag] = ind2sub(RoiArray(1).RoiImageSize,RoiArray(1).RoiIndexes);
                    L = sub2ind(MaxSize,Row,Col,Pag);
                end;
                for n = 2:length(RoiArray)
                    N = length(L);
                    LIndex = false(N,1);                    
                    if isequal(MaxSize,RoiArray(n).RoiImageSize)
                        L2 = RoiArray(n).RoiIndexes;
                    else
                        [Row,Col,Pag] = ind2sub(RoiArray(n).RoiImageSize,RoiArray(n).RoiIndexes);
                        L2 = sub2ind(MaxSize,Row,Col,Pag);
                    end;
                    for ii = 1:N
                        if any(L2 == L(ii))
                            LIndex(ii) = true;
                        end;
                    end;
                    L = L(LIndex);
                end;
            end;
            NewRoi = ImageRoi(MaxSize,L);
            NewRoi.Is2D = All2D;
            NewRoi = NewRoi.RoiGenericName('AND-');
        end;
        
        function NewRoi = OrN(RoiArray)
            % OR operation between multiple ROIs
            % INPUT ARGUMENTS:
            % RoiArray:            Array of Rois to be considered
            % OUTPUT ARGUMENTS:
            % NewRoi:              Output ImageRoi object
            % -----------------------------------------------------------------
            % Check maximum size and compatibility (ROIs belonging to same image)
            Compatibles = true;
            N = length(RoiArray(1).RoiIndexes);
            All2D = RoiArray(1).Is2D;            
            MaxSize = RoiArray(1).RoiImageSize;
            for n = 2:length(RoiArray)
                N = N + length(RoiArray(n).RoiIndexes);
                if ~isequal(MaxSize,RoiArray(n).RoiImageSize);
                    Compatibles = false;
                    MaxSize = max(MaxSize,RoiArray(n).RoiImageSize);
                end;
                All2D = All2D & RoiArray(n).Is2D;
            end;
            % Preallocation of the list of indexes Initialization
            L = zeros(N,1,'uint32');
            N = length(RoiArray(1).RoiIndexes);
            % OR operation between compatible ROIs, all belonging to the same image
            if Compatibles      
                L(1:N) = RoiArray(1).RoiIndexes;
                for n = 2:length(RoiArray)
                    N2 = length(RoiArray(n).RoiIndexes);
                    RIndex = true(N2,1);
                    for ii = 1:N2
                        if any(RoiArray(n).RoiIndexes(ii) == L)
                            RIndex(ii) = false;
                        end;
                    end;
                    k = N + 1;
                    N = N + sum(RIndex);
                    L(k:N) = RoiArray(n).RoiIndexes(RIndex);
                end;
            % OR operation between incompatible ROIs
            else
                if isequal(MaxSize,RoiArray(1).RoiImageSize)
                    L(1:N) = RoiArray(1).RoiIndexes;
                else
                    [Row,Col,Pag] = ind2sub(RoiArray(1).RoiImageSize,RoiArray(1).RoiIndexes);
                    L(1:N) = sub2ind(MaxSize,Row,Col,Pag);
                end;
                for n = 2:length(RoiArray)
                    if isequal(MaxSize,RoiArray(n).RoiImageSize)
                        L2 = RoiArray(n).RoiIndexes;
                    else
                        [Row,Col,Pag] = ind2sub(RoiArray(n).RoiImageSize,RoiArray(n).RoiIndexes);
                        L2 = sub2ind(MaxSize,Row,Col,Pag);
                    end;
                    N2 = length(L2);
                    RIndex = true(N2,1);
                    for ii = 1:N2
                        if any(L2(ii) == L)
                            RIndex(ii) = false;
                        end;
                    end;
                    k = N + 1;
                    N = N + sum(RIndex);
                    L(k:N) = L2(RIndex);
                end;
            end;
            NewRoi = ImageRoi(MaxSize,L(1:N),0,0,0);
            NewRoi.Is2D = All2D;
            NewRoi = NewRoi.RoiGenericName('OR-');            
        end;
        
        function NewRoi = ExOrN(RoiArray)
            % Exclusive OR operation between multiple ROIs
            % INPUT ARGUMENTS:
            % RoiArray:            Array of Rois to be considered
            % OUTPUT ARGUMENTS:
            % NewRoi:              Output ImageRoi object
            % -----------------------------------------------------------------
            % Check maximum size and compatibility (ROIs belonging to same image)
            Compatibles = true;
            N = length(RoiArray(1).RoiIndexes);
            All2D = RoiArray(1).Is2D;            
            MaxSize = RoiArray(1).RoiImageSize;
            for n = 2:length(RoiArray)
                N = N + length(RoiArray(n).RoiIndexes);
                if ~isequal(MaxSize,RoiArray(n).RoiImageSize);
                    Compatibles = false;
                    MaxSize = max(MaxSize,RoiArray(n).RoiImageSize);
                end;
                 All2D = All2D & RoiArray(n).Is2D;
           end;
            % Preallocation of the list of indexes Initialization
            L = zeros(N,1,'uint32');
            N = length(RoiArray(1).RoiIndexes);
            % EX-OR operation between compatible ROIs, all belonging to the same image
            if Compatibles      
                L(1:N) = RoiArray(1).RoiIndexes;
                for n = 2:length(RoiArray)
                    LIndex = false(N,1);
                    N2 = length(RoiArray(n).RoiIndexes);
                    RIndex = true(N2,1);
                    for ii = 1:N2
                        O = RoiArray(n).RoiIndexes(ii) == L(1:N);
                        if any(O)
                            RIndex(ii) = false;
                            LIndex = LIndex | O;
                        end;
                    end;
                    LIndex = ~LIndex;
                    N = sum(LIndex);
                    L(1:N) = L(LIndex);
                    k = N + 1;
                    N = N + sum(RIndex);
                    L(k:N) = RoiArray(n).RoiIndexes(RIndex);
                end;
            % EX-OR operation between incompatible ROIs
            else
                if isequal(MaxSize,RoiArray(1).RoiImageSize)
                    L(1:N) = RoiArray(1).RoiIndexes;
                else
                    [Row,Col,Pag] = ind2sub(RoiArray(1).RoiImageSize,RoiArray(1).RoiIndexes);
                    L(1:N) = sub2ind(MaxSize,Row,Col,Pag);
                end;
                for n = 2:length(RoiArray)
                    if isequal(MaxSize,RoiArray(n).RoiImageSize)
                        L2 = RoiArray(n).RoiIndexes;
                    else
                        [Row,Col,Pag] = ind2sub(RoiArray(n).RoiImageSize,RoiArray(n).RoiIndexes);
                        L2 = sub2ind(MaxSize,Row,Col,Pag);
                    end;
                    LIndex = false(N,1);
                    N2 = length(L2);
                    RIndex = true(N2,1);
                    for ii = 1:N2
                        O = L2(ii) == L;
                        if any(O)
                            RIndex(ii) = false;
                            LIndex = LIndex | O;
                        end;
                    end;
                    LIndex = ~LIndex;
                    N = sum(LIndex);
                    L(1:N) = L(LIndex);
                    k = N + 1;
                    N = N + sum(RIndex);
                    L(k:N) = L2(RIndex);
                end;
            end;
            NewRoi = ImageRoi(MaxSize,L(1:N),0,0,0);
            NewRoi.Is2D = All2D;
            NewRoi = NewRoi.RoiGenericName('XOR-');
        end;
        
    end
    
end

