function [EPoints] = VDist2EllipsoidSpecial(LSemiAxis,LPoints)
% Function that the distance of each points to an ellipsoid, and the homologous points
% for the first octant
%
% Version:  0.1 (12.03.2014)
%
% INPUT ARGUMENTS
% LsemiAxis:    Local Semiaxis (a>b>c)
% LPoints:      nx3 table with (x,y,z) coordinates of the points
%
% OUTPUT ARGUMENTS
% Dist:         nx1 vector withthe distances from each point to the ellipsoid
% EPoints:      nx3 table with the (x,y,z) coordinates of the homologous points
%               on the ellipsoid
% -------------------------------------------------------------------------
% Initialization
% -------------------------------------------------------------------------
IndexesZ = LPoints(:,3) > 0;
IndexesY = LPoints(:,2) > 0;
IndexesX = LPoints(:,1) > 0;
EPoints = zeros(size(LPoints));
% -------------------------------------------------------------------------
% Case z>0, y>0, x>0
% -------------------------------------------------------------------------
Indexes = IndexesX & IndexesY & IndexesZ;
if any(Indexes)
    EPoints(Indexes,:) = VNumericalDist2Ellipsoid(LSemiAxis,LPoints(Indexes,:));
end
% -------------------------------------------------------------------------
% Case z>0, y>0, x=0
% -------------------------------------------------------------------------
Indexes = ~IndexesX & IndexesY & IndexesZ;
if any(Indexes)
    EPoints(Indexes,2:3) = VDist2EllipseEspecial(LSemiAxis(2:3),LPoints(Indexes,2:3));
end
% -------------------------------------------------------------------------
% Case z>0, y=0, x>0
% -------------------------------------------------------------------------
Indexes = IndexesX & ~IndexesY & IndexesZ;
if any(Indexes)
    EPoints(Indexes,[1,3]) = VDist2EllipseEspecial(LSemiAxis([1,3]),LPoints(Indexes,[1,3]));
end;
% -------------------------------------------------------------------------
% Case z>0, y=0, x=0
% -------------------------------------------------------------------------
Indexes = ~IndexesX & ~IndexesY & IndexesZ;
EPoints(Indexes,3) = LSemiAxis(3);
% -------------------------------------------------------------------------
% Case z=0, y>=0, x>=0
% -------------------------------------------------------------------------
Indexes = ~IndexesZ;
            
% Sub-ellipse boundary:
Num1 = LSemiAxis(1)*LPoints(Indexes,1);
Den1 = LSemiAxis(1)^2 - LSemiAxis(3)^2;     % Can be zero
Num2 = LSemiAxis(2)*LPoints(Indexes,2);
Den2 = LSemiAxis(2)^2 - LSemiAxis(3)^2;     % Can be zero

% -> Subcase 1: "inside ellipse bounding box" 
IndexesX(:) = false;
Indexes2 = (Num1 < Den1) & (Num2 < Den2);
IndexesX(Indexes) = Indexes2;
R1 = Num1(Indexes2)./Den1;                         % Den1 cannot be zero
R2 = Num2(Indexes2)./Den2;                         % Den2 cannot be zero
R3 = 1 - R1.^2 - R2.^2;

% -> -> Subcase 1a: R3 > 0
IndexesY(:) = false;
Indexes3 = R3 > 0;
IndexesY(IndexesX) = Indexes3;
EPoints(IndexesY,:) = [LSemiAxis(1)*R1(Indexes3), LSemiAxis(2)*R2(Indexes3), LSemiAxis(3)*sqrt(R3(Indexes3))];
% -> -> Subcase 1a: R3 <= 0
IndexesY(IndexesX) = ~Indexes3;
EPoints(IndexesY,1:2) = VDist2EllipseEspecial(LSemiAxis(1:2),LPoints(IndexesY,1:2));

% -> Subcase 2: "outside ellipse bounding box" 
IndexesX(Indexes) = ~Indexes2;
EPoints(IndexesX,1:2) = VDist2EllipseEspecial(LSemiAxis(1:2),LPoints(IndexesX,1:2));

end



%% Distance to ellipse (first quadrant)
function EPoints = VDist2EllipseEspecial(LSemiAxis,LPoints)
% Point in the first quadrant to an ellipse with a>=b
% -------------------------------------------------------------------------
% Initialization
% -------------------------------------------------------------------------
IndexesY = LPoints(:,2) > 0;
IndexesX = LPoints(:,1) > 0;
EPoints = zeros(size(LPoints));
% -------------------------------------------------------------------------
% Case y>0, x>0
% -------------------------------------------------------------------------
Indexes = IndexesX & IndexesY;
if any(Indexes)
    EPoints(Indexes,:) = VNumericalDist2Ellipse(LSemiAxis,LPoints(Indexes,:));
end
% -------------------------------------------------------------------------
% Case y>0, x=0
% -------------------------------------------------------------------------
Indexes = ~IndexesX & IndexesY;
EPoints(Indexes,2) = LSemiAxis(2);
% -------------------------------------------------------------------------
% Case y=0, x>=0
% -------------------------------------------------------------------------
Indexes = ~IndexesY;

% Define subinterval
Num = LSemiAxis(1)* LPoints(Indexes,1);
Den = LSemiAxis(1)^2 - LSemiAxis(2)^2;

% -> Subcase 1: Num < Den
IndexesX(:) = false;
Indexes2 = Num < Den;
IndexesX(Indexes) = Indexes2;
EPoints(IndexesX,1) = Num(Indexes2) / Den;
EPoints(IndexesX,2) = LSemiAxis(2)*sqrt(1-EPoints(IndexesX,1).^2);
EPoints(IndexesX,1) = LSemiAxis(1)*EPoints(IndexesX,1);

% Subcase 2: Num >= Den
Indexes2 = ~Indexes2;
IndexesX(Indexes) = ~Indexes2;
EPoints(IndexesX,1) = LSemiAxis(1);

end



%% Numerical method for generic distance to ellipse (First quadrant)
function EPoints = VNumericalDist2Ellipse(LSemiAxis,LPoints)
% Application of Newton's method together with Bisection 
% a>b, x>0, y>0
if isempty(LPoints)
    EPoints = [];
    return;
end

MAX_ERROR = 0.00001;  % Newton error

a2 = LSemiAxis(1)^2;
b2 = LSemiAxis(2)^2;
ax_ = LSemiAxis(1)*LPoints(:,1);
by_ = LSemiAxis(2)*LPoints(:,2);
ax_2 = ax_.^2;
by_2 = by_.^2;

t0 = -b2 + by_;
t1 = -b2 + sqrt(ax_2 + by_2);
% t00 = t0;
% t11 = t1;
% tMin = -b2; % * ones(size(LPoints,1),1);
% tDeltaMin =  min(LSemiAxis.^2)*0.01;
tDeltaMin =  min(LSemiAxis)*0.01;
tDelta = inf;

tprev = (t0 + t1) / 2;


% t = t0;
% ApplyBisection = false(size(LPoints,1),1);
F = Inf(size(LPoints,1),1);
while max(F) > MAX_ERROR
% while min(tDelta) > tDeltaMin

    t = (t0 + t1) / 2;

    % Apply Newton
    F1d = (a2 + t);
    F2d = (b2 + t);
    F = (ax_./F1d).^2 + (by_./F2d).^2 - 1;
%     Fp = -2*( ax_2./(F1d.^3) +  by_2./(F2d.^3) );
%     tn = t - F./Fp;

    % Apply bisection
    Indexes = F>0;
    t0(Indexes) = t(Indexes);
    Indexes = F < 0;
    t1(Indexes) = t(Indexes);
%    tDelta = abs(t - tprev);
%    tprev = t;


%     Indexes = (tn <= tMin) | ApplyBisection;
%     Indexes2 = F>0;
%     t0(Indexes & Indexes2) = t(Indexes);
%     Indexes2 = ~Indexes2;
%     t1(Indexes & ~Indexes2) = t(Indexes);
%     tn(Indexes) = (t0(Indexes) + t1(Indexes))/2;
% %     if any(Indexes)
% %         Indexes2 = Indexes & (F>0);
% %         if any(Indexes2)
% %             t00(Indexes2) = t(Indexes2);
% %             t11(Indexes2) = t1(Indexes2);
% %         end;
% %         Indexes2 = Indexes & (F<=0);
% %         if any(Indexes2)
% %             t00(Indexes2) = t0(Indexes2);
% %             t11(Indexes2) = t(Indexes2);
% %         end;
% %         tn(Indexes) = (t00(Indexes) + t11(Indexes))/2;
% %         ApplyBisection(Indexes) = true;        
% %     end;



%     F1d = (a2 + t);
%     F2d = (b2 + t);
%     F = (ax_./F1d).^2 + (by_./F2d).^2 - 1;
%     F = abs(F);
%     t = tn;
end;

EPoints = LPoints.*[a2./(a2+t),b2./(b2+t)];

    
end


%% Numerical method for generic distance to ellipsoid (First octant)
function EPoints = VNumericalDist2Ellipsoid(LSemiAxis,LPoints)
% Application of Newton's method together with Bisection 
% a>b>c, x>0, y>0, z>0


MAX_ERROR = 0.0001;  % Newton error

a2 = LSemiAxis(1)^2;
b2 = LSemiAxis(2)^2;
c2 = LSemiAxis(3)^2;
ax_ = LSemiAxis(1)*LPoints(:,1);
by_ = LSemiAxis(2)*LPoints(:,2);
cz_ = LSemiAxis(3)*LPoints(:,3);
ax_2 = ax_.^2;
by_2 = by_.^2;
cz_2 = cz_.^2;

t0 = -c2 + cz_;
t1 = -c2 + sqrt(ax_2 + by_2+ cz_2);

F = Inf(size(LPoints,1),1);
Indexes = true(size(LPoints,1),1);

while any(Indexes) && max(abs(F(Indexes))) > MAX_ERROR

    t = (t0 + t1)/2;
    
    F1d = (a2 + t);
    F2d = (b2 + t);
    F3d = (c2 + t);
    F = (ax_./F1d).^2 + (by_./F2d).^2 + (cz_./F3d).^2 - 1;
    
    Indexes2 = F>0;
    t0(Indexes2) = t(Indexes2);
    
    Indexes2 = F<0;
    t1(Indexes2) = t(Indexes2);
    
    Indexes(F==0) = false;

end;

EPoints = LPoints.*[a2./(a2+t),b2./(b2+t),c2./(c2+t)];
end



% %% Numerical method for generic distance to ellipsoid (First octant)
% function EPoints = VNumericalDist2Ellipsoid(LSemiAxis,LPoints)
% % Application of Newton's method together with Bisection 
% % a>b>c, x>0, y>0, z>0
% 
% 
% MAX_ERROR = 0.00001;  % Newton error
% 
% a2 = LSemiAxis(1)^2;
% b2 = LSemiAxis(2)^2;
% c2 = LSemiAxis(3)^2;
% ax_ = LSemiAxis(1)*LPoints(:,1);
% by_ = LSemiAxis(2)*LPoints(:,2);
% cz_ = LSemiAxis(3)*LPoints(:,3);
% ax_2 = ax_.^2;
% by_2 = by_.^2;
% cz_2 = cz_.^2;
% 
% % t0 = -c2 + by_;
% t0 = -c2 + cz_;
% t1 = -c2 + sqrt(ax_2 + by_2+ cz_2);
% t00 = t0;
% t11 = t1;
% 
% tMin = -c2; % * ones(size(LPoints,1),1);
% % tDeltaMin =  min(LSemiAxis.^2)*0.01;
% tDeltaMin =  min(LSemiAxis)*0.01;
% tDelta = inf;
% 
% % t = t0;
% tn = (t0 + t1)/2;
% F = Inf(size(LPoints,1),1);
% ApplyBisection = false(size(LPoints,1),1);
% 
% while max(F) > MAX_ERROR
% % while min(tDelta) > tDeltaMin
%     
%     
%     % Apply Newton
%     F1d = (a2 + tn);
%     F2d = (b2 + tn);
%     F3d = (c2 + tn);
% %     F1d = (a2 + t);
% %     F2d = (b2 + t);
% %     F3d = (c2 + t);
%     F = (ax_./F1d).^2 + (by_./F2d).^2 + (cz_./F3d).^2 - 1;
% %     Fp = -2*( ax_2./(F1d.^3) +  by_2./(F2d.^3) + cz_2./(F3d.^3));
% %     tn = t - F./Fp;
%     
%     % Apply bisection
% %     Indexes = (tn <= tMin) | ApplyBisection;
% %     Indexes2 = F>0;
% %     t0(Indexes & Indexes2) = t(Indexes);
% %     Indexes2 = ~Indexes2;
% %     t1(Indexes & ~Indexes2) = t(Indexes);
% %     tn(Indexes) = (t0(Indexes) + t1(Indexes))/2;
% %     if any(Indexes)
% 
% %         Indexes2 = Indexes & (F>0);
%         Indexes2 = F>0;
%         
% %         if any(Indexes2)
%             t0(Indexes2) = tn(Indexes2);
% %             t11(Indexes2) = t1(Indexes2);
% %         end;
% %         Indexes2 = Indexes & (F<=0);
%         Indexes2 = F < 0;
%         
% %         if any(Indexes2)
% %             t00(Indexes2) = t0(Indexes2);
%             t1(Indexes2) = tn(Indexes2);
% %         end;
% 
% 
% %         tn(Indexes) = (t00(Indexes) + t11(Indexes))/2;
% %         tn = (t0 + t1)/2;
%         t = (t0 + t1)/2;
%         
% %         ApplyBisection(Indexes) = true;
% %     end;
% 
% %     F1d = (a2 + t);
% %     F2d = (b2 + t);
% %     F3d = (c2 + t);
% %     F = (ax_./F1d).^2 + (by_./F2d).^2 + (cz_./F3d).^2 - 1;
% 
% %     F = abs(F);
% %    tDelta = abs(t - tn);
%     tn = t;
% end;
% 
% EPoints = LPoints.*[a2./(a2+t),b2./(b2+t),c2./(c2+t)];
% end








