function [Serie]=bf_imread(Filename,varargin)
% Read 5D stacks from using BioFormat package
%
% Version:  0.02 (28.03.2013) Output Metadata List
%           0.01 (26.03.2013) Modified from bfopen.m from Bio-formats
%
% function [Serie]=bf_imread(Filename,varargin)
%
% INPUT ARGUMENTS:
% Filename:     Filename (with path). If not provided, a dialog will appear
% x:            (Optional) A scalar giving the x-origin of the tile.
%               Default: 1
% y:            (Optional) A scalar giving the y-origin of the tile.
%               Default: 1
% w:            (Optional) A scalar giving the width of the tile. 
%               Set to the width of the plane by default.
% h:            (Optional) A scalar giving the height of the tile.
%               Set to the height of the plane by default.
%
% OUTPUT ARGUMENTS:
% Serie{}:      Cell struct with the following fields:
%   .Stack          Stack in XYZCT order;
%   .Width          X size
%   .Height         Y size
%   .Planes         Z size
%   .Channels       C size    
%   .Frames         T size
%   .NumImages      Number of XY images 
%   .Map            Associated color map (assumed to be the same for
%                   the whole stack
%   .Metadata       String with the Metadata;
%   .MetadataList   Cell list with the Metadata information (',' are
%                   considered line separation)
%   .OMEMetadata    String with the OME Metadata;


%imports images using the BioFormats package
%you can load multiple z and t slices at once, e.g. zplanes=[1 2 5] loads
%first,second and fifth z-slice in a 3D-Stack 
%
%if loading multiple z slices and tframes, everything is returned in one 3D
%Stack with order ZT. Only one channel can be imported at once
%
%use imreadBFmeta() to get corresponding metadata of the image file
%
% To use the function, you have to download loci_tools.jar 
% from http://www.loci.wisc.edu/bio-formats/downloads
% make sure to have copied the file loci_tools.jar, in the folder where the
% function is placed (or to your work folder)
%
%
%
% For static loading, you can add the library to MATLAB's class path:
%     1. Type "edit classpath.txt" at the MATLAB prompt.
%     2. Go to the end of the file, and add the path to your JAR file
%        (e.g., C:/Program Files/MATLAB/work/loci_tools.jar).
%     3. Save the file and restart MATLAB.
%
% modified from imopen.m
% G. Hernn Sendra 2013




%% Load the Bio-Formats library into the MATLAB environment


% Toggle the autoloadBioFormats flag to control automatic loading
% of the Bio-Formats library using the javaaddpath command.
%
% For static loading, you can add the library to MATLAB's class path:
%     1. Type "edit classpath.txt" at the MATLAB prompt.
%     2. Go to the end of the file, and add the path to your JAR file
%        (e.g., C:/Program Files/MATLAB/work/loci_tools.jar).
%     3. Save the file and restart MATLAB.
%
% There are advantages to using the static approach over javaaddpath:
%     1. If you use bfopen within a loop, it saves on overhead
%        to avoid calling the javaaddpath command repeatedly.
%     2. Calling 'javaaddpath' may erase certain global parameters.
autoloadBioFormats = true;

% load the Bio-Formats library into the MATLAB environment
status = bfCheckJavaPath(autoloadBioFormats);
assert(status, ['Missing Bio-Formats library. Either add loci_tools.jar '...
    'to the static Java path or add it to the Matlab path.']);

%% Check input file

% Prompt for a file if not input
if nargin == 0 || exist(Filename, 'file') == 0
  [file, path] = uigetfile(bfGetFileExtensions, 'Choose a file to open');
  Filename = [path file];
  if isequal(path, 0) || isequal(file, 0), 
      return; 
  end
end

%% Initialize logging
loci.common.DebugTools.enableLogging('INFO');
    

%% Get the channel filler

% Toggle the stitchFiles flag to control grouping of similarly
% named files into a single dataset based on file numbering.
stitchFiles = 0;

r = bfGetReader(Filename, stitchFiles);


%% Test plane size
if nargin >=4
    planeSize = loci.formats.FormatTools.getPlaneSize(r, varargin{3}, varargin{4});
else
    planeSize = loci.formats.FormatTools.getPlaneSize(r);
end

if planeSize/(1024)^3 >= 2,
    error(['Image plane too large. Only 2GB of data can be extracted '...
        'at one time. You can workaround the problem by opening '...
        'the plane in tiles.']);
end

%% Read series
numSeries = r.getSeriesCount();
% result = cell(numSeries, 2);
Serie = cell(numSeries,1);
for s = 1:numSeries
    fprintf('Reading series #%d', s);
    r.setSeries(s - 1);
    pixelType = r.getPixelType();
    bpp = loci.formats.FormatTools.getBytesPerPixel(pixelType);
    bppMax = power(2, bpp * 8);
    numImages = r.getImageCount();
%     imageList = cell(numImages, 2);
%     colorMaps = cell(numImages);
    
    % Stack size
    arr = bfGetPlane(r, 1, varargin{:});
    sizeHeight = size(arr,1);    
    sizeWidth = size(arr,2);
    sizeZ = r.getSizeZ();
    sizeC = r.getSizeC();
    sizeT = r.getSizeT();

    Stack = uint8(zeros(sizeHeight,sizeWidth,sizeZ,sizeC,sizeT));
    switch class(arr)
        case 'uint8'
            Stack = uint8(Stack);
        case 'uint16'
            Stack = uint16(Stack);
        case 'uint32'
            Stack = uint32(Stack);
        case 'uint64'
            Stack = uint64(Stack);
    end

    
    
    for i = 1:numImages
        if mod(i, 72) == 1
            fprintf('\n    ');
        end
        fprintf('.');
        arr = bfGetPlane(r, i, varargin{:});

        % build an informative title for our figure
%         label = id;
%         if numSeries > 1
%             qs = int2str(s);
%             label = [label, '; series ', qs, '/', int2str(numSeries)];
%         end
        Index = [1,1,1];
        if numImages > 1
%             qi = int2str(i);
%             label = [label, '; plane ', qi, '/', int2str(numImages)];
%             if r.isOrderCertain()
%                 lz = 'Z';
%                 lc = 'C';
%                 lt = 'T';
%             else
%                 lz = 'Z?';
%                 lc = 'C?';
%                 lt = 'T?';
%             end
            zct = r.getZCTCoords(i - 1);
            if sizeZ > 1
                Index(1) = zct(1)+1;
%                 qz = int2str(zct(1) + 1);
%                 label = [label, '; ', lz, '=', qz, '/', int2str(sizeZ)];
            end
            if sizeC > 1
                Index(2) = zct(2)+1;                
%                 qc = int2str(zct(2) + 1);
%                 label = [label, '; ', lc, '=', qc, '/', int2str(sizeC)];
            end
            if sizeT > 1
                Index(3) = zct(3)+1;                
%                 qt = int2str(zct(3) + 1);
%                 label = [label, '; ', lt, '=', qt, '/', int2str(sizeT)];
            end
        end

        % save image plane and label into the list

        Stack(:,:,Index(1),Index(2),Index(3)) = arr;
%         imageList{i, 1} = arr;
%         imageList{i, 2} = label;
    end


    % save images and metadata into our master series list
    Serie{s}.Stack = Stack;
    Serie{s}.X = sizeWidth;
    Serie{s}.Y = sizeHeight;
    Serie{s}.Planes = sizeZ;
    Serie{s}.Channels = sizeC;    
    Serie{s}.Frames = sizeT;
    Serie{s}.NumImages = numImages;

    % retrieve color map data
    if bpp == 1
        Serie{s}.Map = r.get8BitLookupTable()';
    else
        Serie{s}.Map = r.get16BitLookupTable()';
    end
    
    warning off
    if ~isempty(Serie{s}.Map)
        newMap = single(Serie{s}.Map);
        newMap(newMap < 0) = newMap(newMap < 0) + bppMax;
        Serie{s}.Map = newMap / (bppMax - 1);
    end
    warning on
   

    % extract metadata table for this series
    Serie{s}.Metadata = r.getMetadata();
    Serie{s}.OMEMetadata = r.getMetadataStore();
    
    
%     result{s, 1} = imageList;
%     result{s, 2} = metadataList;
%     result{s, 3} = colorMaps;
%     result{s, 4} = r.getMetadataStore();
    fprintf('\n');
end
r.close();



